@@ -88,6 +88,7 @@
 #define OPT_FLAGS_C_STATES	 STRESS_BIT_ULL(57)	/* --c-states */
 #define OPT_FLAGS_STRESSOR_TIME	 STRESS_BIT_ULL(58)	/* --stressor-time */
 #define OPT_FLAGS_TASKSET_RANDOM STRESS_BIT_ULL(59)	/* --taskset-random */
+#define OPT_FLAGS_BUILDINFO	 STRESS_BIT_ULL(60)	/* --buildinfo */
 
 #define OPT_FLAGS_MINMAX_MASK		\
 	(OPT_FLAGS_MINIMIZE | OPT_FLAGS_MAXIMIZE)
@@ -234,6 +235,8 @@ typedef enum {
 	OPT_bubblesort_ops,
 	OPT_bubblesort_size,
 
+	OPT_buildinfo,
+
 	OPT_c_states,
 
 	OPT_class,
@@ -384,6 +387,9 @@ typedef enum {
 	OPT_dynlib,
 	OPT_dynlib_ops,
 
+	OPT_easy_opcode,
+	OPT_easy_opcode_ops,
+
 	OPT_eigen,
 	OPT_eigen_ops,
 	OPT_eigen_method,
@@ -473,6 +479,9 @@ typedef enum {
 	OPT_filename_ops,
 	OPT_filename_opts,
 
+	OPT_filerace,
+	OPT_filerace_ops,
+
 	OPT_flipflop,
 	OPT_flipflop_bits,
 	OPT_flipflop_ops,
@@ -620,6 +629,7 @@ typedef enum {
 	OPT_insertionsort_size,
 
 	OPT_intmath,
+	OPT_intmath_fast,
 	OPT_intmath_method,
 	OPT_intmath_ops,
 
@@ -823,6 +833,7 @@ typedef enum {
 	OPT_memrate,
 	OPT_memrate_bytes,
 	OPT_memrate_flush,
+	OPT_memrate_method,
 	OPT_memrate_ops,
 	OPT_memrate_rd_mbs,
 	OPT_memrate_wr_mbs,
@@ -915,6 +926,11 @@ typedef enum {
 	OPT_mmapmany_numa,
 	OPT_mmapmany_ops,
 
+	OPT_mmaptorture,
+	OPT_mmaptorture_bytes,
+	OPT_mmaptorture_msync,
+	OPT_mmaptorture_ops,
+
 	OPT_module,
 	OPT_module_name,
 	OPT_module_no_modver,
@@ -1040,6 +1056,7 @@ typedef enum {
 	OPT_pci,
 	OPT_pci_dev,
 	OPT_pci_ops,
+	OPT_pci_ops_rate,
 
 	OPT_perf_stats,
 
@@ -1055,6 +1072,10 @@ typedef enum {
 	OPT_physpage_ops,
 	OPT_physpage_mtrr,
 
+	OPT_physmmap,
+	OPT_physmmap_ops,
+	OPT_physmmap_read,
+
 	OPT_pidfd,
 	OPT_pidfd_ops,
 
@@ -1607,6 +1628,9 @@ typedef enum {
 	OPT_udp_flood_domain,
 	OPT_udp_flood_if,
 
+	OPT_umask,
+	OPT_umask_ops,
+
 	OPT_umount,
 	OPT_umount_ops,
 
@@ -1699,6 +1723,7 @@ typedef enum {
 	OPT_vma_ops,
 
 	OPT_vmstat,
+	OPT_vmstat_units,
 
 	OPT_vnni,
 	OPT_vnni_intrinsic,
