@@ -49,6 +49,7 @@ static const stress_opt_t opts[] = {
     defined(HAVE_LINUX_SOCKET_H) &&	\
     defined(AF_ALG)
 
+static volatile bool do_jmp = true;
 static sigjmp_buf jmpbuf;
 
 #if !defined(SOL_ALG)
@@ -136,8 +137,10 @@ static void MLOCKED_TEXT stress_af_alg_alarm_handler(int signum)
 	 * If we've not stopped after 5 seconds then an af-alg
 	 * got stuck, so force  jmp to terminate path
 	 */
-	if (UNLIKELY(count++ > 5))
+	if (UNLIKELY(do_jmp && count++ > 5)) {
+		do_jmp = false;
 		siglongjmp(jmpbuf, 1);
+	}
 }
 
 /*
@@ -293,7 +296,7 @@ retry_bind:
 					stress_af_alg_ignore(args, info, "send()");
 					break;
 				}
-				pr_fail("%s: %s (%s): send failed: errno=%d (%s)\n",
+				pr_fail("%s: %s (%s): send failed, errno=%d (%s)\n",
 						args->name, info->name, info->type,
 						errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -309,7 +312,7 @@ retry_bind:
 					break;
 				if (errno == EOPNOTSUPP)
 					goto err_abort;
-				pr_fail("%s: %s (%s): recv failed: errno=%d (%s)\n",
+				pr_fail("%s: %s (%s): recv failed, errno=%d (%s)\n",
 					args->name, info->name, info->type,
 					errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -324,7 +327,7 @@ retry_bind:
 			info->metrics.count += 1.0;
 		}
 		stress_bogo_inc(args);
-		if (args->max_ops && (stress_bogo_get(args) >= args->max_ops)) {
+		if (args->bogo.max_ops && (stress_bogo_get(args) >= args->bogo.max_ops)) {
 			rc = EXIT_SUCCESS;
 			goto err_close;
 		}
@@ -541,7 +544,7 @@ retry_bind:
 				stress_af_alg_ignore(args, info, "sendmsg()");
 				break;
 			}
-			pr_fail("%s: %s (%s): sendmsg failed: errno=%d (%s)\n",
+			pr_fail("%s: %s (%s): sendmsg failed, errno=%d (%s)\n",
 				args->name, info->name, info->type,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
@@ -554,7 +557,7 @@ retry_bind:
 					break;
 				if (errno == EOPNOTSUPP)
 					goto err_abort;
-				pr_fail("%s: %s (%s): read failed: errno=%d (%s)\n",
+				pr_fail("%s: %s (%s): read failed, errno=%d (%s)\n",
 					args->name, info->name, info->type,
 					errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -605,14 +608,14 @@ retry_bind:
 				stress_af_alg_ignore(args, info, "sendmsg()");
 				break;
 			}
-			pr_fail("%s: %s (%s): sendmsg failed: errno=%d (%s)\n",
+			pr_fail("%s: %s (%s): sendmsg failed, errno=%d (%s)\n",
 				args->name, info->name, info->type,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
 			goto err_close;
 		}
 		if (UNLIKELY(read(fd, output, DATA_LEN) != DATA_LEN)) {
-			pr_fail("%s: %s (%s): read failed: errno=%d (%s)\n",
+			pr_fail("%s: %s (%s): read failed, errno=%d (%s)\n",
 				args->name, info->name, info->type,
 				errno, strerror(errno));
 			rc = EXIT_FAILURE;
@@ -740,7 +743,7 @@ retry_bind:
 			info->metrics.count += 1.0;
 		}
 		stress_bogo_inc(args);
-		if (args->max_ops && (stress_bogo_get(args) >= args->max_ops)) {
+		if (args->bogo.max_ops && (stress_bogo_get(args) >= args->bogo.max_ops)) {
 			rc = EXIT_SUCCESS;
 			goto err_close;
 		}
@@ -977,6 +980,7 @@ static int stress_af_alg(stress_args_t *args)
 
 	rc = EXIT_SUCCESS;
 deinit:
+	do_jmp = false;
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	for (idx = 0, info = crypto_info_list; info; info = info->next) {
