@@ -117,21 +117,23 @@ static int stress_bigheap_child(stress_args_t *args, void *context)
 {
 	uint64_t bigheap_growth = DEFAULT_BIGHEAP_GROWTH;
 	size_t bigheap_bytes = DEFAULT_BIGHEAP_BYTES;
-	NOCLOBBER void *ptr = NULL, *last_ptr = NULL;
+	NOCLOBBER void *ptr = NULL;
+	NOCLOBBER const void *last_ptr = NULL;
 	NOCLOBBER uint8_t *last_ptr_end = NULL;
-	NOCLOBBER size_t size = 0;
+	NOCLOBBER size_t size = 0, stride;
 	NOCLOBBER double duration = 0.0, count = 0.0;
 	NOCLOBBER bool segv_reported = false;
 	const size_t page_size = args->page_size;
-	const size_t stride = page_size;
 	double rate;
 	const bool verify = !!(g_opt_flags & OPT_FLAGS_VERIFY);
 	const bool oom_avoid = !!(g_opt_flags & OPT_FLAGS_OOM_AVOID);
+	const bool aggressive = !!(g_opt_flags & OPT_FLAGS_AGGRESSIVE);
 	bool bigheap_mlock = false;
 	struct sigaction action;
 	int ret;
 	NOCLOBBER int rc = EXIT_SUCCESS;
 
+	stride = (g_opt_flags & OPT_FLAGS_AGGRESSIVE) ? sizeof(uintptr_t) : page_size;
 	fault_addr = NULL;
 	signo = -1;
 	sigcode = -1;
@@ -140,7 +142,12 @@ static int stress_bigheap_child(stress_args_t *args, void *context)
 	(void)context;
 
 	(void)stress_get_setting("bigheap-mlock", &bigheap_mlock);
-	(void)stress_get_setting("bigheap-bytes", &bigheap_bytes);
+	if (!stress_get_setting("bigheap-bytes", &bigheap_bytes)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			bigheap_bytes = MAX_BIGHEAP_BYTES;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			bigheap_bytes = MIN_BIGHEAP_BYTES;
+	}
 	if (!stress_get_setting("bigheap-growth", &bigheap_growth)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			bigheap_growth = MAX_BIGHEAP_GROWTH;
@@ -225,9 +232,22 @@ static int stress_bigheap_child(stress_args_t *args, void *context)
 		if (old_ptr) {
 			phase = STRESS_BIGHEAP_REALLOC;
 			ptr = realloc(old_ptr, size);
+			if (g_opt_flags & OPT_FLAGS_AGGRESSIVE) {
+				if (LIKELY(ptr != NULL)) {
+					old_ptr = ptr;
+					size += 64;
+					ptr = realloc(old_ptr, size);
+					if (LIKELY(ptr != NULL))
+						stress_bogo_inc(args);
+				}
+			}
 		} else {
 			phase = STRESS_BIGHEAP_MALLOC;
-			ptr = malloc(size);
+			if (UNLIKELY(aggressive)) {
+				ptr = calloc(1, size);
+			} else {
+				ptr = malloc(size);
+			}
 		}
 		if (UNLIKELY(ptr == NULL)) {
 			phase = STRESS_BIGHEAP_OUT_OF_MEMORY;
@@ -256,7 +276,7 @@ static int stress_bigheap_child(stress_args_t *args, void *context)
 				*uintptr = (uintptr_t)uintptr;
 			}
 			while (uintptr < uintptr_end) {
-				if (!stress_continue(args))
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
 				*uintptr = (uintptr_t)uintptr;
 				uintptr += stride / sizeof(uintptr_t);
