@@ -152,7 +152,6 @@ static int OPTIMIZE3 stress_brk_child(stress_args_t *args, void *context)
 		if (reset_brk || (stress_brk_abs(ptr, start_ptr) >= brk_context->brk_bytes)) {
 			intptr_t diff;
 
-			ptr = start_ptr;
 			VOID_RET(int, shim_brk(start_ptr));
 			VOID_RET(void *, shim_sbrk(0));
 
@@ -323,9 +322,9 @@ static int stress_brk(stress_args_t *args)
 	(void)stress_get_setting("brk-notouch", &brk_context->brk_notouch);
 
 #if !defined(MCL_FUTURE)
-	if ((args->instance == 0) && brk_context.brk_mlock) {
+	if ((args->instance == 0) && brk_context->brk_mlock) {
 		pr_inf("%s: --brk-mlock option was enabled but support for "
-			"mlock(MCL_FUTURE) is not available\n",
+			"mlockall(MCL_FUTURE) is not available\n",
 			args->name);
 	}
 #endif
