@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+
+#include <ctype.h>
 
 #if defined(HAVE_SYS_CAPABILITY_H)
 #include <sys/capability.h>
@@ -33,7 +36,7 @@ static const stress_help_t help[] = {
     defined(_LINUX_CAPABILITY_U32S_3)
 
 static int stress_capgetset_pid(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const pid_t pid,
 	const bool do_set,
 	const bool exists)
@@ -42,8 +45,8 @@ static int stress_capgetset_pid(
 	struct __user_cap_header_struct uch;
 	struct __user_cap_data_struct ucd[_LINUX_CAPABILITY_U32S_3];
 
-	(void)memset(&uch, 0, sizeof uch);
-	(void)memset(ucd, 0, sizeof ucd);
+	(void)shim_memset(&uch, 0, sizeof uch);
+	(void)shim_memset(ucd, 0, sizeof ucd);
 
 #if defined(_LINUX_CAPABILITY_VERSION_3)
 	uch.version = _LINUX_CAPABILITY_VERSION_3;
@@ -53,8 +56,9 @@ static int stress_capgetset_pid(
 	if (ret < 0) {
 		if (((errno == ESRCH) && exists) ||
 		    (errno != ESRCH)) {
-			pr_fail("%s: capget on pid %" PRIdMAX " failed: errno=%d (%s)\n",
+			pr_fail("%s: capget on PID %" PRIdMAX " failed, errno=%d (%s)\n",
 				args->name, (intmax_t)pid, errno, strerror(errno));
+			return EXIT_FAILURE;
 		}
 	}
 
@@ -63,8 +67,9 @@ static int stress_capgetset_pid(
 		if (ret < 0) {
 			if (((errno == ESRCH) && exists) ||
 			    (errno != ESRCH)) {
-				pr_fail("%s: capget on pid %" PRIdMAX " failed: errno=%d (%s)\n",
+				pr_fail("%s: capget on PID %" PRIdMAX " failed, errno=%d (%s)\n",
 					args->name, (intmax_t)pid, errno, strerror(errno));
+				return EXIT_FAILURE;
 			}
 		}
 
@@ -133,7 +138,7 @@ static int stress_capgetset_pid(
 	uch.pid = pid;
 	ret = capget(&uch, ucd);
 
-	inc_counter(args);
+	stress_bogo_inc(args);
 
 	return ret;
 }
@@ -142,56 +147,60 @@ static int stress_capgetset_pid(
  *  stress_cap
  *	stress capabilities (trivial)
  */
-static int stress_cap(const stress_args_t *args)
+static int stress_cap(stress_args_t *args)
 {
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
 		DIR *dir;
 
 		stress_capgetset_pid(args, 1, false, true);
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 		stress_capgetset_pid(args, args->pid, true, true);
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 		stress_capgetset_pid(args, getppid(), false, false);
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		dir = opendir("/proc");
 		if (dir) {
-			struct dirent *d;
+			const struct dirent *d;
 
 			while ((d = readdir(dir)) != NULL) {
 				intmax_t p;
 
-				if (!isdigit(d->d_name[0]))
+				if (!isdigit((unsigned char)d->d_name[0]))
 					continue;
 				if (sscanf(d->d_name, "%" SCNdMAX, &p) != 1)
 					continue;
 				stress_capgetset_pid(args, (pid_t)p, false, false);
-				if (!keep_stressing(args))
+				if (UNLIKELY(!stress_continue(args)))
 					break;
 			}
 			(void)closedir(dir);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_cap_info = {
+const stressor_info_t stress_cap_info = {
 	.stressor = stress_cap,
 	.class = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_cap_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_cap_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
-	.help = help
+	.help = help,
+	.verify = VERIFY_ALWAYS,
+	.unimplemented_reason = "built without sys/capability.h"
 };
 #endif
