@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,13 +18,25 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-killpid.h"
+#include "core-lock.h"
+
+#include <math.h>
+#include <sched.h>
+#include <time.h>
 
 #define DEFAULT_DELAY_NS	(100000)
-#define MAX_SAMPLES		(10000000)
+#define MAX_SAMPLES		(100000000)
 #define DEFAULT_SAMPLES		(10000)
 #define MAX_BUCKETS		(250)
 
+typedef struct {
+	void *lock;			/* lock protecting count */
+	uint32_t count;			/* count of error messages emitted */
+} stress_cyclic_state_t;
+
 typedef struct {
 	const int	policy;		/* scheduler policy */
 	const char	*name;		/* name of scheduler policy */
@@ -38,6 +50,7 @@ typedef struct {
 	size_t		latencies_size;	/* size of latencies allocation */
 	size_t		cyclic_samples;	/* number of latency samples */
 	size_t		index;		/* index into latencies */
+	size_t		index_reqd;	/* theoretic size of index required for the run */
 	int32_t		min_prio;	/* min priority allowed */
 	int32_t		max_prio;	/* max priority allowed */
 	double		ns;		/* total nanosecond latency */
@@ -46,7 +59,7 @@ typedef struct {
 	double		std_dev;	/* standard deviation */
 } stress_rt_stats_t;
 
-typedef int (*stress_cyclic_func)(const stress_args_t *args, stress_rt_stats_t *rt_stats, uint64_t cyclic_sleep);
+typedef int (*stress_cyclic_func)(stress_args_t *args, stress_rt_stats_t *rt_stats, uint64_t cyclic_sleep);
 
 typedef struct {
 	const char 		 *name;
@@ -65,7 +78,9 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,			NULL }
 };
 
-static const stress_policy_t policies[] = {
+static stress_cyclic_state_t *stress_cyclic_state = MAP_FAILED;
+
+static const stress_policy_t cyclic_policies[] = {
 #if defined(SCHED_DEADLINE)
 	{ SCHED_DEADLINE, "SCHED_DEADLINE",  "deadline" },
 #endif
@@ -77,61 +92,30 @@ static const stress_policy_t policies[] = {
 #endif
 };
 
-static const size_t num_policies = SIZEOF_ARRAY(policies);
+#define NUM_CYCLIC_POLICIES	(SIZEOF_ARRAY(cyclic_policies))
 
-static int stress_set_cyclic_sleep(const char *opt)
+static void stress_cyclic_init(const uint32_t instances)
 {
-	uint64_t cyclic_sleep;
+	(void)instances;
 
-	cyclic_sleep = stress_get_uint64(opt);
-	stress_check_range("cyclic-sleep", cyclic_sleep,
-		1, STRESS_NANOSECOND);
-	return stress_set_setting("cyclic-sleep", TYPE_ID_UINT64, &cyclic_sleep);
+	stress_cyclic_state = (stress_cyclic_state_t *)
+		stress_mmap_populate(NULL, sizeof(*stress_cyclic_state),
+				PROT_READ | PROT_WRITE,
+				MAP_ANONYMOUS | MAP_SHARED, -1, 0);
+	if (stress_cyclic_state == MAP_FAILED)
+		return;
+
+	stress_set_vma_anon_name(stress_cyclic_state, sizeof(*stress_cyclic_state), "cyclic-state");
+	stress_cyclic_state->lock = stress_lock_create("cyclic-state");
 }
 
-static int stress_set_cyclic_policy(const char *opt)
+static void stress_cyclic_deinit(void)
 {
-	size_t policy;
-
-	for (policy = 0; policy < num_policies; policy++) {
-		if (!strcmp(opt, policies[policy].opt_name)) {
-			stress_set_setting("cyclic-policy", TYPE_ID_SIZE_T, &policy);
-			return 0;
-		}
+	if (stress_cyclic_state != MAP_FAILED) {
+		if (stress_cyclic_state->lock)
+			stress_lock_destroy(stress_cyclic_state->lock);
+		(void)munmap((void *)stress_cyclic_state, sizeof(*stress_cyclic_state));
 	}
-	(void)fprintf(stderr, "invalid cyclic-policy '%s', policies allowed are:", opt);
-	for (policy = 0; policy < num_policies; policy++) {
-		(void)fprintf(stderr, " %s", policies[policy].opt_name);
-	}
-	(void)fprintf(stderr, "\n");
-	return -1;
-}
-
-static int stress_set_cyclic_prio(const char *opt)
-{
-	int32_t cyclic_prio;
-
-	cyclic_prio = stress_get_int32(opt);
-	stress_check_range("cyclic-prio", (uint64_t)cyclic_prio, 1, 100);
-	return stress_set_setting("cyclic-prio", TYPE_ID_INT32, &cyclic_prio);
-}
-
-static int stress_set_cyclic_dist(const char *opt)
-{
-	uint64_t cyclic_dist;
-
-	cyclic_dist = stress_get_uint64(opt);
-	stress_check_range("cyclic-dist", cyclic_dist, 1, 10000000);
-	return stress_set_setting("cyclic-dist", TYPE_ID_UINT64, &cyclic_dist);
-}
-
-static int stress_set_cyclic_samples(const char *opt)
-{
-	size_t cyclic_samples;
-
-	cyclic_samples = (size_t)stress_get_uint64(opt);
-	stress_check_range("cyclic-samples", cyclic_samples, 1, MAX_SAMPLES);
-	return stress_set_setting("cyclic-samples", TYPE_ID_SIZE_T, &cyclic_samples);
 }
 
 #if (defined(HAVE_CLOCK_GETTIME) && defined(HAVE_CLOCK_NANOSLEEP)) ||	\
@@ -152,6 +136,7 @@ static void stress_cyclic_stats(
 
 	if (rt_stats->index < rt_stats->cyclic_samples)
 		rt_stats->latencies[rt_stats->index++] = delta_ns;
+	rt_stats->index_reqd++;
 
 	rt_stats->ns += (double)delta_ns;
 }
@@ -166,7 +151,7 @@ static void stress_cyclic_stats(
  *	measure latencies with clock_nanosleep
  */
 static int stress_cyclic_clock_nanosleep(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_rt_stats_t *rt_stats,
 	const uint64_t cyclic_sleep)
 {
@@ -180,7 +165,7 @@ static int stress_cyclic_clock_nanosleep(
 	(void)clock_gettime(CLOCK_REALTIME, &t1);
 	ret = clock_nanosleep(CLOCK_REALTIME, 0, &t, &trem);
 	(void)clock_gettime(CLOCK_REALTIME, &t2);
-	if (ret == 0)
+	if (LIKELY(ret == 0))
 		stress_cyclic_stats(rt_stats, cyclic_sleep, &t1, &t2);
 	return 0;
 }
@@ -195,7 +180,7 @@ static int stress_cyclic_clock_nanosleep(
  *	measure latencies with posix nanosleep
  */
 static int stress_cyclic_posix_nanosleep(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_rt_stats_t *rt_stats,
 	const uint64_t cyclic_sleep)
 {
@@ -209,7 +194,7 @@ static int stress_cyclic_posix_nanosleep(
 	(void)clock_gettime(CLOCK_REALTIME, &t1);
 	ret = nanosleep(&t, &trem);
 	(void)clock_gettime(CLOCK_REALTIME, &t2);
-	if (ret == 0)
+	if (LIKELY(ret == 0))
 		stress_cyclic_stats(rt_stats, cyclic_sleep, &t1, &t2);
 	return 0;
 }
@@ -223,7 +208,7 @@ static int stress_cyclic_posix_nanosleep(
  *	measure latencies of heavy polling the clock
  */
 static int stress_cyclic_poll(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_rt_stats_t *rt_stats,
 	const uint64_t cyclic_sleep)
 {
@@ -252,6 +237,7 @@ static int stress_cyclic_poll(
 
 			if (rt_stats->index < rt_stats->cyclic_samples)
 				rt_stats->latencies[rt_stats->index++] = delta_ns;
+			rt_stats->index_reqd++;
 
 			rt_stats->ns += (double)delta_ns;
 			break;
@@ -270,7 +256,7 @@ static int stress_cyclic_poll(
  *	measure latencies with pselect sleep
  */
 static int stress_cyclic_pselect(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_rt_stats_t *rt_stats,
 	const uint64_t cyclic_sleep)
 {
@@ -284,7 +270,7 @@ static int stress_cyclic_pselect(
 	(void)clock_gettime(CLOCK_REALTIME, &t1);
 	ret = pselect(0, NULL, NULL,NULL, &t, NULL);
 	(void)clock_gettime(CLOCK_REALTIME, &t2);
-	if (ret == 0)
+	if (LIKELY(ret == 0))
 		stress_cyclic_stats(rt_stats, cyclic_sleep, &t1, &t2);
 	return 0;
 }
@@ -310,7 +296,7 @@ static void MLOCKED_TEXT stress_cyclic_itimer_handler(int sig)
  *	measure latencies with itimers
  */
 static int stress_cyclic_itimer(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_rt_stats_t *rt_stats,
 	const uint64_t cyclic_sleep)
 {
@@ -328,14 +314,14 @@ static int stress_cyclic_itimer(
 	if (stress_sighandler(args->name, SIGRTMIN, stress_cyclic_itimer_handler, &old_action) < 0)
 		return ret;
 
-	(void)memset(&sev, 0, sizeof(sev));
+	(void)shim_memset(&sev, 0, sizeof(sev));
 	sev.sigev_notify = SIGEV_SIGNAL;
 	sev.sigev_signo = SIGRTMIN;
 	sev.sigev_value.sival_ptr = &timerid;
 	if (timer_create(CLOCK_REALTIME, &sev, &timerid) < 0)
 		goto restore;
 
-	(void)memset(&itimer_time, 0, sizeof(itimer_time));
+	(void)shim_memset(&itimer_time, 0, sizeof(itimer_time));
 	(void)clock_gettime(CLOCK_REALTIME, &t1);
 	if (timer_settime(timerid, 0, &timer, NULL) < 0)
 		goto restore;
@@ -351,6 +337,7 @@ static int stress_cyclic_itimer(
 
 	if (rt_stats->index < rt_stats->cyclic_samples)
 		rt_stats->latencies[rt_stats->index++] = delta_ns;
+	rt_stats->index_reqd++;
 
 	rt_stats->ns += (double)delta_ns;
 
@@ -359,7 +346,7 @@ static int stress_cyclic_itimer(
 	ret = 0;
 tidy:
 	/* And cancel timer */
-	(void)memset(&timer, 0, sizeof(timer));
+	(void)shim_memset(&timer, 0, sizeof(timer));
 	(void)timer_settime(timerid, 0, &timer, NULL);
 restore:
 	stress_sigrestore(args->name, SIGRTMIN, &old_action);
@@ -375,7 +362,7 @@ restore:
  *	measure latencies with usleep
  */
 static int stress_cyclic_usleep(
-	const stress_args_t *args,
+	stress_args_t *args,
 	stress_rt_stats_t *rt_stats,
 	const uint64_t cyclic_sleep)
 {
@@ -388,7 +375,7 @@ static int stress_cyclic_usleep(
 	(void)clock_gettime(CLOCK_REALTIME, &t1);
 	ret = usleep(usecs);
 	(void)clock_gettime(CLOCK_REALTIME, &t2);
-	if (ret == 0)
+	if (LIKELY(ret == 0))
 		stress_cyclic_stats(rt_stats, cyclic_sleep, &t1, &t2);
 	return 0;
 }
@@ -406,7 +393,7 @@ static void NORETURN MLOCKED_TEXT stress_rlimit_handler(int signum)
 {
 	(void)signum;
 
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 	siglongjmp(jmp_env, 1);
 }
 
@@ -441,7 +428,7 @@ static void stress_rt_stats(stress_rt_stats_t *rt_stats)
 	rt_stats->latency_mode = 0;
 
 	for (i = 0; i < rt_stats->index; i++) {
-		int64_t ns = rt_stats->latencies[i];
+		const int64_t ns = rt_stats->latencies[i];
 
 		if (ns > rt_stats->max_ns)
 			rt_stats->max_ns = ns;
@@ -458,7 +445,7 @@ static void stress_rt_stats(stress_rt_stats_t *rt_stats)
 	current = rt_stats->latency_mode = rt_stats->latencies[0];
 
 	for (i = 0; i < rt_stats->index; i++) {
-		int64_t ns = rt_stats->latencies[i];
+		const int64_t ns = rt_stats->latencies[i];
 		double diff;
 
 		if (ns == current) {
@@ -513,33 +500,9 @@ static const stress_cyclic_method_info_t cyclic_methods[] = {
 #if defined(HAVE_CLOCK_GETTIME)
 	{ "usleep",	stress_cyclic_usleep },
 #endif
-
-	{ NULL,		NULL }
 };
 
-/*
- *  stress_set_cyclic_method()
- *	set the default cyclic method
- */
-static int stress_set_cyclic_method(const char *name)
-{
-	stress_cyclic_method_info_t const *info;
-
-	for (info = cyclic_methods; info->func; info++) {
-		if (!strcmp(info->name, name)) {
-			stress_set_setting("cyclic-method", TYPE_ID_UINTPTR_T, &info);
-			return 0;
-		}
-	}
-
-	(void)fprintf(stderr, "cyclic-method must be one of:");
-	for (info = cyclic_methods; info->func; info++) {
-		(void)fprintf(stderr, " %s", info->name);
-	}
-	(void)fprintf(stderr, "\n");
-
-	return -1;
-}
+#define NUM_CYCLIC_METHODS	(SIZEOF_ARRAY(cyclic_methods))
 
 /*
  *  stress_rt_dist()
@@ -562,12 +525,12 @@ static void stress_rt_dist(
 
 	dist = (int64_t *)calloc(dist_size, sizeof(*dist));
 	if (!dist) {
-		pr_inf("%s: cannot allocat distribution stats buffer, cannot log distribution\n", name);
+		pr_inf("%s: cannot allocate distribution stats buffer, cannot log distribution\n", name);
 		return;
 	}
 
 	for (i = 0; i < (ssize_t)rt_stats->index; i++) {
-		int64_t lat = rt_stats->latencies[i] / cyclic_dist;
+		const int64_t lat = rt_stats->latencies[i] / cyclic_dist;
 
 		if (lat < (int64_t)dist_size)
 			dist[lat]++;
@@ -620,10 +583,9 @@ static int stress_cyclic_supported(const char *name)
 	return 0;
 }
 
-static int stress_cyclic(const stress_args_t *args)
+static int stress_cyclic(stress_args_t *args)
 {
-	const stress_cyclic_method_info_t *cyclic_method = &cyclic_methods[0];
-	const uint32_t num_instances = args->num_instances;
+	const uint32_t instances = args->instances;
 	struct sigaction old_action_xcpu;
 	struct rlimit rlim;
 	pid_t pid;
@@ -632,18 +594,15 @@ static int stress_cyclic(const stress_args_t *args)
 	uint64_t cyclic_dist = 0;
 	int32_t cyclic_prio = INT32_MAX;
 	size_t cyclic_samples = DEFAULT_SAMPLES;
-	int policy;
+	NOCLOBBER int policy;
+	int rc = EXIT_SUCCESS;
 	size_t cyclic_policy = 0;
+	size_t cyclic_method = 0;
 	const double start = stress_time_now();
 	stress_rt_stats_t *rt_stats;
 	const size_t page_size = args->page_size;
 	const size_t size = (sizeof(*rt_stats) + page_size - 1) & (~(page_size - 1));
 	stress_cyclic_func func;
-#if defined(MAP_POPULATE)
-	const int mmap_flags = MAP_SHARED | MAP_ANONYMOUS | MAP_POPULATE;
-#else
-	const int mmap_flags = MAP_SHARED | MAP_ANONYMOUS;
-#endif
 
 	timeout  = g_opt_timeout;
 	(void)stress_get_setting("cyclic-dist", &cyclic_dist);
@@ -653,17 +612,39 @@ static int stress_cyclic(const stress_args_t *args)
 	(void)stress_get_setting("cyclic-samples", &cyclic_samples);
 	(void)stress_get_setting("cyclic-sleep", &cyclic_sleep);
 
-	func = cyclic_method->func;
-	policy = policies[cyclic_policy].policy;
-
-	if (!args->instance) {
-		if (num_policies == 0) {
+	if (NUM_CYCLIC_POLICIES == 0) {
+		if (!args->instance) {
 			pr_inf_skip("%s: no scheduling policies "
 				"available, skipping test\n",
 				args->name);
-			return EXIT_NOT_IMPLEMENTED;
 		}
+		return EXIT_NOT_IMPLEMENTED;
 	}
+	if ((ssize_t)cyclic_policy >= (ssize_t)NUM_CYCLIC_POLICIES) {
+		if (!args->instance) {
+			pr_err("%s: cyclic-policy %zu is out of range\n",
+				args->name, cyclic_policy);
+		}
+		return EXIT_FAILURE;
+	}
+	if (NUM_CYCLIC_METHODS == 0) {
+		if (!args->instance) {
+			pr_inf_skip("%s: no cyclic methods"
+				"available, skipping test\n",
+				args->name);
+		}
+		return EXIT_NOT_IMPLEMENTED;
+	}
+	if ((ssize_t)cyclic_method >= (ssize_t)NUM_CYCLIC_METHODS) {
+		if (!args->instance) {
+			pr_err("%s: cyclic-method %zu is out of range\n",
+				args->name, cyclic_method);
+		}
+		return EXIT_FAILURE;
+	}
+
+	func = cyclic_methods[cyclic_method].func;
+	policy = cyclic_policies[cyclic_policy].policy;
 
 	if (g_opt_timeout == TIMEOUT_NOT_SET) {
 		timeout = 60;
@@ -671,28 +652,33 @@ static int stress_cyclic(const stress_args_t *args)
 			"be %" PRIu64 " seconds\n", args->name, timeout);
 	}
 
-	if ((num_instances > 1) && (args->instance == 0)) {
+	if ((instances > 1) && (args->instance == 0)) {
 		pr_inf("%s: for best results, run just 1 instance of "
 			"this stressor\n", args->name);
 	}
 
-	rt_stats = (stress_rt_stats_t *)mmap(NULL, size,
-						PROT_READ | PROT_WRITE, mmap_flags, -1, 0);
+	rt_stats = (stress_rt_stats_t *)stress_mmap_populate(NULL, size,
+						PROT_READ | PROT_WRITE,
+						MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (rt_stats == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of shared statistics data failed: %d (%s)\n",
+		pr_inf_skip("%s: mmap of shared statistics data failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(rt_stats, size, "rt-statistics");
 	rt_stats->cyclic_samples = cyclic_samples;
 	rt_stats->latencies_size = cyclic_samples * sizeof(*rt_stats->latencies);
-	rt_stats->latencies = (int64_t *)mmap(NULL, rt_stats->latencies_size,
-						PROT_READ | PROT_WRITE, mmap_flags, -1, 0);
+	rt_stats->latencies = (int64_t *)stress_mmap_populate(NULL,
+						rt_stats->latencies_size,
+						PROT_READ | PROT_WRITE,
+						MAP_SHARED | MAP_ANONYMOUS, -1, 0);
 	if (rt_stats->latencies == MAP_FAILED) {
-		pr_inf_skip("%s: mmap of %zd samples failed: %d (%s)\n",
+		pr_inf_skip("%s: mmap of %zd samples failed, errno=%d (%s)\n",
 			args->name, cyclic_samples, errno, strerror(errno));
 		(void)munmap((void *)rt_stats, size);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(rt_stats->latencies, rt_stats->latencies_size, "latencies");
 	rt_stats->min_ns = INT64_MAX;
 	rt_stats->max_ns = INT64_MIN;
 	rt_stats->ns = 0.0;
@@ -715,44 +701,31 @@ static int stress_cyclic(const stress_args_t *args)
 	}
 
 	if (args->instance == 0)
-		pr_dbg("%s: using method '%s'\n", args->name, cyclic_method->name);
+		pr_dbg("%s: using method '%s'\n", args->name, cyclic_methods[cyclic_method].name);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
 		pr_inf("%s: cannot fork, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
+		(void)munmap((void *)rt_stats->latencies, rt_stats->latencies_size);
+		(void)munmap((void *)rt_stats, size);
 		return EXIT_NO_RESOURCE;
 	} else if (pid == 0) {
 #if defined(HAVE_SCHED_GET_PRIORITY_MIN) &&	\
     defined(HAVE_SCHED_GET_PRIORITY_MAX)
 		const pid_t mypid = getpid();
-#endif
-#if defined(HAVE_ATOMIC)
-		uint32_t count;
 #endif
 		int ret;
-		NOCLOBBER int rc = EXIT_FAILURE;
-
-#if defined(HAVE_ATOMIC)
-		__sync_fetch_and_add(&g_shared->softlockup_count, 1);
-
-		/*
-		 * Wait until all instances have reached this point
-		 */
-		do {
-			if ((stress_time_now() - start) > (double)timeout)
-				goto tidy_ok;
-			(void)usleep(50000);
-			__atomic_load(&g_shared->softlockup_count, &count, __ATOMIC_RELAXED);
-		} while (keep_stressing(args) && count < num_instances);
-#endif
+		NOCLOBBER int ncrc = EXIT_FAILURE;
 
 		/*
 		 * We run the stressor as a child so that
@@ -770,13 +743,13 @@ again:
 		(void)setrlimit(RLIMIT_RTTIME, &rlim);
 #endif
 
-		if (stress_sighandler(args->name, SIGXCPU, stress_rlimit_handler, &old_action_xcpu) < 0)
-			goto tidy;
-
 		ret = sigsetjmp(jmp_env, 1);
 		if (ret)
 			goto tidy_ok;
 
+		if (stress_sighandler(args->name, SIGXCPU, stress_rlimit_handler, &old_action_xcpu) < 0)
+			goto tidy;
+
 #if defined(HAVE_SCHED_GET_PRIORITY_MIN) &&	\
     defined(HAVE_SCHED_GET_PRIORITY_MAX)
 #if defined(SCHED_DEADLINE)
@@ -793,59 +766,70 @@ redo_policy:
 			 *  which users the older and smaller attr structure.
 			 */
 			if ((errno == E2BIG) &&
-			    (policies[cyclic_policy].policy == SCHED_DEADLINE)) {
+			    (cyclic_policies[cyclic_policy].policy == SCHED_DEADLINE)) {
 				cyclic_policy = 1;
-				policy = policies[cyclic_policy].policy;
+				if ((ssize_t)cyclic_policy >= (ssize_t)NUM_CYCLIC_POLICIES) {
+					pr_inf("%s: DEADLINE not supported by kernel, no other policies "
+						"available. skipping stressor\n", args->name);
+					ncrc = EXIT_NO_RESOURCE;
+					goto finish;
+				}
+				policy = cyclic_policies[cyclic_policy].policy;
 #if defined(HAVE_SCHED_GET_PRIORITY_MAX)
 				rt_stats->max_prio = sched_get_priority_max(policy);
 #else
 				rt_stats->max_prio = 0;
 #endif
 				pr_inf("%s: DEADLINE not supported by kernel, defaulting to %s\n",
-					args->name, policies[cyclic_policy].name);
+					args->name, cyclic_policies[cyclic_policy].name);
 				goto redo_policy;
 			}
 #endif
 			if (errno != EPERM) {
+				uint32_t count = 0;
 				const char *msg = (errno == EBUSY) ?
-					", (recommend setting --sched-runtime to less than 90000)" : "";
+					", (recommend setting --sched-runtime to less than 90000 or run one instance of cyclic stressor)" : "";
 
-				pr_fail("%s: sched_setscheduler "
-					"failed: errno=%d (%s) "
-					"for scheduler policy %s%s\n",
-					args->name, errno, strerror(errno),
-					policies[cyclic_policy].name,
-					msg);
+				if (stress_cyclic_state != MAP_FAILED) {
+					(void)stress_lock_acquire(stress_cyclic_state->lock);
+					count = stress_cyclic_state->count;
+					stress_cyclic_state->count++;
+					(void)stress_lock_release(stress_cyclic_state->lock);
+				}
+
+				if (count == 0)
+					pr_fail("%s: sched_setscheduler "
+						"failed, errno=%d (%s) "
+						"for scheduler policy %s%s\n",
+						args->name, errno, strerror(errno),
+						cyclic_policies[cyclic_policy].name,
+						msg);
 			}
 			goto tidy;
 		}
 #endif
 		do {
 			func(args, rt_stats, cyclic_sleep);
-			inc_counter(args);
+			stress_bogo_inc(args);
 
 			/* Ensure we NEVER spin forever */
-			if ((stress_time_now() - start) > (double)timeout)
+			if (UNLIKELY((stress_time_now() - start) > (double)timeout))
 				break;
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 
 tidy_ok:
-		rc = EXIT_SUCCESS;
+		ncrc = EXIT_SUCCESS;
 tidy:
 		(void)fflush(stdout);
-		_exit(rc);
+		(void)munmap((void *)rt_stats->latencies, rt_stats->latencies_size);
+		(void)munmap((void *)rt_stats, size);
+		_exit(ncrc);
 	} else {
-		int status;
-
 		VOID_RET(int, stress_set_sched(args->pid, policy, rt_stats->max_prio, true));
 
 		(void)pause();
-		(void)kill(pid, SIGKILL);
-#if defined(HAVE_ATOMIC)
-		__sync_fetch_and_sub(&g_shared->softlockup_count, 1);
-#endif
-
-		(void)shim_waitpid(pid, &status, 0);
+		stress_force_killed_bogo(args);
+		(void)stress_kill_pid_wait(pid, NULL);
 	}
 
 	stress_rt_stats(rt_stats);
@@ -866,9 +850,10 @@ tidy:
 				99.99,
 			};
 
+			pr_block_begin();
 			pr_inf("%s: sched %s: %" PRIu64 " ns delay, %zd samples\n",
 				args->name,
-				policies[cyclic_policy].name,
+				cyclic_policies[cyclic_policy].name,
 				cyclic_sleep,
 				rt_stats->index);
 			pr_inf( "%s:   mean: %.2f ns, mode: %" PRId64 " ns\n",
@@ -890,11 +875,15 @@ tidy:
 					rt_stats->latencies[j]);
 			}
 			stress_rt_dist(args->name, rt_stats, (int64_t)cyclic_dist);
-			pr_unlock();
+
+			if (rt_stats->index < rt_stats->index_reqd)
+				pr_inf("%s: Note: --cyclic-samples needed to be %zd to capture all the data for this run\n",
+					args->name, rt_stats->index_reqd);
+			pr_block_end();
 		} else {
 			pr_inf("%s: %10s: no latency information available\n",
 				args->name,
-				policies[cyclic_policy].name);
+				cyclic_policies[cyclic_policy].name);
 		}
 	}
 
@@ -904,23 +893,37 @@ finish:
 	(void)munmap((void *)rt_stats->latencies, rt_stats->latencies_size);
 	(void)munmap((void *)rt_stats, size);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_cyclic_dist,	stress_set_cyclic_dist },
-	{ OPT_cyclic_method,	stress_set_cyclic_method },
-	{ OPT_cyclic_policy,	stress_set_cyclic_policy },
-	{ OPT_cyclic_prio, 	stress_set_cyclic_prio },
-	{ OPT_cyclic_sleep,	stress_set_cyclic_sleep },
-	{ OPT_cyclic_samples,	stress_set_cyclic_samples },
-	{ 0,			NULL }
+static const char *stress_cyclic_methods(const size_t i)
+{
+	return (NUM_CYCLIC_METHODS == 0) ? NULL :
+		(((ssize_t)i < (ssize_t)NUM_CYCLIC_METHODS) ? cyclic_methods[i].name : NULL);
+}
+
+static const char *stress_cyclic_policies(const size_t i)
+{
+	return (NUM_CYCLIC_POLICIES == 0) ? NULL :
+		(((ssize_t)i < (ssize_t)NUM_CYCLIC_POLICIES) ? cyclic_policies[i].opt_name : NULL);
 };
 
-stressor_info_t stress_cyclic_info = {
+static const stress_opt_t opts[] = {
+	{ OPT_cyclic_dist,    "cyclic-dist",    TYPE_ID_UINT64, 1, 10000000, NULL },
+	{ OPT_cyclic_method,  "cyclic-method",  TYPE_ID_SIZE_T_METHOD, 0, 0, stress_cyclic_methods },
+	{ OPT_cyclic_policy,  "cyclic-policy",  TYPE_ID_SIZE_T_METHOD, 0, 0, stress_cyclic_policies },
+	{ OPT_cyclic_prio,    "cyclic-prio",    TYPE_ID_INT32, 1, 100, NULL },
+	{ OPT_cyclic_sleep,   "cyclic-sleep",   TYPE_ID_UINT64, 1, STRESS_NANOSECOND, NULL },
+	{ OPT_cyclic_samples, "cyclic-samples", TYPE_ID_SIZE_T, 1, MAX_SAMPLES, NULL },
+	END_OPT,
+};
+
+const stressor_info_t stress_cyclic_info = {
 	.stressor = stress_cyclic,
 	.supported = stress_cyclic_supported,
 	.class = CLASS_SCHEDULER | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.init = stress_cyclic_init,
+	.deinit = stress_cyclic_deinit,
 	.help = help
 };
