@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,8 @@
 #include "stress-ng.h"
 #include "core-arch.h"
 #include "core-cpu.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
 #include "core-pragma.h"
 
 static const stress_help_t help[] = {
@@ -33,33 +35,35 @@ static const stress_help_t help[] = {
 
 #define HASH_SYSCALL_SIZE	(1987)
 
-#define CALL_UNDEFINED		(0)
-#define CALL_BY_SYSCALL		(1)	/* syscall() call */
-#define CALL_BY_X86_SYSCALL	(2)	/* x86 syscall opcode */
-#define CALL_BY_X86_INT80	(3)	/* x86 int $80 */
-
 #if defined(__NR_syscalls)
 #define MAX_SYSCALL	__NR_syscalls
 #else
 #define MAX_SYSCALL	(2048)		/* Guess */
 #endif
 
-#if defined(__linux__) &&       \
-    !defined(__TINYC__) &&	\
-    !defined(__PCC__) &&	\
-   (defined(__x86_64__) || defined(__x86_64))
+#if defined(__linux__) &&       	\
+    !defined(HAVE_COMPILER_TCC) &&	\
+    !defined(HAVE_COMPILER_PCC) &&	\
+    defined(STRESS_ARCH_X86_64)
 #define STRESS_EXERCISE_X86_SYSCALL
 #endif
 
 #if defined(__linux__) &&       \
     defined(STRESS_ARCH_X86) &&	\
-    NEED_GNUC(9,3,0)
+    (NEED_GNUC(9, 3, 0) || NEED_CLANG(9, 0, 0))
 #define STRESS_EXERCISE_X86_0X80
 #endif
 
+#define CALL_UNDEFINED		(0)
+#define CALL_BY_SYSCALL		(1)	/* syscall() call */
+#define CALL_BY_X86_SYSCALL	(2)	/* x86 syscall opcode */
+#if defined(STRESS_EXERCISE_X86_0X80)
+#define CALL_BY_X86_INT80	(3)	/* x86 int $80 */
+#endif
+
 typedef struct hash_syscall {
 	struct hash_syscall *next;
-	long	number;
+	long int number;
 } stress_hash_syscall_t;
 
 static stress_hash_syscall_t *hash_syscall_table[HASH_SYSCALL_SIZE];
@@ -74,28 +78,28 @@ static bool stress_x86syscall_available;
  *  x86_64_syscall6()
  *      syscall 6 arg wrapper
  */
-static inline long x86_64_syscall6(
-	long number, long arg1, long arg2,
-	long arg3, long arg4, long arg5, long arg6)
+static inline long int x86_64_syscall6(
+	long int number, long int arg1, long int arg2,
+	long int arg3, long int arg4, long int arg5, long int arg6)
 {
-	long ret;
+	long int ret;
 
 	stress_call_type = CALL_BY_X86_SYSCALL;
 
 	{
-		long tmp_arg1 = arg1;
-		long tmp_arg2 = arg2;
-		long tmp_arg3 = arg3;
-		long tmp_arg4 = arg4;
-		long tmp_arg5 = arg5;
-		long tmp_arg6 = arg6;
+		long int tmp_arg1 = arg1;
+		long int tmp_arg2 = arg2;
+		long int tmp_arg3 = arg3;
+		long int tmp_arg4 = arg4;
+		long int tmp_arg5 = arg5;
+		long int tmp_arg6 = arg6;
 
-		register long asm_arg1 __asm__ ("rdi") = tmp_arg1;
-		register long asm_arg2 __asm__ ("rsi") = tmp_arg2;
-		register long asm_arg3 __asm__ ("rdx") = tmp_arg3;
-		register long asm_arg4 __asm__ ("r10") = tmp_arg4;
-		register long asm_arg5 __asm__ ("r8")  = tmp_arg5;
-		register long asm_arg6 __asm__ ("r9")  = tmp_arg6;
+		register long int asm_arg1 __asm__ ("rdi") = tmp_arg1;
+		register long int asm_arg2 __asm__ ("rsi") = tmp_arg2;
+		register long int asm_arg3 __asm__ ("rdx") = tmp_arg3;
+		register long int asm_arg4 __asm__ ("r10") = tmp_arg4;
+		register long int asm_arg5 __asm__ ("r8")  = tmp_arg5;
+		register long int asm_arg6 __asm__ ("r9")  = tmp_arg6;
 
 		__asm__ __volatile__("syscall\n\t"
 			: "=a" (ret)
@@ -113,9 +117,9 @@ static inline long x86_64_syscall6(
 
 #if defined(STRESS_EXERCISE_X86_0X80)
 static inline int x86_0x80_syscall6(
-	long number, long arg1, long arg2,
-	long arg3, long arg4, long arg5,
-	long arg6)
+	long int number, long int arg1, long int arg2,
+	long int arg3, long int arg4, int long arg5,
+	long int arg6)
 {
 	int ret;
 
@@ -152,9 +156,9 @@ static inline int x86_0x80_syscall6(
 
 STRESS_PRAGMA_PUSH
 STRESS_PRAGMA_WARN_OFF
-static inline long syscall7(long number, long arg1, long arg2,
-			    long arg3, long arg4, long arg5,
-			    long arg6, long arg7)
+static inline long int syscall7(long int number, long int arg1, long int arg2,
+			    long int arg3, long int arg4, long int arg5,
+			    long int arg6, long int arg7)
 {
 	stress_call_type = CALL_BY_SYSCALL;
 
@@ -170,7 +174,7 @@ static void exit_if_child(const pid_t pid)
 	}
 }
 
-static void itimer_set(const stress_args_t *args)
+static void itimer_set(stress_args_t *args)
 {
 	struct itimerval it;
 
@@ -189,16 +193,16 @@ static void itimer_set(const stress_args_t *args)
 }
 
 static void exercise_syscall(
-	const stress_args_t *args,
-	long number, long arg1, long arg2,
-	long arg3, long arg4, long arg5,
-	long arg6, long arg7)
+	stress_args_t *args,
+	long int number, long int arg1, long int arg2,
+	long int arg3, long int arg4, long int arg5,
+	long int arg6, long int arg7)
 {
 	int ret;
 	NOCLOBBER bool enosys = false;
 	const pid_t pid = getpid();
 
-	if (!keep_stressing(args))
+	if (UNLIKELY(!stress_continue(args)))
 		_exit(EXIT_SUCCESS);
 
 	itimer_set(args);
@@ -316,11 +320,12 @@ static const int syscall_ignore[] = {
 #endif
 };
 
-static inline bool HOT OPTIMIZE3 syscall_find(long number)
+static inline bool OPTIMIZE3 syscall_find(long int number)
 {
 	register stress_hash_syscall_t *h;
 	register int i;
-	register const long number16 = number & 0xffff;
+	register const long int number16 = number & 0xffff;
+	const unsigned long int idx = (unsigned long int)number;
 
 	/* Really make sure some syscalls are never called */
 	for (i = 0; i < (int)SIZEOF_ARRAY(syscall_ignore); i++) {
@@ -328,7 +333,7 @@ static inline bool HOT OPTIMIZE3 syscall_find(long number)
 			return true;
 	}
 
-	h = hash_syscall_table[number % HASH_SYSCALL_SIZE];
+	h = hash_syscall_table[idx % HASH_SYSCALL_SIZE];
 	while (h) {
 		if (h->number == number)
 			return true;
@@ -337,9 +342,10 @@ static inline bool HOT OPTIMIZE3 syscall_find(long number)
 	return false;
 }
 
-static inline void HOT OPTIMIZE3 syscall_add(const long number)
+static inline void OPTIMIZE3 syscall_add(const long int number)
 {
-	const long hash = number % HASH_SYSCALL_SIZE;
+	const unsigned long int idx = (unsigned long int)number;
+	const unsigned long int hash = idx % HASH_SYSCALL_SIZE;
 	stress_hash_syscall_t *newh, *h = hash_syscall_table[hash];
 
 	while (h) {
@@ -348,7 +354,7 @@ static inline void HOT OPTIMIZE3 syscall_add(const long number)
 		h = h->next;
 	}
 
-	newh = malloc(sizeof(*newh));
+	newh = (stress_hash_syscall_t *)malloc(sizeof(*newh));
 	if (!newh)
 		return;	/* Can't add, ignore */
 
@@ -360,10 +366,9 @@ static inline void HOT OPTIMIZE3 syscall_add(const long number)
 static inline void syscall_free(void)
 {
 	size_t i;
-	stress_hash_syscall_t *h;
 
 	for (i = 0; i < HASH_SYSCALL_SIZE; i++) {
-		h = hash_syscall_table[i];
+		stress_hash_syscall_t *h = hash_syscall_table[i];
 
 		while (h) {
 			stress_hash_syscall_t *next = h->next;
@@ -374,22 +379,26 @@ static inline void syscall_free(void)
 	}
 }
 
+
 static const int sigs[] = {
 #if defined(SIGILL)
 	SIGILL,
 #endif
-#if defined(SIGTRAP)
-	SIGTRAP,
-#endif
-#if defined(SIGFPE)
-	SIGFPE,
-#endif
 #if defined(SIGBUS)
 	SIGBUS,
 #endif
 #if defined(SIGSEGV)
 	SIGSEGV,
 #endif
+};
+
+static const int exit_sigs[] = {
+#if defined(SIGTRAP)
+	SIGTRAP,
+#endif
+#if defined(SIGFPE)
+	SIGFPE,
+#endif
 #if defined(SIGIOT)
 	SIGIOT,
 #endif
@@ -407,7 +416,7 @@ static const int sigs[] = {
 #endif
 };
 
-static const long skip_syscalls[] = {
+static const long int skip_syscalls[] = {
 /* Traditional SYS_ syscall interface */
 #if defined(SYS_accept)
 	SYS_accept,
@@ -589,6 +598,9 @@ static const long skip_syscalls[] = {
 #if defined(SYS_fchmodat)
 	SYS_fchmodat,
 #endif
+#if defined(SYS_fchmodat2)
+	SYS_fchmodat2,
+#endif
 #if defined(SYS_fchown)
 	SYS_fchown,
 #endif
@@ -919,9 +931,18 @@ static const long skip_syscalls[] = {
 #if defined(SYS_lstat64)
 	SYS_lstat64,
 #endif
+#if defined(SYS_lsm_get_self_attr)
+	SYS_lsm_get_self_attr,
+#endif
+#if defined(SYS_lsm_list_modules)
+	SYS_lsm_list_modules,
+#endif
 #if defined(SYS_madvise)
 	SYS_madvise,
 #endif
+#if defined(SYS_map_shadow_stack)
+	SYS_map_shadow_stack,
+#endif
 #if defined(SYS_mbind)
 	SYS_mbind,
 #endif
@@ -1006,6 +1027,9 @@ static const long skip_syscalls[] = {
 #if defined(SYS_mremap)
 	SYS_mremap,
 #endif
+#if defined(SYS_mseal)
+	SYS_mseal,
+#endif
 #if defined(SYS_msgctl)
 	SYS_msgctl,
 #endif
@@ -1276,9 +1300,6 @@ static const long skip_syscalls[] = {
 #if defined(SYS_sched_rr_get_interval)
 	SYS_sched_rr_get_interval,
 #endif
-#if defined(SYS_sched_rr_stress_get_interval)
-	SYS_sched_rr_stress_get_interval,
-#endif
 #if defined(SYS_sched_setaffinity)
 	SYS_sched_setaffinity,
 #endif
@@ -1716,6 +1737,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_arc_gettls)
 	__NR_arc_gettls,
 #endif
+#if defined(__NR_arch_prctl)
+	__NR_arch_prctl,
+#endif
 #if defined(__NR_arch_specific_syscall)
 	__NR_arch_specific_syscall,
 #endif
@@ -1728,6 +1752,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_arm_fadvise64_64)
 	__NR_arm_fadvise64_64,
 #endif
+#if defined(__NR_arm_sync_file_range)
+	__NR_arm_sync_file_range,
+#endif
 #if defined(__NR_atomic_barrier)
 	__NR_atomic_barrier,
 #endif
@@ -1752,6 +1779,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_break)
 	__NR_break,
 #endif
+#if defined(__NR_breakpoint)
+	__NR_breakpoint,
+#endif
 #if defined(__NR_brk)
 	__NR_brk,
 #endif
@@ -1806,6 +1836,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_clock_nanosleep)
 	__NR_clock_nanosleep,
 #endif
+#if defined(__NR_clock_nanosleep_time64)
+	__NR_clock_nanosleep_time64,
+#endif
 #if defined(__NR_clock_nanosleep64)
 	__NR_clock_nanosleep64,
 #endif
@@ -1938,6 +1971,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_faccessat)
 	__NR_faccessat,
 #endif
+#if defined(__NR_faccessat2)
+	__NR_faccessat2,
+#endif
 #if defined(__NR_fadvise64)
 	__NR_fadvise64,
 #endif
@@ -1971,6 +2007,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_fchmodat)
 	__NR_fchmodat,
 #endif
+#if defined(__NR_fchmodat2)
+	__NR_fchmodat2,
+#endif
 #if defined(__NR_fchown)
 	__NR_fchown,
 #endif
@@ -2061,6 +2100,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_futex_time64)
 	__NR_futex_time64,
 #endif
+#if defined(__NR_futex_waitv)
+	__NR_futex_waitv,
+#endif
 #if defined(__NR_futimens)
 	__NR_futimens,
 #endif
@@ -2079,6 +2121,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_get_thread_area)
 	__NR_get_thread_area,
 #endif
+#if defined(__NR_get_tls)
+	__NR_get_tls,
+#endif
 #if defined(__NR_gettls)
 	__NR_gettls,
 #endif
@@ -2364,6 +2409,12 @@ static const long skip_syscalls[] = {
 #if defined(__NR_lstat64)
 	__NR_lstat64,
 #endif
+#if defined(__NR_lsm_get_self_attr)
+	__NR_lsm_get_self_attr,
+#endif
+#if defined(__NR_lsm_list_modules)
+	__NR_lsm_list_modules,
+#endif
 #if defined(__NR_lws_entries)
 	__NR_lws_entries,
 #endif
@@ -2373,6 +2424,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_madvise1)
 	__NR_madvise1,
 #endif
+#if defined(__NR_map_shadow_stack)
+	__NR_map_shadow_stack,
+#endif
 #if defined(__NR_mbind)
 	__NR_mbind,
 #endif
@@ -2493,6 +2547,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_mremap)
 	__NR_mremap,
 #endif
+#if defined(__NR_mseal)
+	__NR_mseal,
+#endif
 #if defined(__NR_msgctl)
 	__NR_msgctl,
 #endif
@@ -2607,13 +2664,342 @@ static const long skip_syscalls[] = {
 #if defined(__NR_openat)
 	__NR_openat,
 #endif
+#if defined(__NR_openat2)
+	__NR_openat2,
+#endif
 #if defined(__NR_or1k_atomic)
 	__NR_or1k_atomic,
 #endif
-/* TODO: all NR_osf calls */
 #if defined(__NR_osf_adjtime)
 	__NR_osf_adjtime,
 #endif
+#if defined(__NR_osf_afs_syscall)
+	__NR_osf_afs_syscall,
+#endif
+#if defined(__NR_osf_alt_plock)
+	__NR_osf_alt_plock,
+#endif
+#if defined(__NR_osf_alt_setsid)
+	__NR_osf_alt_setsid,
+#endif
+#if defined(__NR_osf_alt_sigpending)
+	__NR_osf_alt_sigpending,
+#endif
+#if defined(__NR_osf_asynch_daemon)
+	__NR_osf_asynch_daemon,
+#endif
+#if defined(__NR_osf_audcntl)
+	__NR_osf_audcntl,
+#endif
+#if defined(__NR_osf_audgen)
+	__NR_osf_audgen,
+#endif
+#if defined(__NR_osf_chflags)
+	__NR_osf_chflags,
+#endif
+#if defined(__NR_osf_execve)
+	__NR_osf_execve,
+#endif
+#if defined(__NR_osf_exportfs)
+	__NR_osf_exportfs,
+#endif
+#if defined(__NR_osf_fchflags)
+	__NR_osf_fchflags,
+#endif
+#if defined(__NR_osf_fdatasync)
+	__NR_osf_fdatasync,
+#endif
+#if defined(__NR_osf_fpathconf)
+	__NR_osf_fpathconf,
+#endif
+#if defined(__NR_osf_fstat)
+	__NR_osf_fstat,
+#endif
+#if defined(__NR_osf_fstatfs64)
+	__NR_osf_fstatfs64,
+#endif
+#if defined(__NR_osf_fstatfs)
+	__NR_osf_fstatfs,
+#endif
+#if defined(__NR_osf_fuser)
+	__NR_osf_fuser,
+#endif
+#if defined(__NR_osf_getaddressconf)
+	__NR_osf_getaddressconf,
+#endif
+#if defined(__NR_osf_getdirentries)
+	__NR_osf_getdirentries,
+#endif
+#if defined(__NR_osf_getdomainname)
+	__NR_osf_getdomainname,
+#endif
+#if defined(__NR_osf_getfh)
+	__NR_osf_getfh,
+#endif
+#if defined(__NR_osf_getfsstat)
+	__NR_osf_getfsstat,
+#endif
+#if defined(__NR_osf_gethostid)
+	__NR_osf_gethostid,
+#endif
+#if defined(__NR_osf_getitimer)
+	__NR_osf_getitimer,
+#endif
+#if defined(__NR_osf_getlogin)
+	__NR_osf_getlogin,
+#endif
+#if defined(__NR_osf_getmnt)
+	__NR_osf_getmnt,
+#endif
+#if defined(__NR_osf_getrusage)
+	__NR_osf_getrusage,
+#endif
+#if defined(__NR_osf_getsysinfo)
+	__NR_osf_getsysinfo,
+#endif
+#if defined(__NR_osf_gettimeofday)
+	__NR_osf_gettimeofday,
+#endif
+#if defined(__NR_osf_kloadcall)
+	__NR_osf_kloadcall,
+#endif
+#if defined(__NR_osf_kmodcall)
+	__NR_osf_kmodcall,
+#endif
+#if defined(__NR_osf_lstat)
+	__NR_osf_lstat,
+#endif
+#if defined(__NR_osf_memcntl)
+	__NR_osf_memcntl,
+#endif
+#if defined(__NR_osf_mincore)
+	__NR_osf_mincore,
+#endif
+#if defined(__NR_osf_mount)
+	__NR_osf_mount,
+#endif
+#if defined(__NR_osf_mremap)
+	__NR_osf_mremap,
+#endif
+#if defined(__NR_osf_msfs_syscall)
+	__NR_osf_msfs_syscall,
+#endif
+#if defined(__NR_osf_msleep)
+	__NR_osf_msleep,
+#endif
+#if defined(__NR_osf_mvalid)
+	__NR_osf_mvalid,
+#endif
+#if defined(__NR_osf_mwakeup)
+	__NR_osf_mwakeup,
+#endif
+#if defined(__NR_osf_naccept)
+	__NR_osf_naccept,
+#endif
+#if defined(__NR_osf_nfssvc)
+	__NR_osf_nfssvc,
+#endif
+#if defined(__NR_osf_ngetpeername)
+	__NR_osf_ngetpeername,
+#endif
+#if defined(__NR_osf_ngetsockname)
+	__NR_osf_ngetsockname,
+#endif
+#if defined(__NR_osf_nrecvfrom)
+	__NR_osf_nrecvfrom,
+#endif
+#if defined(__NR_osf_nrecvmsg)
+	__NR_osf_nrecvmsg,
+#endif
+#if defined(__NR_osf_nsendmsg)
+	__NR_osf_nsendmsg,
+#endif
+#if defined(__NR_osf_ntp_adjtime)
+	__NR_osf_ntp_adjtime,
+#endif
+#if defined(__NR_osf_ntp_gettime)
+	__NR_osf_ntp_gettime,
+#endif
+#if defined(__NR_osf_old_creat)
+	__NR_osf_old_creat,
+#endif
+#if defined(__NR_osf_old_fstat)
+	__NR_osf_old_fstat,
+#endif
+#if defined(__NR_osf_old_getpgrp)
+	__NR_osf_old_getpgrp,
+#endif
+#if defined(__NR_osf_old_killpg)
+	__NR_osf_old_killpg,
+#endif
+#if defined(__NR_osf_old_lstat)
+	__NR_osf_old_lstat,
+#endif
+#if defined(__NR_osf_old_open)
+	__NR_osf_old_open,
+#endif
+#if defined(__NR_osf_old_sigaction)
+	__NR_osf_old_sigaction,
+#endif
+#if defined(__NR_osf_old_sigblock)
+	__NR_osf_old_sigblock,
+#endif
+#if defined(__NR_osf_old_sigreturn)
+	__NR_osf_old_sigreturn,
+#endif
+#if defined(__NR_osf_old_sigsetmask)
+	__NR_osf_old_sigsetmask,
+#endif
+#if defined(__NR_osf_old_sigvec)
+	__NR_osf_old_sigvec,
+#endif
+#if defined(__NR_osf_old_stat)
+	__NR_osf_old_stat,
+#endif
+#if defined(__NR_osf_old_vadvise)
+	__NR_osf_old_vadvise,
+#endif
+#if defined(__NR_osf_old_vtrace)
+	__NR_osf_old_vtrace,
+#endif
+#if defined(__NR_osf_old_wait)
+	__NR_osf_old_wait,
+#endif
+#if defined(__NR_osf_oldquota)
+	__NR_osf_oldquota,
+#endif
+#if defined(__NR_osf_pathconf)
+	__NR_osf_pathconf,
+#endif
+#if defined(__NR_osf_pid_block)
+	__NR_osf_pid_block,
+#endif
+#if defined(__NR_osf_pid_unblock)
+	__NR_osf_pid_unblock,
+#endif
+#if defined(__NR_osf_plock)
+	__NR_osf_plock,
+#endif
+#if defined(__NR_osf_priocntlset)
+	__NR_osf_priocntlset,
+#endif
+#if defined(__NR_osf_profil)
+	__NR_osf_profil,
+#endif
+#if defined(__NR_osf_proplist_syscall)
+	__NR_osf_proplist_syscall,
+#endif
+#if defined(__NR_osf_reboot)
+	__NR_osf_reboot,
+#endif
+#if defined(__NR_osf_revoke)
+	__NR_osf_revoke,
+#endif
+#if defined(__NR_osf_sbrk)
+	__NR_osf_sbrk,
+#endif
+#if defined(__NR_osf_security)
+	__NR_osf_security,
+#endif
+#if defined(__NR_osf_select)
+	__NR_osf_select,
+#endif
+#if defined(__NR_osf_set_program_attributes)
+	__NR_osf_set_program_attributes,
+#endif
+#if defined(__NR_osf_set_speculative)
+	__NR_osf_set_speculative,
+#endif
+#if defined(__NR_osf_sethostid)
+	__NR_osf_sethostid,
+#endif
+#if defined(__NR_osf_setitimer)
+	__NR_osf_setitimer,
+#endif
+#if defined(__NR_osf_setlogin)
+	__NR_osf_setlogin,
+#endif
+#if defined(__NR_osf_setsysinfo)
+	__NR_osf_setsysinfo,
+#endif
+#if defined(__NR_osf_settimeofday)
+	__NR_osf_settimeofday,
+#endif
+#if defined(__NR_osf_shmat)
+	__NR_osf_shmat,
+#endif
+#if defined(__NR_osf_signal)
+	__NR_osf_signal,
+#endif
+#if defined(__NR_osf_sigprocmask)
+	__NR_osf_sigprocmask,
+#endif
+#if defined(__NR_osf_sigsendset)
+	__NR_osf_sigsendset,
+#endif
+#if defined(__NR_osf_sigstack)
+	__NR_osf_sigstack,
+#endif
+#if defined(__NR_osf_sigwaitprim)
+	__NR_osf_sigwaitprim,
+#endif
+#if defined(__NR_osf_sstk)
+	__NR_osf_sstk,
+#endif
+#if defined(__NR_osf_stat)
+	__NR_osf_stat,
+#endif
+#if defined(__NR_osf_statfs64)
+	__NR_osf_statfs64,
+#endif
+#if defined(__NR_osf_statfs)
+	__NR_osf_statfs,
+#endif
+#if defined(__NR_osf_subsys_info)
+	__NR_osf_subsys_info,
+#endif
+#if defined(__NR_osf_swapctl)
+	__NR_osf_swapctl,
+#endif
+#if defined(__NR_osf_swapon)
+	__NR_osf_swapon,
+#endif
+#if defined(__NR_osf_syscall)
+	__NR_osf_syscall,
+#endif
+#if defined(__NR_osf_sysinfo)
+	__NR_osf_sysinfo,
+#endif
+#if defined(__NR_osf_table)
+	__NR_osf_table,
+#endif
+#if defined(__NR_osf_uadmin)
+	__NR_osf_uadmin,
+#endif
+#if defined(__NR_osf_usleep_thread)
+	__NR_osf_usleep_thread,
+#endif
+#if defined(__NR_osf_uswitch)
+	__NR_osf_uswitch,
+#endif
+#if defined(__NR_osf_utc_adjtime)
+	__NR_osf_utc_adjtime,
+#endif
+#if defined(__NR_osf_utc_gettime)
+	__NR_osf_utc_gettime,
+#endif
+#if defined(__NR_osf_utimes)
+	__NR_osf_utimes,
+#endif
+#if defined(__NR_osf_utsname)
+	__NR_osf_utsname,
+#endif
+#if defined(__NR_osf_wait4)
+	__NR_osf_wait4,
+#endif
+#if defined(__NR_osf_waitid)
+	__NR_osf_waitid,
+#endif
 #if defined(__NR_pause)
 	__NR_pause,
 #endif
@@ -3010,6 +3396,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_set_tid_address)
 	__NR_set_tid_address,
 #endif
+#if defined(__NR_set_tls)
+	__NR_set_tls,
+#endif
 #if defined(__NR_setdomainname)
 	__NR_setdomainname,
 #endif
@@ -3244,6 +3633,15 @@ static const long skip_syscalls[] = {
 #if defined(__NR_sys_debug_setcontext)
 	__NR_sys_debug_setcontext,
 #endif
+#if defined(__NR_sys_epoll_create)
+	__NR_sys_epoll_create,
+#endif
+#if defined(__NR_sys_epoll_ctl)
+	__NR_sys_epoll_ctl,
+#endif
+#if defined(__NR_sys_epoll_wait)
+	__NR_sys_epoll_wait,
+#endif
 #if defined(__NR_syscall)
 	__NR_syscall,
 #endif
@@ -3343,6 +3741,9 @@ static const long skip_syscalls[] = {
 #if defined(__NR_tuxcall)
 	__NR_tuxcall,
 #endif
+#if defined(__NR_udftrap)
+	__NR_udftrap,
+#endif
 #if defined(__NR_ugetrlimit)
 	__NR_ugetrlimit,
 #endif
@@ -3394,6 +3795,12 @@ static const long skip_syscalls[] = {
 #if defined(__NR_userfaultfd)
 	__NR_userfaultfd,
 #endif
+#if defined(__NR_usr26)
+	__NR_usr26,
+#endif
+#if defined(__NR_usr32)
+	__NR_usr32,
+#endif
 #if defined(__NR_ustat)
 	__NR_ustat,
 #endif
@@ -3458,7 +3865,7 @@ static const long skip_syscalls[] = {
  *  limit_procs()
  *	try to limit child resources
  */
-static void limit_procs(const unsigned long procs)
+static void limit_procs(const unsigned long int procs)
 {
 #if defined(RLIMIT_CPU) || defined(RLIMIT_NPROC)
 	struct rlimit lim;
@@ -3470,21 +3877,21 @@ static void limit_procs(const unsigned long procs)
 	(void)setrlimit(RLIMIT_CPU, &lim);
 #endif
 #if defined(RLIMIT_NPROC)
-	lim.rlim_cur = (unsigned long)procs;
-	lim.rlim_max = (unsigned long)procs;
+	lim.rlim_cur = (unsigned long int)procs;
+	lim.rlim_max = (unsigned long int)procs;
 	(void)setrlimit(RLIMIT_NPROC, &lim);
 #else
 	(void)procs;
 #endif
 }
 
-static void NORETURN MLOCKED_TEXT stress_badhandler(int signum)
+static void NORETURN MLOCKED_TEXT stress_exit_handler(int signum)
 {
 	(void)signum;
 	_exit(1);
 }
 
-static void NORETURN MLOCKED_TEXT stress_sigill_handler(int signum)
+static void NORETURN MLOCKED_TEXT stress_sig_handler(int signum)
 {
 	(void)signum;
 
@@ -3500,9 +3907,9 @@ static void NORETURN MLOCKED_TEXT stress_sigill_handler(int signum)
  *  the parent
  */
 static inline int stress_do_syscall(
-	const stress_args_t *args,
-	const long number,
-	const bool random)
+	stress_args_t *args,
+	const long int number,
+	const bool do_random)
 {
 	pid_t pid;
 	int rc = 0;
@@ -3511,34 +3918,35 @@ static inline int stress_do_syscall(
 	if (syscall_find(number)) {
 		return rc;
 	}
-	if (!keep_stressing(args))
+	if (UNLIKELY(!stress_continue(args)))
 		return 0;
 	pid = fork();
 	if (pid < 0) {
 		_exit(EXIT_NO_RESOURCE);
 	} else if (pid == 0) {
 		size_t i;
-		long arg;
+		long int arg;
 		char buffer[1024];
 
 		/* Try to limit child from spawning */
 		limit_procs(2);
 
 		/* We don't want bad ops clobbering this region */
-		stress_shared_unmap();
+		stress_shared_readonly();
 
 		/* Drop all capabilities */
 		if (stress_drop_capabilities(args->name) < 0) {
 			_exit(EXIT_NO_RESOURCE);
 		}
+		for (i = 0; i < SIZEOF_ARRAY(exit_sigs); i++) {
+			if (stress_sighandler(args->name, exit_sigs[i], stress_exit_handler, NULL) < 0)
+				_exit(EXIT_FAILURE);
+		}
 		for (i = 0; i < SIZEOF_ARRAY(sigs); i++) {
-			if (stress_sighandler(args->name, sigs[i], stress_badhandler, NULL) < 0)
+			if (stress_sighandler(args->name, sigs[i], stress_sig_handler, NULL) < 0)
 				_exit(EXIT_FAILURE);
 		}
-		if (stress_sighandler(args->name, SIGILL, stress_sigill_handler, NULL) < 0)
-			_exit(EXIT_FAILURE);
 
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
@@ -3558,19 +3966,20 @@ static inline int stress_do_syscall(
 		}
 
 		exercise_syscall(args, number,
-			(long)stress_mwc64(), (long)stress_mwc64(),
-			(long)stress_mwc64(), (long)stress_mwc64(),
-			(long)stress_mwc64(), (long)stress_mwc64(),
-			(long)stress_mwc64());
+			(long int)stress_mwc64(), (long int)stress_mwc64(),
+			(long int)stress_mwc64(), (long int)stress_mwc64(),
+			(long int)stress_mwc64(), (long int)stress_mwc64(),
+			(long int)stress_mwc64());
 
 		exercise_syscall(args, number,
-			(long)buffer, (long)buffer,
-			(long)buffer, (long)buffer,
-			(long)buffer, (long)buffer,
-			(long)buffer);
+			(long int)buffer, (long int)buffer,
+			(long int)buffer, (long int)buffer,
+			(long int)buffer, (long int)buffer,
+			(long int)buffer);
 		_exit(0);
 	} else {
-		int ret, status;
+		pid_t ret;
+		int status;
 
 		/*
 		 * Don't use shim_waitpid here, we want to force
@@ -3578,19 +3987,17 @@ static inline int stress_do_syscall(
 		 */
 		ret = waitpid(pid, &status, 0);
 		if (ret < 0) {
-			(void)kill(pid, SIGKILL);
-			ret = waitpid(pid, &status, 0);
-			(void)ret;
-
+			(void)stress_kill_pid(pid);
+			(void)shim_waitpid(pid, &status, 0);
 		}
 		rc = WEXITSTATUS(status);
 
 		/* Add to known syscalls that don't return ENOSYS */
 		if (rc != ENOSYS) {
-			if ((!random) || (number < 65536))
+			if ((!do_random) || (number < 65536))
 				syscall_add(number);
 		}
-		inc_counter(args);
+		stress_bogo_inc(args);
 	}
 	return rc;
 }
@@ -3599,38 +4006,45 @@ static inline int stress_do_syscall(
  *  stress_enosys
  *	stress system calls
  */
-static int stress_enosys(const stress_args_t *args)
+static int stress_enosys(stress_args_t *args)
 {
 	pid_t pid;
 
 #if defined(STRESS_EXERCISE_X86_SYSCALL)
 	stress_x86syscall_available = stress_cpu_x86_has_syscall();
 #endif
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
-	if (!keep_stressing(args))
+	if (UNLIKELY(!stress_continue(args)))
 		return EXIT_SUCCESS;
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto deinit;
-		pr_err("%s: fork failed: errno=%d: (%s)\n",
+		pr_err("%s: fork failed, errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
 	} else if (pid > 0) {
-		int status, ret;
+		pid_t ret;
+		int status;
 
-		(void)setpgid(pid, g_pgrp);
 		/* Parent, wait for child */
 		ret = shim_waitpid(pid, &status, 0);
 		if (ret < 0) {
 			if (errno != EINTR)
-				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
-			(void)kill(pid, SIGALRM);
+				pr_dbg("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+					args->name, (intmax_t)pid, errno, strerror(errno));
+			(void)shim_kill(pid, SIGALRM);
+
+			/* Still alive, kill it */
+			if (shim_kill(pid, 0) == 0) {
+				stress_force_killed_bogo(args);
+				(void)stress_kill_pid(pid);
+			}
 			(void)shim_waitpid(pid, &status, 0);
-			(void)kill(pid, SIGKILL);
 		} else if (WIFSIGNALED(status)) {
 			pr_dbg("%s: child died: %s (instance %d)\n",
 				args->name, stress_strsignal(WTERMSIG(status)),
@@ -3655,19 +4069,25 @@ again:
 			}
 		}
 	} else {
-		const unsigned long mask = ULONG_MAX;
+		const unsigned long int mask24 = 0xffffffUL;
+#if ULONG_MAX > 0xffffffff
+		const unsigned long int mask40 = 0xffffffffffUL;
+		const unsigned long int mask48 = 0xffffffffffffUL;
+		const unsigned long int mask56 = 0xffffffffffffffUL;
+#endif
+		const unsigned long int mask64 = ULONG_MAX;
+
 		ssize_t j;
 
 		/* Child, wrapped to catch OOMs */
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			_exit(0);
 
-		(void)setpgid(0, g_pgrp);
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
 		/* Make sure this is killable by OOM killer */
-		stress_set_oom_adjustment(args->name, true);
+		stress_set_oom_adjustment(args, true);
 
 		//limit_procs(2);
 
@@ -3675,58 +4095,72 @@ again:
 			syscall_add(skip_syscalls[j]);
 
 		do {
-			long number;
+			unsigned long int number;
 
 			/* Low sequential syscalls */
 			for (number = 0; number < MAX_SYSCALL + 1024; number++) {
-				if (!keep_stressing(args))
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
 				stress_do_syscall(args, number, false);
 			}
 
 			/* Random syscalls */
 			for (j = 0; j < 1024; j++) {
-				if (!keep_stressing(args))
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
-				stress_do_syscall(args, stress_mwc8() & mask, true);
-				if (!keep_stressing(args))
+				stress_do_syscall(args, (long int)stress_mwc8() & mask64, true);
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
-				stress_do_syscall(args, stress_mwc16() & mask, true);
-				if (!keep_stressing(args))
+				stress_do_syscall(args, (long int)stress_mwc16() & mask64, true);
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
-				stress_do_syscall(args, stress_mwc32() & mask, true);
-				if (!keep_stressing(args))
+				stress_do_syscall(args, (long int)stress_mwc32() & mask24, true);
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
-				stress_do_syscall(args, (long)(stress_mwc64() & mask), true);
+				stress_do_syscall(args, (long int)stress_mwc32() & mask64, true);
+				if (UNLIKELY(!stress_continue(args)))
+					goto finish;
+#if ULONG_MAX > 0xffffffff
+				stress_do_syscall(args, (long int)(stress_mwc64() & mask40), true);
+				if (UNLIKELY(!stress_continue(args)))
+					goto finish;
+				stress_do_syscall(args, (long int)(stress_mwc64() & mask48), true);
+				if (UNLIKELY(!stress_continue(args)))
+					goto finish;
+				stress_do_syscall(args, (long int)(stress_mwc64() & mask56), true);
+				if (UNLIKELY(!stress_continue(args)))
+					goto finish;
+				stress_do_syscall(args, (long int)(stress_mwc64() & mask64), true);
+#endif
 			}
 
 			/* Various bit masks */
 			for (number = 1; number; number <<= 1) {
-				if (!keep_stressing(args))
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
 				stress_do_syscall(args, number, false);
-				if (!keep_stressing(args))
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
 				stress_do_syscall(args, number | 1, false);
-				if (!keep_stressing(args))
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
-				stress_do_syscall(args, number | (number << 1), false);
-				if (!keep_stressing(args))
+				stress_do_syscall(args, number | (number << 1U), false);
+				if (UNLIKELY(!stress_continue(args)))
 					goto finish;
 				stress_do_syscall(args, ~number, false);
 			}
 
 			/* Various high syscalls */
 			for (number = 0xff; number; number <<= 8) {
-				long n;
+				long int n;
 
 				for (n = 0; n < 0x100; n++) {
-					if (!keep_stressing(args))
+					if (UNLIKELY(!stress_continue(args)))
 						goto finish;
 					stress_do_syscall(args, n + number, false);
 				}
 			}
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
 finish:
 		syscall_free();
 		_exit(EXIT_SUCCESS);
@@ -3736,15 +4170,16 @@ deinit:
 
 	return EXIT_SUCCESS;
 }
-stressor_info_t stress_enosys_info = {
+const stressor_info_t stress_enosys_info = {
 	.stressor = stress_enosys,
 	.class = CLASS_OS,
 	.help = help
 };
 #else
-stressor_info_t stress_enosys_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_enosys_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
-	.help = help
+	.help = help,
+	.unimplemented_reason = "built without sys/syscall.h or syscall support"
 };
 #endif
