@@ -763,12 +763,12 @@ static void stress_filerace_lockofd_rd(const int fd, const char *filename)
 {
 	/* lock 4096 bytes in first 64K  of file */
 	off_t offset = stress_mwc16() & ~(off_t)(4095);
-	char data[4096];
 
 	(void)filename;
 
 	if (lseek(fd, offset, SEEK_SET) >= 0) {
 		struct flock f;
+		char data[4096];
 
 		f.l_type = F_RDLCK;
 		f.l_whence = SEEK_SET;
@@ -1344,9 +1344,10 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 		char filename2[PATH_MAX];
 		int flag;
 		int which = stress_mwc8modn(11);
+#if defined(O_DIRECTORY)
 		int fd;
+#endif
 		DIR *dir;
-		struct dirent *d;
 		struct stat buf;
 		uint8_t n;
 
@@ -1403,7 +1404,7 @@ static void stress_filerace_child(stress_args_t *args, const char *pathname, con
 			if (stress_mwc8() < 8) {
 				dir = opendir(pathname);
 				if (dir) {
-					while ((d = readdir(dir)) != NULL) {
+					while (readdir(dir) != NULL) {
 					}
 					(void)closedir(dir);
 				}
