@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -21,6 +21,8 @@
 #include "core-builtin.h"
 #include "core-killpid.h"
 
+#include <sys/file.h>
+
 static const stress_help_t help[] = {
 	{ NULL,	"flock N",	"start N workers locking a single file" },
 	{ NULL,	"flock-ops N",	"stop after N flock bogo operations" },
@@ -48,13 +50,13 @@ static int stress_flock_child(
 
 	fd1 = open(filename, O_RDONLY);
 	if (fd1 < 0) {
-		pr_err("%s: failed to open %s: errno=%d (%s)\n",
+		pr_err("%s: failed to open %s, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
 	fd2 = open(filename, O_RDONLY);
 	if (fd2 < 0) {
-		pr_err("%s: failed to open %s: errno=%d (%s)\n",
+		pr_err("%s: failed to open %s, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		(void)close(fd1);
 		return EXIT_FAILURE;
@@ -70,14 +72,14 @@ static int stress_flock_child(
 			lock_count += 1.0;
 
 			cont = stress_continue(args);
-			if (cont)
+			if (LIKELY(cont))
 				stress_bogo_inc(args);
 
 			/*
 			 *  we have an exclusive lock on the fd, so re-doing
 			 *  the exclusive lock with LOCK_NB should not succeed
 			 */
-			if (flock(fd2, LOCK_EX | LOCK_NB) == 0) {
+			if (UNLIKELY(flock(fd2, LOCK_EX | LOCK_NB) == 0)) {
 				pr_fail("%s: unexpectedly able to double lock file using LOCK_EX, expecting error EAGAIN\n",
 					args->name);
 				rc = EXIT_FAILURE;
@@ -86,11 +88,11 @@ static int stress_flock_child(
 			}
 
 			t = stress_time_now();
-			if (flock(fd1, LOCK_UN) == 0) {
+			if (LIKELY(flock(fd1, LOCK_UN) == 0)) {
 				unlock_duration += stress_time_now() - t;
 				unlock_count += 1.0;
 			}
-			if (!cont)
+			if (UNLIKELY(!cont))
 				break;
 		}
 #else
@@ -110,15 +112,15 @@ static int stress_flock_child(
 			lock_count += 1.0;
 
 			cont = stress_continue(args);
-			if (cont)
+			if (LIKELY(cont))
 				stress_bogo_inc(args);
 
 			t = stress_time_now();
-			if (flock(fd1, LOCK_UN) == 0) {
+			if (LIKELY(flock(fd1, LOCK_UN) == 0)) {
 				unlock_duration += stress_time_now() - t;
 				unlock_count += 1.0;
 			}
-			if (!cont)
+			if (UNLIKELY(!cont))
 				break;
 		}
 
@@ -129,7 +131,7 @@ static int stress_flock_child(
 			int ret;
 
 			ret = flock(fd1, LOCK_NB);
-			if (ret == 0) {
+			if (UNLIKELY(ret == 0)) {
 				pr_fail("%s: flock failed expected EINVAL, instead got "
 					"errno=%d (%s)\n", args->name, errno, strerror(errno));
 				(void)flock(fd1, LOCK_UN);
@@ -140,7 +142,7 @@ static int stress_flock_child(
 #endif
 
 #if defined(LOCK_SH)
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		t = stress_time_now();
@@ -149,15 +151,15 @@ static int stress_flock_child(
 			lock_count += 1.0;
 
 			cont = stress_continue(args);
-			if (cont)
+			if (LIKELY(cont))
 				stress_bogo_inc(args);
 
 			t = stress_time_now();
-			if (flock(fd1, LOCK_UN) == 0) {
+			if (LIKELY(flock(fd1, LOCK_UN) == 0)) {
 				unlock_duration += stress_time_now() - t;
 				unlock_count += 1.0;
 			}
-			if (!cont)
+			if (UNLIKELY(!cont))
 				break;
 		}
 #else
@@ -166,7 +168,7 @@ static int stress_flock_child(
 
 #if defined(LOCK_SH) &&		\
     defined(LOCK_NB)
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		t = stress_time_now();
@@ -175,15 +177,15 @@ static int stress_flock_child(
 			lock_count += 1.0;
 
 			cont = stress_continue(args);
-			if (cont)
+			if (LIKELY(cont))
 				stress_bogo_inc(args);
 
 			t = stress_time_now();
-			if (flock(fd1, LOCK_UN) == 0) {
+			if (LIKELY(flock(fd1, LOCK_UN) == 0)) {
 				unlock_duration += stress_time_now() - t;
 				unlock_count += 1.0;
 			}
-			if (!cont)
+			if (UNLIKELY(!cont))
 				break;
 		}
 #else
@@ -192,7 +194,7 @@ static int stress_flock_child(
 
 #if defined(LOCK_MAND) &&	\
     defined(LOCK_READ)
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		t = stress_time_now();
@@ -201,15 +203,15 @@ static int stress_flock_child(
 			lock_count += 1.0;
 
 			cont = stress_continue(args);
-			if (cont)
+			if (LIKELY(cont))
 				stress_bogo_inc(args);
 
 			t = stress_time_now();
-			if (flock(fd1, LOCK_UN) == 0) {
+			if (LIKELY(flock(fd1, LOCK_UN) == 0)) {
 				unlock_duration += stress_time_now() - t;
 				unlock_count += 1.0;
 			}
-			if (!cont)
+			if (UNLIKELY(!cont))
 				break;
 		}
 #else
@@ -218,7 +220,7 @@ static int stress_flock_child(
 
 #if defined(LOCK_MAND) &&	\
     defined(LOCK_WRITE)
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		t = stress_time_now();
@@ -227,15 +229,15 @@ static int stress_flock_child(
 			lock_count += 1.0;
 
 			cont = stress_continue(args);
-			if (cont)
+			if (LIKELY(cont))
 				stress_bogo_inc(args);
 
 			t = stress_time_now();
-			if (flock(fd1, LOCK_UN) == 0) {
+			if (LIKELY(flock(fd1, LOCK_UN) == 0)) {
 				unlock_duration += stress_time_now() - t;
 				unlock_count += 1.0;
 			}
-			if (!cont)
+			if (UNLIKELY(!cont))
 				break;
 		}
 #else
@@ -244,7 +246,7 @@ static int stress_flock_child(
 
 #if defined(LOCK_EX) &&		\
     defined(LOCK_SH)
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		/* Exercise invalid lock combination */
@@ -254,35 +256,34 @@ static int stress_flock_child(
 			lock_count += 1.0;
 
 			cont = stress_continue(args);
-			if (cont)
+			if (LIKELY(cont))
 				stress_bogo_inc(args);
 
 			t = stress_time_now();
-			if (flock(fd1, LOCK_UN) == 0) {
+			if (LIKELY(flock(fd1, LOCK_UN) == 0)) {
 				unlock_duration += stress_time_now() - t;
 				unlock_count += 1.0;
 			}
-			if (!cont)
+			if (UNLIKELY(!cont))
 				break;
 		}
 #else
 		UNEXPECTED
 #endif
 #if defined(__linux__)
-		if ((i & 0xff) == 0) {
-			char buf[4096];
-
-			VOID_RET(ssize_t, stress_system_read("/proc/locks", buf, sizeof(buf)));
-		}
+		if (UNLIKELY((i & 0xff) == 0))
+			(void)stress_system_discard("/proc/locks");
+#else
+		(void)i;
 #endif
 	}
 	if (save_metrics) {
 		rate = (lock_count > 0.0) ? lock_duration / lock_count : 0.0;
 		stress_metrics_set(args, 0, "nanosecs per flock lock call",
-			rate * STRESS_DBL_NANOSECOND, STRESS_HARMONIC_MEAN);
+			rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
 		rate = (unlock_count > 0.0) ? unlock_duration / unlock_count : 0.0;
 		stress_metrics_set(args, 1, "nanosecs per flock unlock call",
-			rate * STRESS_DBL_NANOSECOND, STRESS_HARMONIC_MEAN);
+			rate * STRESS_DBL_NANOSECOND, STRESS_METRIC_HARMONIC_MEAN);
 	}
 	(void)close(fd2);
 	(void)close(fd1);
@@ -299,60 +300,79 @@ static int stress_flock(stress_args_t *args)
 	int fd, ret, rc = EXIT_FAILURE;
 	const int bad_fd = stress_get_bad_fd();
 	size_t i;
-	pid_t pids[MAX_FLOCK_STRESSORS];
+	stress_pid_t *s_pids, *s_pids_head = NULL;
 	char filename[PATH_MAX];
 
+	s_pids = stress_sync_s_pids_mmap(MAX_FLOCK_STRESSORS);
+	if (s_pids == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, MAX_FLOCK_STRESSORS);
+		return EXIT_NO_RESOURCE;
+        }
+
 	ret = stress_temp_dir_mk_args(args);
-	if (ret < 0)
-		return stress_exit_status(-ret);
+	if (ret < 0) {
+		rc = stress_exit_status(-ret);
+		goto err_free_s_pids;
+	}
 
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
 	fd = open(filename, O_CREAT | O_RDWR, S_IRUSR | S_IWUSR);
 	if (fd < 0) {
-		pr_err("%s: failed to create %s: errno=%d (%s)\n",
+		pr_err("%s: failed to create %s, errno=%d (%s)\n",
 			args->name, filename, errno, strerror(errno));
 		goto err;
 	}
 	(void)close(fd);
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
-
-	(void)shim_memset(pids, 0, sizeof(pids));
 	for (i = 0; i < MAX_FLOCK_STRESSORS; i++) {
-		pids[i] = fork();
-		if (pids[i] < 0) {
+		stress_sync_start_init(&s_pids[i]);
+
+		s_pids[i].pid = fork();
+		if (s_pids[i].pid < 0) {
 			goto reap;
-		} else if (pids[i] == 0) {
+		} else if (s_pids[i].pid == 0) {
+			s_pids[i].pid = getpid();
+			stress_sync_start_wait_s_pid(&s_pids[i]);
+
 			stress_parent_died_alarm();
 			(void)sched_settings_apply(true);
 
 			_exit(stress_flock_child(args, filename, bad_fd, false));
+		} else {
+			stress_sync_start_s_pid_list_add(&s_pids_head, &s_pids[i]);
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_sync_start_cont_list(s_pids_head);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	stress_flock_child(args, filename, bad_fd, true);
 	rc = EXIT_SUCCESS;
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	stress_kill_and_wait_many(args, pids, MAX_FLOCK_STRESSORS, SIGALRM, true);
+	stress_kill_and_wait_many(args, s_pids, MAX_FLOCK_STRESSORS, SIGALRM, true);
 	(void)shim_unlink(filename);
 err:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 	(void)stress_temp_dir_rm_args(args);
+err_free_s_pids:
+	(void)stress_sync_s_pids_munmap(s_pids, MAX_FLOCK_STRESSORS);
 
 	return rc;
 }
 
-stressor_info_t stress_flock_info = {
+const stressor_info_t stress_flock_info = {
 	.stressor = stress_flock,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_flock_info = {
+const stressor_info_t stress_flock_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
