@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-out-of-memory.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"handle N",	"start N workers exercising name_to_handle_at" },
@@ -59,7 +61,7 @@ static void free_mount_info(const int mounts)
  *  get_mount_info()
  *	parse mount information from /proc/self/mountinfo
  */
-static int get_mount_info(const stress_args_t *args)
+static int get_mount_info(stress_args_t *args)
 {
 	FILE *fp;
 	int mounts = 0;
@@ -69,7 +71,7 @@ static int get_mount_info(const stress_args_t *args)
 		return -1;
 	}
 
-	(void)memset(&mount_info, 0, sizeof(mount_info));
+	(void)shim_memset(&mount_info, 0, sizeof(mount_info));
 
 	while (mounts < MAX_MOUNT_IDS) {
 		char mount_path[PATH_MAX + 1];
@@ -110,37 +112,40 @@ static int get_mount_info(const stress_args_t *args)
  *	wrapper because the allocations may cause memory
  *	failures on high memory pressure environments.
  */
-static int stress_handle_child(const stress_args_t *args, void *context)
+static int stress_handle_child(stress_args_t *args, void *context)
 {
 	const int mounts = *((int *)context);
 	const int bad_fd = stress_get_bad_fd();
+	int rc = EXIT_SUCCESS;
 
 	do {
 		struct file_handle *fhp, *tmp;
 		int mount_id, mount_fd, fd, i;
 		char *ptr;
 
-		fhp = malloc(sizeof(*fhp));
-		if (!fhp)
+		fhp = (struct file_handle *)malloc(sizeof(*fhp));
+		if (UNLIKELY(!fhp))
 			continue;
 
 		fhp->handle_bytes = 0;
-		if ((name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, 0) != -1) &&
-		    (errno != EOVERFLOW)) {
+		if (UNLIKELY((name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, 0) != -1) &&
+		             (errno != EOVERFLOW))) {
 			pr_fail("%s: name_to_handle_at failed to get file handle size, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			rc = EXIT_FAILURE;
 			free(fhp);
 			break;
 		}
 		tmp = realloc(fhp, sizeof(*tmp) + fhp->handle_bytes);
-		if (tmp == NULL) {
+		if (UNLIKELY(!tmp)) {
 			free(fhp);
 			continue;
 		}
 		fhp = tmp;
-		if (name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, 0) < 0) {
+		if (UNLIKELY(name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, 0) < 0)) {
 			pr_fail("%s: name_to_handle_at failed to get file handle, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			rc = EXIT_FAILURE;
 			free(fhp);
 			break;
 		}
@@ -152,24 +157,27 @@ static int stress_handle_child(const stress_args_t *args, void *context)
 				break;
 			}
 		}
-		if (mount_fd == -2) {
+		if (UNLIKELY(mount_fd == -2)) {
 			pr_fail("%s: cannot find mount id %d\n", args->name, mount_id);
+			rc = EXIT_FAILURE;
 			free(fhp);
 			break;
 		}
-		if (mount_fd < 0) {
-			pr_fail("%s: failed to open mount path '%s': errno=%d (%s)\n",
+		if (UNLIKELY(mount_fd < 0)) {
+			pr_fail("%s: failed to open mount path '%s', errno=%d (%s)\n",
 				args->name, mount_info[i].mount_path, errno, strerror(errno));
+			rc = EXIT_FAILURE;
 			free(fhp);
 			break;
 		}
 		fd = open_by_handle_at(mount_fd, fhp, O_RDONLY);
-		if (fd < 0) {
+		if (UNLIKELY(fd < 0)) {
 			/* We don't abort if EPERM occurs, that's not a test failure */
 			if (errno != EPERM) {
-				pr_fail("%s: open_by_handle_at: failed to open: errno=%d (%s)\n",
+				pr_fail("%s: open_by_handle_at: failed to open, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				(void)close(mount_fd);
+				rc = EXIT_FAILURE;
 				free(fhp);
 				break;
 			}
@@ -180,7 +188,7 @@ static int stress_handle_child(const stress_args_t *args, void *context)
 		/* Exercise with large invalid size, EINVAL */
 		fhp->handle_bytes = 4096;
 		tmp = realloc(fhp, sizeof(*tmp) + fhp->handle_bytes);
-		if (!tmp) {
+		if (UNLIKELY(!tmp)) {
 			(void)close(mount_fd);
 			free(fhp);
 			continue;
@@ -204,14 +212,14 @@ static int stress_handle_child(const stress_args_t *args, void *context)
 		fhp->handle_bytes = 32;
 		(void)name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, 0);
 		fd = open_by_handle_at(-1, fhp, O_RDONLY);
-		if (fd >= 0)
+		if (UNLIKELY(fd >= 0))
 			(void)close(fd);
 
 		/* Exercise with invalid mount_fd, part 2 */
 		fhp->handle_bytes = 32;
 		(void)name_to_handle_at(AT_FDCWD, FILENAME, fhp, &mount_id, 0);
 		fd = open_by_handle_at(bad_fd, fhp, O_RDONLY);
-		if (fd >= 0)
+		if (UNLIKELY(fd >= 0))
 			(void)close(fd);
 
 		/*
@@ -220,17 +228,17 @@ static int stress_handle_child(const stress_args_t *args, void *context)
 		 *  failure
 		 */
 		ptr = (char *)shim_unconstify_ptr(&fhp->f_handle);
-		stress_strnrnd(ptr, 32);
+		stress_rndbuf(ptr, 32);
 		fd = open_by_handle_at(mount_fd, fhp, O_RDONLY);
-		if (fd >= 0)
+		if (UNLIKELY(fd >= 0))
 			(void)close(fd);
 
 		(void)close(mount_fd);
 		free(fhp);
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
@@ -238,7 +246,7 @@ static int stress_handle_child(const stress_args_t *args, void *context)
  *	stress system by rapid open/close calls via
  *	name_to_handle_at and open_by_handle_at
  */
-static int stress_handle(const stress_args_t *args)
+static int stress_handle(stress_args_t *args)
 {
 	int mounts, ret;
 
@@ -248,6 +256,8 @@ static int stress_handle(const stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	ret = stress_oomable_child(args, &mounts,
@@ -260,16 +270,18 @@ static int stress_handle(const stress_args_t *args)
 	return ret;
 }
 
-stressor_info_t stress_handle_info = {
+const stressor_info_t stress_handle_info = {
 	.stressor = stress_handle,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_handle_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_handle_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.help = help
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without name_to_handle_at(), open_by_handle_at() or AT_FDCWD"
 };
 #endif
