@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 #define MIN_LEASE_BREAKERS	(1)
 #define MAX_LEASE_BREAKERS	(64)
@@ -36,19 +38,9 @@ static const stress_help_t help[] = {
 	{ NULL, NULL,		    NULL }
 };
 
-static int stress_set_lease_breakers(const char *opt)
-{
-	uint64_t lease_breakers;
-
-	lease_breakers = stress_get_uint64(opt);
-	stress_check_range("lease-breakers", lease_breakers,
-		MIN_LEASE_BREAKERS, MAX_LEASE_BREAKERS);
-	return stress_set_setting("lease-breakers", TYPE_ID_UINT64, &lease_breakers);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_lease_breakers,	stress_set_lease_breakers },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_lease_breakers, "lease-breakers", TYPE_ID_UINT64, MIN_LEASE_BREAKERS, MAX_LEASE_BREAKERS, NULL },
+	END_OPT,
 };
 
 #if defined(F_SETLEASE) &&	\
@@ -80,7 +72,7 @@ static int stress_get_lease(const int fd)
  *	spawn a process
  */
 static pid_t stress_lease_spawn(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *filename)
 {
 	pid_t pid;
@@ -89,23 +81,24 @@ static pid_t stress_lease_spawn(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		return -1;
 	}
 	if (pid == 0) {
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
+		stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 		do {
 			int fd;
 
 			errno = 0;
-			fd = open(filename, O_NONBLOCK | O_WRONLY, S_IRUSR | S_IWUSR);
+			fd = open(filename, O_NONBLOCK | O_WRONLY);
 			if (fd < 0) {
 				if ((errno != EWOULDBLOCK) &&
                                     (errno != EACCES)) {
-					pr_dbg("%s: open failed (child): errno=%d: (%s)\n",
+					pr_dbg("%s: open failed (child), errno=%d: (%s)\n",
 						args->name, errno, strerror(errno));
 					if (count++ > 3)
 						break;
@@ -114,7 +107,9 @@ again:
 			}
 			(void)stress_get_lease(fd);
 			(void)close(fd);
-		} while (keep_stressing(args));
+		} while (stress_continue(args));
+
+		stress_set_proc_state(args->name, STRESS_STATE_WAIT);
 		_exit(EXIT_SUCCESS);
 	}
 	return pid;
@@ -125,19 +120,19 @@ again:
  *	try and get a lease with lock type 'lock'
  */
 static int stress_try_lease(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *filename,
 	const int flags,
 	const int lock)
 {
-	int fd;
+	int fd, rc = EXIT_SUCCESS;
 
-	fd = open(filename, flags, S_IRUSR | S_IWUSR);
+	fd = open(filename, flags);
 	if (fd < 0) {
 		int ret;
 
 		ret = stress_exit_status(errno);
-		pr_err("%s: open failed (parent): errno=%d: (%s)\n",
+		pr_err("%s: open failed (parent), errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
 		return ret;
 	}
@@ -146,41 +141,42 @@ static int stress_try_lease(
 	 *  attempt a lease lock
 	 */
 	while (fcntl(fd, F_SETLEASE, lock) < 0) {
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			goto tidy;
 	}
 	(void)stress_get_lease(fd);
 
-	inc_counter(args);
+	stress_bogo_inc(args);
 	(void)shim_sched_yield();
 
 	/*
 	 *  attempt a lease unlock
 	 */
 	while (fcntl(fd, F_SETLEASE, F_UNLCK) < 0) {
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 		if (errno != EAGAIN) {
-			pr_fail("%s: fcntl failed: errno=%d: (%s)%s\n",
+			pr_fail("%s: fcntl failed, errno=%d: (%s)%s\n",
 				args->name, errno, strerror(errno),
-				stress_fs_type(filename));
+				stress_get_fs_type(filename));
+			rc = EXIT_FAILURE;
 			break;
 		}
 	}
 tidy:
 	(void)close(fd);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
  *  stress_lease
  *	stress fcntl lease activity
  */
-static int stress_lease(const stress_args_t *args)
+static int stress_lease(stress_args_t *args)
 {
 	char filename[PATH_MAX];
-	int ret, fd, status;
+	int ret, fd;
 	pid_t l_pids[MAX_LEASE_BREAKERS];
 	uint64_t i, lease_breakers = DEFAULT_LEASE_BREAKERS;
 	double t1 = 0.0, t2 = 0.0, dt;
@@ -192,7 +188,7 @@ static int stress_lease(const stress_args_t *args)
 			lease_breakers = MIN_LEASE_BREAKERS;
 	}
 
-	(void)memset(l_pids, 0, sizeof(l_pids));
+	(void)shim_memset(l_pids, 0, sizeof(l_pids));
 
 	if (stress_sighandler(args->name, SIGIO, stress_lease_handler, NULL) < 0)
 		return EXIT_FAILURE;
@@ -206,7 +202,7 @@ static int stress_lease(const stress_args_t *args)
 	fd = creat(filename, S_IRUSR | S_IWUSR);
 	if (fd < 0) {
 		ret = stress_exit_status(errno);
-		pr_err("%s: creat failed: errno=%d: (%s)\n",
+		pr_err("%s: creat failed, errno=%d: (%s)\n",
 			args->name, errno, strerror(errno));
 		(void)stress_temp_dir_rm_args(args);
 		return ret;
@@ -224,6 +220,8 @@ static int stress_lease(const stress_args_t *args)
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	t1 = stress_time_now();
@@ -234,17 +232,15 @@ static int stress_lease(const stress_args_t *args)
 		ret = stress_try_lease(args, filename, O_RDONLY, F_RDLCK);
 		if (ret != EXIT_SUCCESS)
 			break;
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 	t2 = stress_time_now();
 
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	for (i = 0; i < lease_breakers; i++) {
-		if (l_pids[i]) {
-			(void)kill(l_pids[i], SIGKILL);
-			(void)shim_waitpid(l_pids[i], &status, 0);
-		}
+		if (l_pids[i])
+			(void)stress_kill_pid_wait(l_pids[i], NULL);
 	}
 
 	(void)shim_unlink(filename);
@@ -254,24 +250,25 @@ reap:
 	dt = t2 - t1;
 	if (dt > 0.0) {
 		stress_metrics_set(args, 0, "lease sigio interrupts per sec",
-			(double)lease_sigio / dt);
+			(double)lease_sigio / dt, STRESS_METRIC_HARMONIC_MEAN);
 	}
 
 	return ret;
 }
 
-stressor_info_t stress_lease_info = {
+const stressor_info_t stress_lease_info = {
 	.stressor = stress_lease,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_lease_info = {
+const stressor_info_t stress_lease_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without fcntl() F_SETLEASE, F_WRLCK or F_UNLCK commands"
 };
