@@ -806,11 +806,13 @@ static void *stress_memthrash_func(void *ctxt)
 	 */
 	(void)sigprocmask(SIG_BLOCK, &set, NULL);
 
-	while (!thread_terminate && stress_continue(args)) {
+	stress_random_small_sleep();
+
+	while (LIKELY(!thread_terminate && stress_continue(args))) {
 		size_t j;
 
-		for (j = MATRIX_SIZE_MIN_SHIFT; j <= MATRIX_SIZE_MAX_SHIFT &&
-		     !thread_terminate && stress_continue(args); j++) {
+		for (j = MATRIX_SIZE_MIN_SHIFT; LIKELY(j <= MATRIX_SIZE_MAX_SHIFT &&
+		     !thread_terminate && stress_continue(args)); j++) {
 			size_t mem_size = 1 << (2 * j);
 			size_t i;
 
@@ -886,19 +888,20 @@ mmap_retry:
 	mem = stress_mmap_populate(NULL, MEM_SIZE, PROT_READ | PROT_WRITE,
 			MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
 	if (mem == MAP_FAILED) {
-		if (!stress_continue_flag()) {
-			pr_dbg("%s: mmap failed: %d %s\n",
+		if (UNLIKELY(!stress_continue_flag())) {
+			pr_dbg("%s: mmap failed, errno=%d %s\n",
 				args->name, errno, strerror(errno));
 			free(pthread_info);
 			return EXIT_NO_RESOURCE;
 		}
 		(void)shim_usleep(100000);
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			goto reap_mem;
 		goto mmap_retry;
 	}
 	stress_set_vma_anon_name(mem, MEM_SIZE, "memthrash-data");
 	(void)stress_madvise_mergeable(mem, MEM_SIZE);
+	(void)stress_madvise_nohugepage(mem, MEM_SIZE);
 
 	for (i = 0; i < max_threads; i++) {
 		pthread_info[i].ret = pthread_create(&pthread_info[i].pthread,
@@ -915,7 +918,7 @@ mmap_retry:
 				args->name, ret, strerror(ret));
 			goto reap;
 		}
-		if (!stress_continue_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			goto reap;
 	}
 	/* Wait for SIGALRM or SIGINT/SIGHUP etc */
@@ -957,7 +960,7 @@ static int stress_memthrash(stress_args_t *args)
 
 	context.args = args;
 	context.total_cpus = (uint32_t)stress_get_processors_online();
-	context.max_threads = stress_memthrash_max(args->num_instances, context.total_cpus);
+	context.max_threads = stress_memthrash_max(args->instances, context.total_cpus);
 #if defined(HAVE_MEMTHRASH_NUMA)
 	{
 		context.numa_mask = stress_numa_mask_alloc();
@@ -976,12 +979,12 @@ static int stress_memthrash(stress_args_t *args)
 		pr_inf("%s: starting %" PRIu32 " thread%s on each of the %"
 			PRIu32 " stressors on a %" PRIu32 " CPU system\n",
 			args->name, context.max_threads, plural(context.max_threads),
-			args->num_instances, context.total_cpus);
-		if (context.max_threads * args->num_instances > context.total_cpus) {
+			args->instances, context.total_cpus);
+		if (context.max_threads * args->instances > context.total_cpus) {
 			pr_inf("%s: this is not an optimal choice of stressors, "
 				"try %" PRIu32 " instead\n",
 			args->name,
-			stress_memthash_optimal(args->num_instances, context.total_cpus));
+			stress_memthash_optimal(args->instances, context.total_cpus));
 		}
 	}
 
