@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-out-of-memory.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"oom-pipe N",	  "start N workers exercising large pipes" },
@@ -81,7 +82,7 @@ static void pipe_fill(
 	}
 }
 
-static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
+static int stress_oom_pipe_child(stress_args_t *args, void *ctxt)
 {
 	stress_oom_pipe_context_t *context = (stress_oom_pipe_context_t *)ctxt;
 	const size_t max_pipes = context->max_fd / 2;
@@ -101,7 +102,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 	for (i = 0; i < max_pipes * 2; i++)
 		fds[i] = -1;
 
-	for (i = 0; keep_stressing(args) && (i < max_pipes); i++) {
+	for (i = 0; LIKELY(stress_continue(args) && (i < max_pipes)); i++) {
 		int *pfd = fds + (2 * i);
 
 		if ((g_opt_flags & OPT_FLAGS_OOM_AVOID) && stress_low_memory(page_size))
@@ -134,7 +135,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 
 	do {
 		/* Set to maximum size */
-		for (i = 0, fd = fds; keep_stressing(args) && (i < max_pipes); i++, fd += 2) {
+		for (i = 0, fd = fds; LIKELY(stress_continue(args) && (i < max_pipes)); i++, fd += 2) {
 			size_t max_size = context->max_pipe_size;
 
 			if ((fd[0] < 0) || (fd[1] < 0))
@@ -150,7 +151,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 				pipe_empty(fd[0], max_size, page_size, rd_buffer);
 		}
 		/* Set to invalid size */
-		for (i = 0, fd = fds; keep_stressing(args) && (i < max_pipes); i++, fd += 2) {
+		for (i = 0, fd = fds; LIKELY(stress_continue(args) && (i < max_pipes)); i++, fd += 2) {
 			if ((fd[0] < 0) || (fd[1] < 0))
 				continue;
 			(void)fcntl(fd[0], F_SETPIPE_SZ, -1);
@@ -158,7 +159,7 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 		}
 
 		/* Set to minimum size */
-		for (i = 0, fd = fds; keep_stressing(args) && (i < max_pipes); i++, fd += 2) {
+		for (i = 0, fd = fds; LIKELY(stress_continue(args) && (i < max_pipes)); i++, fd += 2) {
 			if ((fd[0] < 0) || (fd[1] < 0))
 				continue;
 			(void)fcntl(fd[0], F_SETPIPE_SZ, page_size);
@@ -167,8 +168,8 @@ static int stress_oom_pipe_child(const stress_args_t *args, void *ctxt)
 			if (!aggressive)
 				pipe_empty(fd[0], page_size, page_size, rd_buffer);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
 	/* And close the pipes */
 clean:
@@ -183,7 +184,7 @@ clean:
  *  stress_oom_pipe
  *	stress pipe memory allocation
  */
-static int stress_oom_pipe(const stress_args_t *args)
+static int stress_oom_pipe(stress_args_t *args)
 {
 	const size_t page_size = args->page_size;
 	const size_t buffer_size = page_size << 1;
@@ -191,30 +192,38 @@ static int stress_oom_pipe(const stress_args_t *args)
 	stress_oom_pipe_context_t context;
 	void *buffer;
 
-	buffer = mmap(NULL, buffer_size, PROT_READ | PROT_WRITE,
+	buffer = stress_mmap_populate(NULL, buffer_size, PROT_READ | PROT_WRITE,
 			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
 	if (buffer == MAP_FAILED) {
-		pr_inf("%s: cannot allocate pipe write buffer, skipping stressor\n", args->name);
+		pr_inf_skip("%s: cannot allocate pipe write buffer, skipping stressor\n", args->name);
 		return EXIT_NO_RESOURCE;
 	}
+	stress_set_vma_anon_name(buffer, buffer_size, "rw-pipe-buffer");
 	context.wr_buffer = (uint32_t *)buffer;
 	context.rd_buffer = (uint32_t *)((uintptr_t)buffer + page_size);
 
 	context.max_fd = stress_get_file_limit();
 	context.max_pipe_size = stress_probe_max_pipe_size();
 
-	context.fds = calloc(context.max_fd, sizeof(*context.fds));
+	context.fds = (int *)calloc(context.max_fd, sizeof(*context.fds));
 	if (!context.fds) {
-		pr_inf("%s: cannot allocate %zd file descriptors, skipping stressor\n",
-			args->name, context.max_fd);
-		(void)munmap(buffer, buffer_size);
-		return EXIT_NO_RESOURCE;
+		/* Shrink down */
+		context.max_fd = 1024 * 1024;
+		context.fds = (int *)calloc(context.max_fd, sizeof(*context.fds));
+		if (!context.fds) {
+			pr_inf_skip("%s: cannot allocate %zd file descriptors, skipping stressor\n",
+				args->name, context.max_fd);
+			(void)munmap(buffer, buffer_size);
+			return EXIT_NO_RESOURCE;
+		}
 	}
 
 	if (context.max_pipe_size < page_size)
 		context.max_pipe_size = page_size;
 	context.max_pipe_size &= ~(page_size - 1);
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	rc = stress_oomable_child(args, &context, stress_oom_pipe_child, STRESS_OOMABLE_DROP_CAP);
@@ -227,16 +236,18 @@ static int stress_oom_pipe(const stress_args_t *args)
 	return rc;
 }
 
-stressor_info_t stress_oom_pipe_info = {
+const stressor_info_t stress_oom_pipe_info = {
 	.stressor = stress_oom_pipe,
 	.class = CLASS_MEMORY | CLASS_OS | CLASS_PATHOLOGICAL,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_oom_pipe_info = {
-	.stressor = stress_not_implemented,
+const stressor_info_t stress_oom_pipe_info = {
+	.stressor = stress_unimplemented,
 	.class = CLASS_MEMORY | CLASS_OS | CLASS_PATHOLOGICAL,
-	.help = help
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without F_SETFL, F_SETPIPE_SZ or O_NONBLOCK fcntl() commands"
 };
 #endif
