@@ -203,7 +203,7 @@ again:
 					stress_pidfd_reap(pid, pidfd);
 					return EXIT_NOT_IMPLEMENTED;
 				}
-				pr_fail("%s: pidfd_send_signal failed: errno=%d (%s)\n",
+				pr_fail("%s: pidfd_send_signal failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
 				stress_pidfd_reap(pid, pidfd);
@@ -211,16 +211,22 @@ again:
 			}
 			ret = shim_pidfd_send_signal(pidfd, SIGSTOP, NULL, 0);
 			if (UNLIKELY(ret != 0)) {
-				pr_fail("%s: pidfd_send_signal (SIGSTOP), failed: errno=%d (%s)\n",
+				pr_fail("%s: pidfd_send_signal (SIGSTOP), failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
 			}
 			ret = shim_pidfd_send_signal(pidfd, SIGCONT, NULL, 0);
 			if (UNLIKELY(ret != 0)) {
-				pr_fail("%s: pidfd_send_signal (SIGCONT), failed: errno=%d (%s)\n",
+				pr_fail("%s: pidfd_send_signal (SIGCONT), failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
 			}
+#if defined(PIDFD_SELF_THREAD)
+			(void)shim_pidfd_send_signal(PIDFD_SELF_THREAD, 0, NULL, 0);
+#endif
+#if defined(PIDFD_SELF_THREAD_GROUP)
+			(void)shim_pidfd_send_signal(PIDFD_SELF_THREAD_GROUP, 0, NULL, 0);
+#endif
 			stress_pidfd_reap(pid, pidfd);
 		}
 		stress_bogo_inc(args);
