@@ -152,6 +152,8 @@ static const stress_opt_t opts[] = {
      defined(SCHED_BATCH) ||				\
      defined(SCHED_IDLE))
 
+static double t_end = 0.0;
+
 typedef struct {
 	stress_prio_inv_child_info_t	child_info[MUTEX_PROCS];
 	pthread_mutex_t mutex;
@@ -181,7 +183,7 @@ static void cpu_exercise(const size_t instance, stress_prio_inv_info_t *prio_inv
 
 	do {
 		stress_prio_inv_getrusage(child_info);
-	} while (stress_continue(args));
+	} while (stress_continue(args) && (stress_time_now() < t_end));
 }
 
 /*
@@ -209,7 +211,7 @@ static void mutex_exercise(const size_t instance, stress_prio_inv_info_t *prio_i
 				args->name, errno, strerror(errno));
 			break;
 		}
-	} while (stress_continue(args));
+	} while (stress_continue(args) && (stress_time_now() < t_end));
 }
 
 static int stress_prio_inv_set_prio_policy(
@@ -331,6 +333,8 @@ static int stress_prio_inv(stress_args_t *args)
 		mutex_exercise,
 	};
 
+	t_end = stress_time_now() + (double)g_opt_timeout;
+
 	prio_inv_info = (stress_prio_inv_info_t *)mmap(
 				NULL, sizeof(*prio_inv_info),
 				PROT_READ | PROT_WRITE,
