@@ -19,7 +19,10 @@
  */
 #include "stress-ng.h"
 #include "core-arch.h"
+#include "core-bitops.h"
 #include "core-builtin.h"
+#include "core-capabilities.h"
+#include "core-hash.h"
 #include "core-pthread.h"
 #include "core-put.h"
 
@@ -44,8 +47,13 @@ static const stress_help_t help[] = {
 	{ NULL,	NULL,		NULL }
 };
 
+#define PROCFS_FLAG_READ	(0x01)
+#define PROCFS_FLAG_WRITE	(0x02)
+#define PROCFS_FLAG_TIMEOUT	(0x10)
+#define PROCFS_FLAG_READ_WRITE	(PROCFS_FLAG_READ | PROCFS_FLAG_WRITE)
+
 #if defined(HAVE_LIB_PTHREAD) &&	\
-    defined(__linux__)
+    (defined(__linux__) || defined(__CYGWIN__))
 
 #define PROC_BUF_SZ		(4096)
 #define MAX_PROCFS_THREADS	(4)
@@ -98,7 +106,7 @@ static int stress_dirent_proc_prune(struct dirent **dlist, const int n)
 				ignore = true;
 			else if (isdigit((unsigned char)dlist[i]->d_name[0])) {
 				/* only allow a small numeric files.. */
-				ignore = (digit_count > 1);
+				ignore = (digit_count > 5);
 				digit_count++;
 			}
 
@@ -133,23 +141,26 @@ static int stress_proc_scandir(
 	return n;
 }
 
-static uint32_t path_sum(const char *path)
+/*
+ *  mixup_hash()
+ *	for numerical pids reverse bits to mash order, otherwise
+ *	use fast string hash function
+ */
+static uint32_t mixup_hash(const char *str)
 {
-	const char *ptr = path;
-	register uint32_t sum = mixup;
+	if (isdigit((int)str[0])) {
+		const uint32_t val = atol(str);
 
-	while (*ptr) {
-		sum <<= 1;
-		sum += (uint32_t)*(ptr++);
+		return stress_reverse32(val ^ mixup);
 	}
+	return stress_hash_pjw(str) ^ mixup;
 
-	return sum;
 }
 
 static int mixup_sort(const struct dirent **d1, const struct dirent **d2)
 {
-	register const uint32_t s1 = path_sum((*d1)->d_name);
-	register const uint32_t s2 = path_sum((*d2)->d_name);
+	register const uint32_t s1 = mixup_hash((*d1)->d_name);
+	register const uint32_t s2 = mixup_hash((*d2)->d_name);
 
 	if (s1 == s2)
 		return 0;
@@ -275,9 +286,7 @@ static inline void stress_proc_rw(
 		struct stat statbuf;
 		size_t len;
 		ssize_t i;
-		bool timeout = false;
-		bool writeable = true;
-		bool skip_fifo = true;
+		int procfs_flag = PROCFS_FLAG_READ_WRITE;
 
 		ret = shim_pthread_spin_lock(&lock);
 		if (ret)
@@ -285,16 +294,16 @@ static inline void stress_proc_rw(
 		(void)shim_strscpy(path, proc_path, sizeof(path));
 		(void)shim_pthread_spin_unlock(&lock);
 
-		if (!*path || !stress_continue_flag())
+redo:
+		if (UNLIKELY(!*path || !stress_continue_flag()))
 			break;
 
 		if (!strncmp(path, "/proc/self", 10))
-			writeable = false;
-		if (!strncmp(path, "/proc", 5) && isdigit((unsigned char)path[5]))
-			writeable = false;
+			procfs_flag &= ~PROCFS_FLAG_WRITE;
+		if (!strncmp(path, "/proc/", 6) && isdigit((unsigned char)path[6]))
+			procfs_flag &= ~PROCFS_FLAG_WRITE;
 
 		t_start = stress_time_now();
-
 		if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
 			return;
 
@@ -311,16 +320,57 @@ static inline void stress_proc_rw(
 		}
 
 		/*
-		 *  fstat the file
+		 *  fstat the file, skip char and block devices
 		 */
+#if defined(S_IFMT)
 		ret = shim_fstat(fd, &statbuf);
 		if (ret == 0) {
-#if defined(S_IFMT) &&	\
-    defined(S_IFIFO)
-			if ((statbuf.st_mode & S_IFMT) != S_IFIFO)
-				skip_fifo = false;
+			char linkpath[PATH_MAX + 1];
+
+			switch (statbuf.st_mode & S_IFMT) {
+#if defined(S_IFLNK)
+			case S_IFLNK:
+				/* should never get here */
+				ret = shim_readlink(path, linkpath, sizeof(linkpath) - 1);
+				(void)close(fd);
+				if (ret < 0)
+					return;
+				linkpath[ret] = '\0';
+				(void)shim_strscpy(path, linkpath, sizeof(path));
+				goto redo;
 #endif
+#if defined(S_IFIFO)
+			case S_IFIFO:
+				/* Avoid reading/writing pipes */
+				procfs_flag &= ~PROCFS_FLAG_READ_WRITE;
+				break;
+#endif
+#if defined(S_IFCHR)
+			case S_IFCHR:
+				/*
+				 *  Reading from char devices such as tty devices steals
+				 *  user input so avoid using these
+				 */
+				procfs_flag &= ~PROCFS_FLAG_READ_WRITE;
+				break;
+#endif
+#if defined(S_IFBLK)
+			case S_IFBLK:
+				/* Avoid reading/writing block devices */
+				procfs_flag &= ~PROCFS_FLAG_READ_WRITE;
+				break;
+#endif
+#if defined(S_IFSOCK)
+			case S_IFSOCK:
+				/* Avoid reading/writing sockets */
+				procfs_flag &= ~PROCFS_FLAG_READ_WRITE;
+				break;
+#endif
+			default:
+				break;
+			}
 		}
+#endif
 
 #if defined(__linux__)
 		/*
@@ -357,81 +407,83 @@ static inline void stress_proc_rw(
 		/*
 		 *  Multiple randomly sized reads
 		 */
-		for (i = 0; i < 4096 * PROC_BUF_SZ; i++) {
-			const ssize_t sz = 1 + stress_mwc32modn((uint32_t)sizeof(buffer));
+		if (procfs_flag & PROCFS_FLAG_READ) {
+			for (i = 0; i < 4096 * PROC_BUF_SZ; i++) {
+				const ssize_t sz = 1 + stress_mwc32modn((uint32_t)sizeof(buffer));
 
-			if (!stress_continue_flag())
-				break;
-			ret = read(fd, buffer, (size_t)sz);
-			if (ret < 0)
-				break;
-			if (ret < sz)
-				break;
-			i += sz;
+				if (UNLIKELY(!stress_continue_flag()))
+					break;
+				ret = read(fd, buffer, (size_t)sz);
+				if (ret < 0)
+					break;
+				if (ret < sz)
+					break;
+				i += sz;
 
+				if ((stress_time_now() - t_start) > threshold)
+					goto timeout_close;
+			}
+			(void)close(fd);
+
+			/* Multiple 1 char sized reads */
+			if (procfs_flag & PROCFS_FLAG_READ) {
+				if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
+					return;
+				for (i = 0; ; i++) {
+					if (UNLIKELY(!stress_continue_flag()))
+						break;
+					if (((i & 0x0f) == 0) && ((stress_time_now() - t_start) > threshold))
+						goto timeout_close;
+					ret = read(fd, buffer, 1);
+					if (ret < 1)
+						break;
+				}
+				(void)close(fd);
+			}
+
+			if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
+				return;
 			if ((stress_time_now() - t_start) > threshold)
 				goto timeout_close;
-		}
-		(void)close(fd);
+			/*
+			 *  Zero sized reads
+			 */
+			ret = read(fd, buffer, 0);
+			if (ret < 0)
+				goto err;
+			/*
+			 *  Broken offset reads, see Linux commit
+			 *  3bfa7e141b0bbb818b25e0daafb65aee92e49ac4
+			 *  "fs/seq_file.c: seq_read(): add info message
+			 *  about buggy .next functions"
+			 */
+			pos = lseek(fd, 0, SEEK_SET);
+			if (pos < 0)
+				goto mmap_test;
+			(void)shim_memset(buffer, 0, sizeof(buffer));
+			ret = read(fd, buffer, sizeof(buffer));
+			if (ret < 0)
+				goto mmap_test;
+			if (ret < (ssize_t)(sizeof(buffer) >> 1)) {
+				char *bptr;
 
-		/* Multiple 1 char sized reads */
-		if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
-			return;
-		for (i = 0; ; i++) {
-			if (!stress_continue_flag())
-				break;
-			if (((i & 0x0f) == 0) && ((stress_time_now() - t_start) > threshold))
-				goto timeout_close;
-			ret = read(fd, buffer, 1);
-			if (ret < 1)
-				break;
-		}
-		(void)close(fd);
+				for (bptr = buffer; *bptr && (*bptr != '\n'); bptr++)
+					;
+				if (*bptr == '\n') {
+					const off_t offset = 2 + (bptr - buffer);
 
-		if ((fd = open(path, O_RDONLY | O_NONBLOCK)) < 0)
-			return;
-		if ((stress_time_now() - t_start) > threshold)
-			goto timeout_close;
-		/*
-		 *  Zero sized reads
-		 */
-		ret = read(fd, buffer, 0);
-		if (ret < 0)
-			goto err;
-		/*
-		 *  Broken offset reads, see Linux commit
-		 *  3bfa7e141b0bbb818b25e0daafb65aee92e49ac4
-		 *  "fs/seq_file.c: seq_read(): add info message
-		 *  about buggy .next functions"
-		 */
-		pos = lseek(fd, 0, SEEK_SET);
-		if (pos < 0)
-			goto mmap_test;
-		(void)shim_memset(buffer, 0, sizeof(buffer));
-		ret = read(fd, buffer, sizeof(buffer));
-		if (ret < 0)
-			goto mmap_test;
-		if (ret < (ssize_t)(sizeof(buffer) >> 1)) {
-			char *bptr;
-
-			for (bptr = buffer; *bptr && (*bptr != '\n'); bptr++)
-				;
-			if (*bptr == '\n') {
-				const off_t offset = 2 + (bptr - buffer);
-
-				pos = lseek(fd, offset, SEEK_SET);
-				if (pos == offset) {
-					/* Causes incorrect 2nd read */
-					VOID_RET(ssize_t, read(fd, buffer, sizeof(buffer)));
+					pos = lseek(fd, offset, SEEK_SET);
+					if (pos == offset) {
+						/* Causes incorrect 2nd read */
+						VOID_RET(ssize_t, read(fd, buffer, sizeof(buffer)));
+					}
 				}
 			}
-		}
 
-		/*
-		 *  exercise 13 x 5 byte reads backwards through procfs file to
-		 *  ensure we perform some weird misaligned non-word sized reads
-		 */
-		if (!skip_fifo) {
+			/*
+			 *  exercise 13 x 5 byte reads backwards through procfs file to
+			 *  ensure we perform some weird misaligned non-word sized reads
+			 */
 			off_t dec;
 
 			pos = lseek(fd, 0, SEEK_END);
@@ -455,22 +507,24 @@ static inline void stress_proc_rw(
 		}
 
 mmap_test:
-		/*
-		 *  mmap it
-		 */
-		ptr = (uint8_t *)mmap(NULL, page_size, PROT_READ,
+		if (procfs_flag & PROCFS_FLAG_READ) {
+			/*
+			 *  mmap it
+			 */
+			ptr = (uint8_t *)mmap(NULL, page_size, PROT_READ,
 #if defined(MAP_POPULATE)
-			MAP_POPULATE |
+				MAP_POPULATE |
 #endif
-			MAP_SHARED | MAP_ANONYMOUS,
-			fd, 0);
-		if (ptr != MAP_FAILED) {
-			stress_uint8_put(*ptr);
-			(void)munmap((void *)ptr, page_size);
-		}
+				MAP_SHARED | MAP_ANONYMOUS,
+				fd, 0);
+			if (ptr != MAP_FAILED) {
+				stress_uint8_put(*ptr);
+				(void)munmap((void *)ptr, page_size);
+			}
 
-		if ((stress_time_now() - t_start) > threshold)
-			goto timeout_close;
+			if ((stress_time_now() - t_start) > threshold)
+				goto timeout_close;
+		}
 
 #if defined(FIONREAD)
 		{
@@ -520,33 +574,35 @@ mmap_test:
 		}
 #endif
 
-		/*
-		 *  Seek and read
-		 */
-		pos = lseek(fd, 0, SEEK_SET);
-		if (pos == (off_t)-1)
-			goto err;
-		pos = lseek(fd, 1, SEEK_CUR);
-		if (pos == (off_t)-1)
-			goto err;
-		pos = lseek(fd, 0, SEEK_END);
-		if (pos == (off_t)-1)
-			goto err;
-		pos = lseek(fd, 1, SEEK_SET);
-		if (pos == (off_t)-1)
+		if (procfs_flag & PROCFS_FLAG_READ) {
+			/*
+			 *  Seek and read
+			 */
+			pos = lseek(fd, 0, SEEK_SET);
+			if (pos == (off_t)-1)
+				goto err;
+			pos = lseek(fd, 1, SEEK_CUR);
+			if (pos == (off_t)-1)
+				goto err;
+			pos = lseek(fd, 0, SEEK_END);
+			if (pos == (off_t)-1)
+				goto err;
+			pos = lseek(fd, 1, SEEK_SET);
+			if (pos == (off_t)-1)
 			goto err;
 
-		if ((stress_time_now() - t_start) > threshold)
-			goto timeout_close;
+			if ((stress_time_now() - t_start) > threshold)
+				goto timeout_close;
 
-		VOID_RET(ssize_t, read(fd, buffer, 1));
+			VOID_RET(ssize_t, read(fd, buffer, 1));
+		}
 err:
 		if ((stress_time_now() - t_start) > threshold)
 			goto timeout_close;
 
 		(void)close(fd);
 
-		if (writeable && ctxt->writeable) {
+		if ((procfs_flag & PROCFS_FLAG_WRITE) && ctxt->writeable) {
 			/*
 			 *  Zero sized writes
 			 */
@@ -584,7 +640,7 @@ err:
 		}
 next:
 		if (loops > 0) {
-			if (timeout)
+			if (procfs_flag & PROCFS_FLAG_TIMEOUT)
 				break;
 			loops--;
 		}
@@ -592,7 +648,7 @@ next:
 
 timeout_close:
 		(void)close(fd);
-		timeout = true;
+		procfs_flag |= PROCFS_FLAG_TIMEOUT;
 		goto next;
 	}
 }
@@ -639,13 +695,23 @@ static void stress_proc_dir(
 	int i, n, ret;
 	char tmp[PATH_MAX];
 
-	if (!stress_continue_flag())
+	if (UNLIKELY(!stress_continue_flag()))
 		return;
 
 	/* Don't want to go too deep */
 	if (depth > 20)
 		return;
 
+#if defined(__CYGWIN__)
+	/*
+	 * Cygwin maps the Windows registry to /proc/registry{,32,64}
+	 * (>1M entries, >50K entries on depth 2) and the path names of the
+	 * NTDLL layer to /proc/sys, ignore both for now
+	 */
+	if (!strncmp(path, "/proc/registry", 14) || !strcmp(path, "/proc/sys"))
+		return;
+#endif
+
 	mixup = stress_mwc32();
 	dlist = NULL;
 	n = stress_proc_scandir(path, &dlist, NULL, mixup_sort);
@@ -655,7 +721,7 @@ static void stress_proc_dir(
 	}
 
 	/* Non-directories files first */
-	for (i = 0; (i < n) && stress_continue_flag(); i++) {
+	for (i = 0; LIKELY((i < n) && stress_continue_flag()); i++) {
 		struct dirent *d = dlist[i];
 		unsigned char type;
 
@@ -687,7 +753,7 @@ static void stress_proc_dir(
 	}
 
 	/* Now recurse on directories */
-	for (i = 0; (i < n) && stress_continue_flag(); i++) {
+	for (i = 0; LIKELY((i < n) && stress_continue_flag()); i++) {
 		struct dirent *d = dlist[i];
 
 		if (d && (shim_dirent_type(path, d) == SHIM_DT_DIR)) {
@@ -716,6 +782,7 @@ static char *stress_random_pid(void)
 
 	(void)shim_strscpy(path, "/proc/self", sizeof(path));
 
+	mixup = stress_mwc32();
 	n = stress_proc_scandir("/proc", &dlist, NULL, mixup_sort);
 	if (!n) {
 		stress_dirent_list_free(dlist, n);
@@ -726,7 +793,7 @@ static char *stress_random_pid(void)
 	 *  try 32 random probes before giving up
 	 */
 	for (i = 0, j = 0; i < 32; i++) {
-		char *name;
+		const char *name;
 
 		j += (size_t)stress_mwc32();
 		j %= (size_t)n;
@@ -766,7 +833,7 @@ static int stress_procfs(stress_args_t *args)
 	stress_ctxt_t ctxt;
 	struct dirent **dlist = NULL;
 
-	n = stress_proc_scandir("/proc", &dlist, NULL, alphasort);
+	n = stress_proc_scandir("/proc", &dlist, NULL, mixup_sort);
 	if (n <= 0)
 		return stress_procfs_no_entries(args);
 
@@ -775,7 +842,7 @@ static int stress_procfs(stress_args_t *args)
 	(void)shim_strscpy(proc_path, "/proc/self", sizeof(proc_path));
 
 	ctxt.args = args;
-	ctxt.writeable = (geteuid() != 0);
+	ctxt.writeable = !stress_check_capability(SHIM_CAP_IS_ROOT);
 
 	rc = shim_pthread_spin_init(&lock, PTHREAD_PROCESS_PRIVATE);
 	if (rc) {
@@ -796,14 +863,14 @@ static int stress_procfs(stress_args_t *args)
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		size_t j = args->instance % (size_t)n;
+		size_t j = stress_mwc32() % n;
 
 		for (i = 0; i < n; i++) {
 			char procfspath[PATH_MAX];
-			const struct dirent *d = dlist[i];
+			const struct dirent *d = dlist[j];
 			unsigned char type;
 
-			if (!stress_continue(args))
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 
 			stress_mk_filename(procfspath, sizeof(procfspath), "/proc", d->d_name);
@@ -820,11 +887,11 @@ static int stress_procfs(stress_args_t *args)
 				stress_proc_dir(&ctxt, procfspath, true, 0);
 			}
 
-			j = (j + args->num_instances) % (size_t)n;
+			j = (j + args->instance + 1) % n;
 			stress_bogo_inc(args);
 		}
 
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		stress_proc_dir(&ctxt, stress_random_pid(), true, 0);
