@@ -1,11 +1,9 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,6 +17,12 @@
  */
 #include "stress-ng.h"
 
+#include <sys/ioctl.h>
+
+#if defined(HAVE_TERMIOS_H)
+#include <termios.h>
+#endif
+
 #if defined(HAVE_TERMIO_H)
 #include <termio.h>
 #endif
@@ -46,23 +50,9 @@ typedef struct {
 
 #endif
 
-/*
- *  stress_set_pty_max()
- *	set ptr maximum
- */
-static int stress_set_pty_max(const char *opt)
-{
-	uint64_t pty_max;
-
-	pty_max = stress_get_uint64(opt);
-	stress_check_range("pty-max", pty_max,
-		MIN_PTYS, MAX_PTYS);
-	return stress_set_setting("pty-max", TYPE_ID_UINT64, &pty_max);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_pty_max,	stress_set_pty_max },
-	{ 0,		NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_pty_max, "pty-max", TYPE_ID_UINT64, MIN_PTYS, MAX_PTYS, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_TERMIOS_H) &&	\
@@ -73,22 +63,30 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *  stress_pty
  *	stress pty handling
  */
-static int stress_pty(const stress_args_t *args)
+static int stress_pty(stress_args_t *args)
 {
 	uint64_t pty_max = DEFAULT_PTYS;
 	stress_pty_info_t *ptys;
 	const pid_t pid = getpid();
+	int rc = EXIT_SUCCESS;
 
-	(void)stress_get_setting("pty-max", &pty_max);
+	if (!stress_get_setting("pty-max", &pty_max)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			pty_max = MAX_PTYS;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			pty_max = MIN_PTYS;
+	}
 
-	ptys = calloc(pty_max, sizeof(*ptys));
+	ptys = (stress_pty_info_t *)calloc(pty_max, sizeof(*ptys));
 	if (!ptys) {
-		pr_inf_skip("%s: allocation of pty array failed: %d (%s) "
+		pr_inf_skip("%s: allocation of pty array failed, errno=%d (%s) "
 			"skipping stressor\n",
 			args->name, errno, strerror(errno));
 		return EXIT_NO_RESOURCE;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -98,41 +96,48 @@ static int stress_pty(const stress_args_t *args)
 			ptys[n].follower = -1;
 			ptys[n].leader = open("/dev/ptmx", O_RDWR);
 			if (ptys[n].leader < 0) {
-				if ((errno != ENOMEM) &&
-				    (errno != ENOSPC) &&
-				    (errno != EIO) &&
-				    (errno != EMFILE)) {
-					pr_fail("%s: open /dev/ptmx failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
-					goto clean;
-				}
+				if ((errno == ENOMEM) ||
+				    (errno == ENOSPC) ||
+				    (errno == EIO) ||
+				    (errno == EMFILE))
+					break;
+				pr_fail("%s: open /dev/ptmx failed, errno=%d (%s)\n",
+					args->name, errno, strerror(errno));
+				rc = EXIT_FAILURE;
+				goto clean;
 			} else {
 				ptys[n].followername = ptsname(ptys[n].leader);
-				if (!ptys[n].followername) {
+				if (UNLIKELY(!ptys[n].followername)) {
 					pr_fail("%s: ptsname failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 					goto clean;
 				}
-				if (grantpt(ptys[n].leader) < 0) {
+				if (UNLIKELY(grantpt(ptys[n].leader) < 0)) {
 					pr_fail("%s: grantpt failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 					goto clean;
 				}
-				if (unlockpt(ptys[n].leader) < 0) {
+				if (UNLIKELY(unlockpt(ptys[n].leader) < 0)) {
 					pr_fail("%s: unlockpt failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 					goto clean;
 				}
 				ptys[n].follower = open(ptys[n].followername, O_RDWR);
-				if (ptys[n].follower < 0) {
+				if (UNLIKELY(ptys[n].follower < 0)) {
+					if (errno == EINTR)
+						break;
 					if (errno != EMFILE) {
 						pr_fail("%s: open %s failed, errno=%d (%s)\n",
 							args->name, ptys[n].followername, errno, strerror(errno));
+						rc = EXIT_FAILURE;
 						goto clean;
 					}
 				}
 			}
-			if (!keep_stressing_flag())
+			if (UNLIKELY(!stress_continue_flag()))
 				goto clean;
 		}
 		/*
@@ -149,38 +154,44 @@ static int stress_pty(const stress_args_t *args)
 			{
 				struct termios ios;
 
-				if (tcgetattr(ptys[i].leader, &ios) < 0) {
+				if (UNLIKELY(tcgetattr(ptys[i].leader, &ios) < 0)) {
 					pr_fail("%s: tcgetattr on leader pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 #endif
 #if defined(HAVE_TCDRAIN)
 			{
-				if (tcdrain(ptys[i].follower) < 0) {
+				if (UNLIKELY((tcdrain(ptys[i].follower) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: tcdrain on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 #endif
 #if defined(HAVE_TCFLUSH)
 			{
 #if defined(TCIFLUSH)
-				if (tcflush(ptys[i].follower, TCIFLUSH) < 0) {
+				if (UNLIKELY(tcflush(ptys[i].follower, TCIFLUSH) < 0)) {
 					pr_fail("%s: tcflush TCIFLUSH on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 #endif
 #if defined(TCOFLUSH)
-				if (tcflush(ptys[i].follower, TCOFLUSH) < 0) {
+				if (UNLIKELY(tcflush(ptys[i].follower, TCOFLUSH) < 0)) {
 					pr_fail("%s: tcflush TCOFLUSH on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 #endif
 #if defined(TCIOFLUSH)
-				if (tcflush(ptys[i].follower, TCIOFLUSH) < 0) {
+				if (UNLIKELY(tcflush(ptys[i].follower, TCIOFLUSH) < 0)) {
 					pr_fail("%s: tcflush TCOOFLUSH on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 #endif
 			}
@@ -189,26 +200,30 @@ static int stress_pty(const stress_args_t *args)
 #if defined(TCOOFF) && \
     defined(TCOON)
 			{
-				if (tcflow(ptys[i].follower, TCOOFF) < 0) {
+				if (UNLIKELY(tcflow(ptys[i].follower, TCOOFF) < 0)) {
 					pr_fail("%s: tcflow TCOOFF on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
-				if (tcflow(ptys[i].follower, TCOON) < 0) {
+				if (UNLIKELY(tcflow(ptys[i].follower, TCOON) < 0)) {
 					pr_fail("%s: tcflow TCOON on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 #endif
 #if defined(TCIOFF) && \
     defined(TCION)
 			{
-				if (tcflow(ptys[i].follower, TCIOFF) < 0) {
+				if (UNLIKELY(tcflow(ptys[i].follower, TCIOFF) < 0)) {
 					pr_fail("%s: tcflow TCIOFF on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
-				if (tcflow(ptys[i].follower, TCION) < 0) {
+				if (UNLIKELY(tcflow(ptys[i].follower, TCION) < 0)) {
 					pr_fail("%s: tcflow TCION on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 #endif
@@ -217,140 +232,168 @@ static int stress_pty(const stress_args_t *args)
 			{
 				struct termios ios;
 
-				if ((ioctl(ptys[i].follower, TCGETS, &ios) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TCGETS, &ios) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TCGETS on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TCSETS)
 			{
 				struct termios ios;
 
-				if ((ioctl(ptys[i].follower, TCSETS, &ios) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TCSETS, &ios) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TCSETS on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TCSETSW)
 			{
 				struct termios ios;
 
-				if ((ioctl(ptys[i].follower, TCSETSW, &ios) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TCSETSW, &ios) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TCSETSW on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TCSETSF)
 			{
 				struct termios ios;
 
-				if ((ioctl(ptys[i].follower, TCSETSF, &ios) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TCSETSF, &ios) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TCSETSF on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TCGETA)
 			{
 				struct termio io;
 
-				if ((ioctl(ptys[i].follower, TCGETA, &io) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TCGETA, &io) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TCGETA on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TCSETA)
 			{
 				struct termio io;
 
-				if ((ioctl(ptys[i].follower, TCSETA, &io) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TCSETA, &io) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TCSETA on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TCSETAW)
 			{
 				struct termio io;
 
-				if ((ioctl(ptys[i].follower, TCSETAW, &io) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TCSETAW, &io) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TCSETAW on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TCSETAF)
 			{
 				struct termio io;
 
-				if ((ioctl(ptys[i].follower, TCSETAF, &io) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TCSETAF, &io) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TCSETAF on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TIOCGLCKTRMIOS)
 			{
 				struct termios ios;
 
-				if ((ioctl(ptys[i].follower, TIOCGLCKTRMIOS, &ios) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TIOCGLCKTRMIOS, &ios) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TIOCGLCKTRMIOS on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TIOCGWINSZ)
 			{
 				struct winsize ws;
 
-				if ((ioctl(ptys[i].follower, TIOCGWINSZ, &ws) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TIOCGWINSZ, &ws) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TIOCGWINSZ on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TIOCSWINSZ)
 			{
 				struct winsize ws;
 
-				if ((ioctl(ptys[i].follower, TIOCSWINSZ, &ws) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TIOCSWINSZ, &ws) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TIOCSWINSZ on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(FIONREAD)
 			{
 				int arg;
 
-				if ((ioctl(ptys[i].follower, FIONREAD, &arg) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, FIONREAD, &arg) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl FIONREAD on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TIOCINQ)
 			{
 				int arg;
 
-				if ((ioctl(ptys[i].follower, TIOCINQ, &arg) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TIOCINQ, &arg) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TIOCINQ on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TIOCOUTQ)
 			{
 				int arg;
 
-				if ((ioctl(ptys[i].follower, TIOCOUTQ, &arg) < 0) &&
-				    (errno != EINTR))
+				if (UNLIKELY((ioctl(ptys[i].follower, TIOCOUTQ, &arg) < 0) &&
+					     (errno != EINTR))) {
 					pr_fail("%s: ioctl TIOCOUTQ on follower pty failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
+				}
 			}
 #endif
 #if defined(TIOCGPTLCK)
@@ -385,9 +428,74 @@ static int stress_pty(const stress_args_t *args)
 				(void)ret;
 			}
 #endif
-			if (!keep_stressing_flag())
+			{
+				struct termios ios;
+
+				if (tcgetattr(ptys[i].follower, &ios) == 0) {
+#if defined(HAVE_CFGETISPEED)
+					(void)cfgetispeed(&ios);
+#endif
+#if defined(HAVE_CFGETOSPEED)
+					(void)cfgetospeed(&ios);
+#endif
+				}
+				if (tcgetattr(ptys[i].leader, &ios) == 0) {
+#if defined(HAVE_CFGETISPEED)
+					(void)cfgetispeed(&ios);
+#endif
+#if defined(HAVE_CFGETOSPEED)
+					(void)cfgetospeed(&ios);
+#endif
+				}
+			}
+			if (UNLIKELY(!stress_continue_flag()))
 				goto clean;
 		}
+#if defined(TIOCSETD) &&	\
+    defined(TIOCGETD) &&	\
+    defined(TCXONC)
+		if ((args->instance == 0) && (fcntl(ptys[i].follower, F_SETFL, O_NONBLOCK) == 0)) {
+#if defined(NR_LDISCS)
+			const int max_ldisc = NR_LDISCS;
+#else
+			const int max_ldisc = 32;
+#endif
+			int ldisc, orig_ldisc;
+
+			if (ioctl(ptys[i].follower, TIOCGETD, &orig_ldisc) == 0) {
+				pr_block_begin();
+				for (ldisc = 0; ldisc < max_ldisc; ldisc++) {
+					int j;
+
+					if (ioctl(ptys[i].follower, TIOCSETD, &ldisc) < 0)
+						break;
+					for (j = 0; j < 256; j++) {
+						if (ioctl(ptys[i].follower, TCXONC, 0) < 0)
+							break;
+						VOID_RET(ssize_t, write(ptys[i].follower, "", 1));
+						if (ioctl(ptys[i].follower, TCXONC, 1) < 0)
+							break;
+					}
+				}
+				VOID_RET(int, ioctl(ptys[i].follower, TIOCSETD, &orig_ldisc));
+				pr_block_end();
+				(void)shim_sched_yield();
+			}
+		}
+#endif
+
+#if defined(HAVE_PATHCONF)
+#if defined(_PC_MAX_CANON)
+		VOID_RET(long int, pathconf("/dev/ptmx", _PC_MAX_CANON));
+#endif
+#if defined(_PC_MAX_INPUT)
+		VOID_RET(long int, pathconf("/dev/ptmx", _PC_MAX_INPUT));
+#endif
+#if defined(_PC_VDISABLE)
+		VOID_RET(long int, pathconf("/dev/ptmx", _PC_VDISABLE));
+#endif
+#endif
+
 clean:
 		/*
 		 *  and close
@@ -398,28 +506,29 @@ clean:
 			if (ptys[i].leader != -1)
 				(void)close(ptys[i].leader);
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
 	free(ptys);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-stressor_info_t stress_pty_info = {
+const stressor_info_t stress_pty_info = {
 	.stressor = stress_pty,
 	.class = CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_pty_info = {
+const stressor_info_t stress_pty_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without termios.h, termio.h or ptsname()"
 };
