@@ -177,7 +177,7 @@ seq_wr_retry:
 
 PRAGMA_UNROLL_N(8)
 		for (j = 0; j < (BUF_SIZE / sizeof(*buf)); j++)
-			buf[j] = (buffer_t)o + j;
+			buf[j] = (buffer_t)(o << 12) + j;
 
 		pret = pwrite(fd, buf, BUF_SIZE, (off_t)i);
 		if (pret <= 0) {
@@ -207,6 +207,13 @@ PRAGMA_UNROLL_N(8)
 	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	if (statbuf.st_size < (off_t)readahead_bytes) {
+		pr_inf_skip("%s: out of free file space on %s, stressor instance %" PRIu32 " terminating early\n",
+			args->name, stress_get_temp_path(), args->instance);
+		rc = EXIT_NO_RESOURCE;
+		goto close_finish;
+	}
+
 	stress_readahead_generate_offsets(offsets, rounded_readahead_bytes);
 
 	do {
@@ -224,8 +231,8 @@ rnd_rd_retry:
 				if ((errno == EAGAIN) || (errno == EINTR))
 					goto rnd_rd_retry;
 				if (errno) {
-					pr_fail("%s: read failed, errno=%d (%s)%s\n",
-						args->name, errno, strerror(errno), fs_type);
+					pr_fail("%s: read failed, errno=%d (%s)%s at offset 0x%" PRIxMAX "\n",
+						args->name, errno, strerror(errno), fs_type, (intmax_t)offsets[i]);
 					goto close_finish;
 				}
 				continue;
@@ -239,10 +246,16 @@ rnd_rd_retry:
 
 PRAGMA_UNROLL_N(8)
 				for (j = 0; j < (BUF_SIZE / sizeof(*buf)); j++) {
-					const buffer_t v = (buffer_t)o + j;
+					const buffer_t v = (buffer_t)(o << 12) + j;
 
-					if (UNLIKELY(buf[j] != v))
+					if (UNLIKELY(buf[j] != v)) {
+						if (baddata == 0) {
+							pr_inf("%s: first data error at offset 0x%" PRIxMAX
+								", got 0x%" PRIx64 ", expecting 0x%" PRIx64 "\n", 
+								args->name, (intmax_t)(offsets[i] + (j * sizeof(*buf))), buf[j], v);
+						}
 						baddata++;
+					}
 				}
 				if (UNLIKELY(baddata)) {
 					pr_fail("%s: error in data between 0x%" PRIxMAX " and 0x%" PRIxMAX "\n",
