@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,6 +20,8 @@
 #include "stress-ng.h"
 #include "core-capabilities.h"
 
+#include <sched.h>
+
 static const stress_help_t help[] = {
 	{ NULL,	"reboot N",	"start N workers that exercise bad reboot calls" },
 	{ NULL,	"reboot-ops N",	"stop after N bogo reboot operations" },
@@ -55,9 +57,9 @@ static const uint32_t boot_magic[] = {
  *  reboot_func()
  *	reboot a process in a PID namespace
  */
-static int NORETURN reboot_clone_func(void *arg)
+static inline void NORETURN no_return_reboot_clone_func(void *arg)
 {
-	size_t i, j = stress_mwc8() % SIZEOF_ARRAY(boot_magic);
+	size_t i, j = stress_mwc8modn(SIZEOF_ARRAY(boot_magic));
 
 	(void)arg;
 
@@ -67,11 +69,20 @@ static int NORETURN reboot_clone_func(void *arg)
 		VOID_RET(int, shim_reboot((int)SHIM_LINUX_BOOT_MAGIC1,
 			(int)boot_magic[j],
 			(int)SHIM_LINUX_REBOOT_CMD_POWER_OFF, NULL));
-		j = (j + 1) % SIZEOF_ARRAY(boot_magic);
+		j++;
+		if (j >= SIZEOF_ARRAY(boot_magic))
+			j = 0;
 	}
 
+	_exit(errno);
+}
+
+static int reboot_clone_func(void *arg)
+{
+	no_return_reboot_clone_func(arg);
+
 	/* Should never get here */
-	exit(errno);
+	return -1;
 }
 #endif
 
@@ -79,13 +90,14 @@ static int NORETURN reboot_clone_func(void *arg)
  *  stress_reboot()
  *	stress reboot system call
  */
-static int stress_reboot(const stress_args_t *args)
+static int stress_reboot(stress_args_t *args)
 {
 	const bool reboot_capable = stress_check_capability(SHIM_CAP_SYS_BOOT);
+	int rc = EXIT_SUCCESS;
 #if defined(HAVE_CLONE)
 	char *stack;
 
-	stack = malloc(CLONE_STACK_SIZE);
+	stack = (char *)malloc(CLONE_STACK_SIZE);
 	if (!stack) {
 		pr_inf_skip("%s: out of memory allocating %zd byte stack, skipping stressor\n",
 			args->name, (size_t)CLONE_STACK_SIZE);
@@ -93,6 +105,8 @@ static int stress_reboot(const stress_args_t *args)
 	}
 #endif
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
@@ -111,22 +125,30 @@ static int stress_reboot(const stress_args_t *args)
 			(void)shim_waitpid(pid, &status, (int)__WCLONE);
 			ret = WEXITSTATUS(status);
 			if (WIFEXITED(status) && (ret != 0)) {
-				pr_fail("%s: reboot in PID namespace failed, errno = %d (%s)\n",
+				pr_fail("%s: reboot in PID namespace failed, errno=%d (%s)\n",
 					args->name, ret, strerror(ret));
+				rc = EXIT_FAILURE;
 			}
 		}
 #endif
 		ret = shim_reboot(0, 0, (int)SHIM_LINUX_REBOOT_CMD_RESTART, 0);
 		if (ret < 0) {
 			if (reboot_capable) {
+				if (errno == EPERM) {
+					pr_inf_skip("%s: no permission, skipping stressor\n", args->name);
+					rc = EXIT_NO_RESOURCE;
+					break;
+				}
 				if (errno != EINVAL) {
 					pr_fail("%s: reboot with incorrect magic didn't return EINVAL, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			} else {
 				if ((errno != EPERM) && (errno != EINVAL)) {
 					pr_fail("%s: reboot when not reboot capable didn't return EPERM, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 		}
@@ -134,14 +156,21 @@ static int stress_reboot(const stress_args_t *args)
 		ret = shim_reboot(0, 0, (int)SHIM_LINUX_REBOOT_CMD_SW_SUSPEND, 0);
 		if (ret < 0) {
 			if (reboot_capable) {
+				if (errno == EPERM) {
+					pr_inf_skip("%s: no permission, skipping stressor\n", args->name);
+					rc = EXIT_NO_RESOURCE;
+					break;
+				}
 				if (errno != EINVAL) {
 					pr_fail("%s: reboot with incorrect magic didn't return EINVAL, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			} else {
 				if ((errno != EPERM) && (errno != EINVAL)) {
 					pr_fail("%s: reboot when not reboot capable didn't return EPERM, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 		}
@@ -159,11 +188,12 @@ static int stress_reboot(const stress_args_t *args)
 				if (errno != EPERM) {
 					pr_fail("%s: reboot when not reboot capable didn't return EPERM, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
+					rc = EXIT_FAILURE;
 				}
 			}
 		}
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
@@ -171,10 +201,10 @@ static int stress_reboot(const stress_args_t *args)
 	free(stack);
 #endif
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-stressor_info_t stress_reboot_info = {
+const stressor_info_t stress_reboot_info = {
 	.stressor = stress_reboot,
 	.class = CLASS_OS,
 	.verify = VERIFY_ALWAYS,
@@ -183,9 +213,10 @@ stressor_info_t stress_reboot_info = {
 
 #else
 
-stressor_info_t stress_reboot_info = {
+const stressor_info_t stress_reboot_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "only supported on Linux with reboot() support"
 };
