@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,7 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 
 #if defined(HAVE_RENAMEAT) &&	\
     defined(O_DIRECTORY)
@@ -42,7 +43,7 @@ static const stress_help_t help[] = {
  *	exercise renameat with various illegal argument combinations
  */
 static int exercise_renameat(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *old_name,
 	const int old_fd,
 	const char *new_name,
@@ -93,7 +94,7 @@ static int exercise_renameat(
  *	exercise renameat2 with various illegal argument combinations
  */
 static int exercise_renameat2(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const char *old_name,
 	const int oldfd,
 	const char *new_name,
@@ -210,7 +211,7 @@ static char *stress_basename(char *filename)
  *  stress_rename()
  *	stress system by renames
  */
-static int stress_rename(const stress_args_t *args)
+static int stress_rename(stress_args_t *args)
 {
 	char name1[PATH_MAX], name2[PATH_MAX];
 	char *oldname = name1, *newname = name2, *tmpname;
@@ -224,8 +225,8 @@ static int stress_rename(const stress_args_t *args)
 	int tmp_fd;
 #endif
 
-	(void)memset(name1, 0, sizeof(name1));
-	(void)memset(name2, 0, sizeof(name2));
+	(void)shim_memset(name1, 0, sizeof(name1));
+	(void)shim_memset(name2, 0, sizeof(name2));
 
 	if (stress_temp_dir_mk(args->name, args->pid, inst1) < 0)
 		return EXIT_FAILURE;
@@ -244,16 +245,19 @@ static int stress_rename(const stress_args_t *args)
 	}
 #endif
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 restart:
 	(void)stress_temp_filename(oldname, PATH_MAX,
 		args->name, args->pid, inst1, i++);
 
 	if ((fp = fopen(oldname, "w+")) == NULL) {
-		int rc = exit_status(errno);
+		int rc = stress_exit_status(errno);
 
-		pr_err("%s: fopen failed: errno=%d: (%s)%s\n",
+		pr_err("%s: fopen failed, errno=%d: (%s)%s\n",
 			args->name, errno, strerror(errno),
-			stress_fs_type(oldname));
+			stress_get_fs_type(oldname));
 		(void)stress_temp_dir_rm(args->name, args->pid, inst1);
 		(void)stress_temp_dir_rm(args->name, args->pid, inst2);
 #if defined(EXERCISE_RENAMEAT) ||	\
@@ -263,11 +267,10 @@ restart:
 #endif
 		return rc;
 	}
+	(void)fflush(fp);
 	(void)fclose(fp);
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
-
-	while (keep_stressing(args)) {
+	while (stress_continue(args)) {
 		(void)stress_temp_filename(newname, PATH_MAX,
 			args->name, args->pid, inst2, i++);
 		if (rename(oldname, newname) < 0) {
@@ -279,8 +282,8 @@ restart:
 		tmpname = oldname;
 		oldname = newname;
 		newname = tmpname;
-		inc_counter(args);
-		if (!keep_stressing(args))
+		stress_bogo_inc(args);
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		(void)stress_temp_filename(newname, PATH_MAX,
@@ -294,8 +297,8 @@ restart:
 		tmpname = oldname;
 		oldname = newname;
 		newname = tmpname;
-		inc_counter(args);
-		if (!keep_stressing(args))
+		stress_bogo_inc(args);
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 #if defined(EXERCISE_RENAMEAT)
@@ -325,8 +328,8 @@ restart:
 			oldname = newname;
 			newname = tmpname;
 
-			inc_counter(args);
-			if (!keep_stressing(args))
+			stress_bogo_inc(args);
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 		}
 #endif
@@ -357,8 +360,8 @@ restart:
 			oldname = newname;
 			newname = tmpname;
 
-			inc_counter(args);
-			if (!keep_stressing(args))
+			stress_bogo_inc(args);
+			if (UNLIKELY(!stress_continue(args)))
 				break;
 		}
 #endif
@@ -381,7 +384,7 @@ restart:
 	return EXIT_SUCCESS;
 }
 
-stressor_info_t stress_rename_info = {
+const stressor_info_t stress_rename_info = {
 	.stressor = stress_rename,
 	.class = CLASS_FILESYSTEM | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
