@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,10 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
+#include "core-out-of-memory.h"
+#include "core-pragma.h"
 
 #define RMAP_CHILD_MAX		(16)
 #define MAPPINGS_MAX		(64)
@@ -48,108 +52,154 @@ static void MLOCKED_TEXT NORETURN stress_rmap_handler(int signum)
 {
 	(void)signum;
 
-	(void)kill(getppid(), SIGALRM);
 	_exit(0);
 }
 
-static void stress_rmap_touch(uint32_t *ptr, const size_t sz)
+static int OPTIMIZE3 stress_rmap_touch(
+	stress_args_t *args,
+	const int child_index,
+	uint32_t *addr,
+	const size_t sz)
 {
-	const uint32_t *end = (uint32_t *)((uintptr_t)ptr + sz);
-	static uint32_t val = 0;
-	register const size_t inc = 64 >> 2; /* sizeof(*ptr) */
+	register uintptr_t *begin = ((uintptr_t *)addr) + child_index;
+	register const uintptr_t *end = (uintptr_t *)((uintptr_t)addr + sz);
+	register uintptr_t *ptr;
+	register const size_t inc = RMAP_CHILD_MAX * sizeof(uintptr_t);
+	register uintptr_t mix = stress_mwc64();
 
-	while (ptr < end) {
-		/* Bump val, never fill memory with non-zero val */
-		val++;
-		val = val ? val : 1;
+	/* fill and put check value in that always has lowest bit set */
+PRAGMA_UNROLL_N(8)
+	for (ptr = begin; ptr < end; ptr += inc) {
+		register const uintptr_t val = (uintptr_t)ptr ^ mix;
 
 		*ptr = val;
-		ptr += inc;
 	}
+
+	/* read back and check */
+PRAGMA_UNROLL_N(8)
+	for (ptr = begin; ptr < end; ptr += inc) {
+		register const uintptr_t chk = (uintptr_t)ptr ^ mix;
+
+		if (UNLIKELY(*ptr != chk)) {
+			pr_fail("%s: address 0x%p check failure, "
+				"got 0x%" PRIxPTR ", "
+				"expected 0x%" PRIxPTR "\n",
+				args->name, ptr, chk, *ptr);
+			return -1;
+		}
+	}
+	return 0;
 }
 
 static void NORETURN stress_rmap_child(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const size_t page_size,
+	const int child_index,
 	uint32_t *mappings[MAPPINGS_MAX])
 {
 	const size_t sz = MAPPING_PAGES * page_size;
+	int rc = EXIT_SUCCESS;
+
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	do {
-		ssize_t i;
+		register ssize_t i;
 		const uint8_t rnd8 = stress_mwc8();
 		const int sync_flag = (rnd8 & 0x80) ? MS_ASYNC : MS_SYNC;
 
 		switch (rnd8 & 3) {
-		case 0: for (i = 0; i < MAPPINGS_MAX; i++) {
+		case 0:
+			for (i = 0; i < MAPPINGS_MAX; i++) {
 				if (mappings[i] != MAP_FAILED) {
-					if (!inc_counter_lock(args, counter_lock, false))
+					if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, false)))
 						break;
-					stress_rmap_touch(mappings[i], sz);
+					if (UNLIKELY(stress_rmap_touch(args, child_index, mappings[i], sz) < 0)) {
+						rc = EXIT_FAILURE;
+						goto fail;
+					}
 					(void)shim_msync(mappings[i], sz, sync_flag);
 				}
 			}
 			break;
-		case 1: for (i = MAPPINGS_MAX - 1; i >= 0; i--) {
+		case 1:
+			for (i = MAPPINGS_MAX - 1; i >= 0; i--) {
 				if (mappings[i] != MAP_FAILED) {
-					if (!inc_counter_lock(args, counter_lock, false))
+					if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, false)))
 						break;
-					stress_rmap_touch(mappings[i], sz);
+					if (UNLIKELY(stress_rmap_touch(args, child_index, mappings[i], sz) < 0)) {
+						rc = EXIT_FAILURE;
+						goto fail;
+					}
 					(void)shim_msync(mappings[i], sz, sync_flag);
 				}
 			}
 			break;
-		case 2: for (i = 0; i < MAPPINGS_MAX; i++) {
+		case 2:
+			for (i = 0; i < MAPPINGS_MAX; i++) {
 				const size_t j = stress_mwc32modn(MAPPINGS_MAX);
 
 				if (mappings[j] != MAP_FAILED) {
-					if (!inc_counter_lock(args, counter_lock, false))
+					if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, false)))
 						break;
-					stress_rmap_touch(mappings[j], sz);
+					if (UNLIKELY(stress_rmap_touch(args, child_index, mappings[j], sz) < 0)) {
+						rc = EXIT_FAILURE;
+						goto fail;
+					}
 					(void)shim_msync(mappings[j], sz, sync_flag);
 				}
 			}
 			break;
-		case 3: for (i = 0; i < MAPPINGS_MAX - 1; i++) {
+		case 3:
+			for (i = 0; i < MAPPINGS_MAX - 1; i++) {
 				if (mappings[i] != MAP_FAILED) {
-					if (!inc_counter_lock(args, counter_lock, false))
+					if (UNLIKELY(!stress_bogo_inc_lock(args, counter_lock, false)))
 						break;
-					stress_rmap_touch(mappings[i], sz);
+					if (UNLIKELY(stress_rmap_touch(args, child_index, mappings[i], sz) > 0)) {
+						rc = EXIT_FAILURE;
+						goto fail;
+					}
 					(void)shim_msync(mappings[i], sz, sync_flag);
 				}
 			}
 			break;
 		}
-	} while (inc_counter_lock(args, counter_lock, true));
+	} while (stress_bogo_inc_lock(args, counter_lock, true));
 
-	(void)kill(getppid(), SIGALRM);
-
-	_exit(0);
+fail:
+	stress_set_proc_state(args->name, STRESS_STATE_WAIT);
+	_exit(rc);
 }
 
 /*
  *  stress_rmap()
  *	stress mmap
  */
-static int stress_rmap(const stress_args_t *args)
+static int stress_rmap(stress_args_t *args)
 {
 	const size_t page_size = args->page_size;
 	const size_t sz = ((MAPPINGS_MAX - 1) + MAPPING_PAGES) * page_size;
-	int fd = -1;
+	int fd = -1, rc;
 	size_t i;
-	ssize_t rc;
-	pid_t pids[RMAP_CHILD_MAX];
+	stress_pid_t *s_pids, *s_pids_head = NULL;
 	uint32_t *mappings[MAPPINGS_MAX];
 	uint32_t *paddings[MAPPINGS_MAX];
 	char filename[PATH_MAX];
 
-	counter_lock = stress_lock_create();
-	if (!counter_lock) {
-		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
+	s_pids = stress_sync_s_pids_mmap(RMAP_CHILD_MAX);
+	if (s_pids == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, RMAP_CHILD_MAX);
 		return EXIT_NO_RESOURCE;
 	}
 
-	(void)memset(pids, 0, sizeof(pids));
+	counter_lock = stress_lock_create("counter");
+	if (!counter_lock) {
+		pr_inf_skip("%s: failed to create counter lock. skipping stressor\n", args->name);
+		(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
+		return EXIT_NO_RESOURCE;
+	}
 
 	for (i = 0; i < MAPPINGS_MAX; i++) {
 		mappings[i] = MAP_FAILED;
@@ -157,11 +207,14 @@ static int stress_rmap(const stress_args_t *args)
 	}
 
 	/* Make sure this is killable by OOM killer */
-	stress_set_oom_adjustment(args->name, true);
+	stress_set_oom_adjustment(args, true);
 
 	rc = stress_temp_dir_mk_args(args);
-	if (rc < 0)
+	if (rc < 0) {
+		(void)stress_lock_destroy(counter_lock);
+		(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 		return stress_exit_status((int)-rc);
+	}
 
 	(void)stress_temp_filename_args(args,
 		filename, sizeof(filename), stress_mwc32());
@@ -173,6 +226,7 @@ static int stress_rmap(const stress_args_t *args)
 		(void)shim_unlink(filename);
 		(void)stress_temp_dir_rm_args(args);
 		(void)stress_lock_destroy(counter_lock);
+		(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 
 		return (int)rc;
 	}
@@ -184,6 +238,7 @@ static int stress_rmap(const stress_args_t *args)
 		(void)close(fd);
 		(void)stress_temp_dir_rm_args(args);
 		(void)stress_lock_destroy(counter_lock);
+		(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 
 		return EXIT_NO_RESOURCE;
 	}
@@ -191,31 +246,38 @@ static int stress_rmap(const stress_args_t *args)
 	for (i = 0; i < MAPPINGS_MAX; i++) {
 		const off_t offset = (off_t)(i * page_size);
 
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto cleanup;
 
 		mappings[i] =
-			(uint32_t *)mmap(0, MAPPING_PAGES * page_size, PROT_READ | PROT_WRITE,
+			(uint32_t *)mmap(NULL, MAPPING_PAGES * page_size, PROT_READ | PROT_WRITE,
 				MAP_SHARED, fd, offset);
 		/* Squeeze at least a page in between each mapping */
 		paddings[i] =
-			(uint32_t *)mmap(0, page_size, PROT_READ | PROT_WRITE,
+			(uint32_t *)mmap(NULL, page_size, PROT_READ | PROT_WRITE,
 				MAP_SHARED | MAP_ANONYMOUS, -1, 0);
+		if (paddings[i] != MAP_FAILED)
+			stress_set_vma_anon_name(paddings[i], page_size, "mmap-padding");
 	}
 
 	/*
 	 *  Spawn children workers
 	 */
 	for (i = 0; i < RMAP_CHILD_MAX; i++) {
-		if (!keep_stressing(args))
+		stress_sync_start_init(&s_pids[i]);
+
+		if (UNLIKELY(!stress_continue(args)))
 			goto cleanup;
 
-		pids[i] = fork();
-		if (pids[i] < 0) {
-			pr_err("%s: fork failed: errno=%d: (%s)\n",
+		s_pids[i].pid = fork();
+		if (s_pids[i].pid < 0) {
+			pr_err("%s: fork failed, errno=%d: (%s)\n",
 				args->name, errno, strerror(errno));
 			goto cleanup;
-		} else if (pids[i] == 0) {
+		} else if (s_pids[i].pid == 0) {
+			s_pids[i].pid = getpid();
+			stress_sync_start_wait_s_pid(&s_pids[i]);
+
 			if (stress_sighandler(args->name, SIGALRM,
 			    stress_rmap_handler, NULL) < 0)
 				_exit(EXIT_FAILURE);
@@ -224,47 +286,30 @@ static int stress_rmap(const stress_args_t *args)
 			(void)sched_settings_apply(true);
 
 			/* Make sure this is killable by OOM killer */
-			stress_set_oom_adjustment(args->name, true);
-			stress_rmap_child(args, page_size, mappings);
+			stress_set_oom_adjustment(args, true);
+
+			stress_rmap_child(args, page_size, i, mappings);
+		} else {
+			stress_sync_start_s_pid_list_add(&s_pids_head, &s_pids[i]);
 		}
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_sync_start_cont_list(s_pids_head);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	/*
 	 *  Wait for SIGINT or SIGALRM
 	 */
-	while (inc_counter_lock(args, counter_lock, false)) {
+	while (stress_bogo_inc_lock(args, counter_lock, false)) {
 		pause();
 	}
 
 cleanup:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 
-	/*
-	 *  Kill and wait for children
-	 */
-	for (i = 0; i < RMAP_CHILD_MAX; i++) {
-		if (pids[i] <= 0)
-			continue;
-		(void)stress_killpid(pids[i]);
-	}
-	for (i = 0; i < RMAP_CHILD_MAX; i++) {
-		int status, ret;
-
-		if (pids[i] <= 0)
-			continue;
-
-		ret = shim_waitpid(pids[i], &status, 0);
-		if (ret < 0) {
-			if (errno != EINTR)
-				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
-			(void)kill(pids[i], SIGTERM);
-			(void)stress_killpid(pids[i]);
-			(void)shim_waitpid(pids[i], &status, 0);
-		}
-	}
+	rc = stress_kill_and_wait_many(args, s_pids, RMAP_CHILD_MAX, SIGALRM, true);
 
 	for (i = 0; i < MAPPINGS_MAX; i++) {
 		if (mappings[i] != MAP_FAILED)
@@ -276,12 +321,14 @@ cleanup:
 	(void)close(fd);
 	(void)stress_temp_dir_rm_args(args);
 	(void)stress_lock_destroy(counter_lock);
+	(void)stress_sync_s_pids_munmap(s_pids, RMAP_CHILD_MAX);
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-stressor_info_t stress_rmap_info = {
+const stressor_info_t stress_rmap_info = {
 	.stressor = stress_rmap,
 	.class = CLASS_OS | CLASS_MEMORY,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
