@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-affinity.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 #include "core-net.h"
 
 #if defined(HAVE_SYS_UN_H)
@@ -38,8 +41,6 @@ UNEXPECTED
 UNEXPECTED
 #endif
 
-#define MIN_SCTP_PORT		(1024)
-#define MAX_SCTP_PORT		(65535)
 #define DEFAULT_SCTP_PORT	(9000)
 
 #define SOCKET_BUF		(8192)	/* Socket I/O buffer size */
@@ -71,80 +72,36 @@ static uint64_t	sigpipe_count;
 UNEXPECTED
 #endif
 
-/*
- *  stress_set_sctp_port()
- *	set port to use
- */
-static int stress_set_sctp_port(const char *opt)
-{
-	int sctp_port;
+/* See include/uapi/linux/sctp.h */
+enum shim_sctp_sched_type {
+	SHIM_SCTP_SS_FCFS,
+	SHIM_SCTP_SS_PRIO,
+	SHIM_SCTP_SS_RR,
+	SHIM_SCTP_SS_FC,
+	SHIM_SCTP_SS_WFQ,
+};
 
-	stress_set_net_port("sctp-port", opt,
-		MIN_SCTP_PORT, MAX_SCTP_PORT - STRESS_PROCS_MAX,
-		&sctp_port);
-	return stress_set_setting("sctp-port", TYPE_ID_INT, &sctp_port);
+static const stress_sctp_sched_t stress_sctp_scheds[] = {
+	{ (int)SHIM_SCTP_SS_FC,		"fc" },
+	{ (int)SHIM_SCTP_SS_FCFS,	"fcfs" },
+	{ (int)SHIM_SCTP_SS_PRIO,	"prio" },
+	{ (int)SHIM_SCTP_SS_RR,		"rr" },
+	{ (int)SHIM_SCTP_SS_WFQ,	"wfq" },
+};
+
+static const char *stress_sctp_sched(const size_t i)
+{
+	return (i < SIZEOF_ARRAY(stress_sctp_scheds)) ? stress_sctp_scheds[i].name : NULL;
 }
 
-/*
- *  stress_set_sctp_sched()
- *	set scheduler to use
- */
-static int stress_set_sctp_sched(const char *opt)
-{
-#if defined(HAVE_SCTP_SCHED_TYPE) && 	\
-    defined(HAVE_SCTP_ASSOC_VALUE)
-	size_t i;
+static int sctp_domain_mask = DOMAIN_INET | DOMAIN_INET6;
 
-	static const stress_sctp_sched_t stress_sctp_scheds[] = {
-		{ (int)SCTP_SS_FCFS,	"fcfs" },
-		{ (int)SCTP_SS_PRIO,	"prio" },
-		{ (int)SCTP_SS_RR,	"rr", },
-	};
-
-	for (i = 0; i < SIZEOF_ARRAY(stress_sctp_scheds); i++) {
-		if (!strcmp(opt, stress_sctp_scheds[i].name)) {
-			return stress_set_setting("sctp-sched", TYPE_ID_INT, &stress_sctp_scheds[i].sched_type);
-		}
-	}
-	(void)fprintf(stderr, "sctp-sched must be one of:");
-	for (i = 0; i < SIZEOF_ARRAY(stress_sctp_scheds); i++) {
-                (void)fprintf(stderr, " %s", stress_sctp_scheds[i].name);
-        }
-        (void)fprintf(stderr, "\n");
-	return -1;
-#else
-	(void)opt;
-	pr_inf("sctp-sched option ignored, no SCTP scheduler types available\n");
-	return 0;
-#endif
-}
-
-/*
- *  stress_set_sctp_domain()
- *	set the socket domain option
- */
-static int stress_set_sctp_domain(const char *name)
-{
-	int ret, sctp_domain;
-
-	ret = stress_set_net_domain(DOMAIN_INET | DOMAIN_INET6, "sctp-domain",
-				     name, &sctp_domain);
-	stress_set_setting("sctp-domain", TYPE_ID_INT, &sctp_domain);
-
-	return ret;
-}
-
-static int stress_set_sctp_if(const char *name)
-{
-        return stress_set_setting("sctp-if", TYPE_ID_STR, name);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_sctp_domain,	stress_set_sctp_domain },
-	{ OPT_sctp_if,		stress_set_sctp_if },
-	{ OPT_sctp_port,	stress_set_sctp_port },
-	{ OPT_sctp_sched,	stress_set_sctp_sched },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_sctp_domain, "sctp-domain", TYPE_ID_INT_DOMAIN, 0, 0, &sctp_domain_mask },
+	{ OPT_sctp_if,     "sctp-if",     TYPE_ID_STR, 0, 0, NULL },
+	{ OPT_sctp_port,   "sctp-port",   TYPE_ID_INT_PORT, MIN_PORT, MAX_PORT, NULL },
+	{ OPT_sctp_sched,  "sctp-sched",  TYPE_ID_SIZE_T_METHOD, 0, 0, stress_sctp_sched },
+	END_OPT,
 };
 
 #if defined(HAVE_LIB_SCTP) &&	\
@@ -158,7 +115,7 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
 							\
 	ret = getsockopt(fd, IPPROTO_SCTP, opt,		\
 		 &info, &opt_len);			\
-	if (ret == 0) {					\
+	if (LIKELY(ret == 0)) {				\
 		VOID_RET(int, setsockopt(fd,		\
 			IPPROTO_SCTP, opt, &info,	\
 			opt_len));			\
@@ -217,7 +174,7 @@ static void stress_sctp_sockopts(const int fd)
 		static bool once = false;
 
 		if (once == false) {
-			STRESS_SCTP_SOCKOPT(SCTP_MAXSEG, int)
+			STRESS_SCTP_SOCKOPT(SCTP_MAXSEG, int64_t)
 			once = true;
 		}
 	}
@@ -252,7 +209,7 @@ static void stress_sctp_sockopts(const int fd)
 		static bool once = false;
 
 		if (once == false) {
-			STRESS_SCTP_SOCKOPT(SCTP_MAX_BURST, unsigned int)
+			STRESS_SCTP_SOCKOPT(SCTP_MAX_BURST, uint64_t)
 			once = true;
 		}
 	}
@@ -363,10 +320,10 @@ static void stress_sctp_sockopts(const int fd)
 	STRESS_SCTP_SOCKOPT(SCTP_EXPOSE_POTENTIALLY_FAILED_STATE, struct sctp_assoc_value)
 #endif
 #if defined(SCTP_REMOTE_UDP_ENCAPS_PORT) && 	\
-    defined(HAVE_SCTP_ASSOCIATION)
-	STRESS_SCTP_SOCKOPT(SCTP_REMOTE_UDP_ENCAPS_PORT, struct sctp_association)
+    defined(HAVE_SCTP_UDPENCAPS)
+	STRESS_SCTP_SOCKOPT(SCTP_REMOTE_UDP_ENCAPS_PORT, struct sctp_udpencaps)
 #else
-	//UNEXPECTED
+	UNEXPECTED
 #endif
 #if defined(SCTP_PLPMTUD_PROBE_INTERVAL) &&	\
     defined(HAVE_SCTP_PROBEINTERVAL)
@@ -380,17 +337,18 @@ static void stress_sctp_sockopts(const int fd)
  *  stress_sctp_client()
  *	client reader
  */
-static int stress_sctp_client(
-	const stress_args_t *args,
+static int OPTIMIZE3 stress_sctp_client(
+	stress_args_t *args,
 	const pid_t mypid,
 	const int sctp_port,
 	const int sctp_domain,
-	const int sctp_sched,
+	const int sctp_sched_type,
 	const char *sctp_if)
 {
 	struct sockaddr *addr;
+	int rc = EXIT_SUCCESS;
 
-	(void)sctp_sched;
+	(void)sctp_sched_type;
 
 	stress_parent_died_alarm();
 	(void)sched_settings_apply(true);
@@ -404,43 +362,45 @@ static int stress_sctp_client(
 		struct sctp_event_subscribe events;
 #endif
 retry:
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			return EXIT_FAILURE;
-		if ((fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP)) < 0) {
+		fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP);
+		if (UNLIKELY(fd < 0)) {
 			if (errno == EPROTONOSUPPORT) {
 				if (args->instance == 0)
 					pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 						args->name);
-				return EXIT_FAILURE;
+				return EXIT_NO_RESOURCE;
 			}
 			pr_fail("%s: socket failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return EXIT_FAILURE;
 		}
 
-		if (stress_set_sockaddr_if(args->name, args->instance, mypid,
+		if (UNLIKELY(stress_set_sockaddr_if(args->name, args->instance, mypid,
 				sctp_domain, sctp_port, sctp_if,
-				&addr, &addr_len, NET_ADDR_LOOPBACK) < 0) {
+				&addr, &addr_len, NET_ADDR_LOOPBACK) < 0)) {
 			(void)close(fd);
 			return EXIT_FAILURE;
 		}
-		if (connect(fd, addr, addr_len) < 0) {
+		if (UNLIKELY(connect(fd, addr, addr_len) < 0)) {
+			const int save_errno = errno;
+
 			(void)close(fd);
 			(void)shim_usleep(10000);
 			retries++;
-			if (retries > 100) {
+			if (UNLIKELY(retries > 100)) {
 				/* Give up.. */
-				pr_fail("%s: connect failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_fail("%s: connect failed after 100 retries, errno=%d (%s)\n",
+					args->name, save_errno, strerror(save_errno));
 				return EXIT_FAILURE;
 			}
 			goto retry;
 		}
 #if defined(HAVE_SCTP_EVENT_SUBSCRIBE)
-		(void)memset(&events, 0, sizeof(events));
+		(void)shim_memset(&events, 0, sizeof(events));
 		events.sctp_data_io_event = 1;
-		if (setsockopt(fd, SOL_SCTP, SCTP_EVENTS, &events,
-			sizeof(events)) < 0) {
+		if (UNLIKELY(setsockopt(fd, SOL_SCTP, SCTP_EVENTS, &events, sizeof(events)) < 0)) {
 			(void)close(fd);
 			pr_fail("%s: setsockopt failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
@@ -449,11 +409,11 @@ retry:
 #endif
 #if defined(HAVE_SCTP_SCHED_TYPE) &&	\
     defined(HAVE_SCTP_ASSOC_VALUE)
-		if (sctp_sched > -1) {
+		if (sctp_sched_type > -1) {
 			struct sctp_assoc_value val;
 
-			(void)memset(&val, 0, sizeof(val));
-			val.assoc_value = (uint32_t)sctp_sched;
+			(void)shim_memset(&val, 0, sizeof(val));
+			val.assoc_value = (uint32_t)sctp_sched_type;
 			(void)setsockopt(fd, SOL_SCTP, SCTP_STREAM_SCHEDULER, &val, sizeof(val));
 		}
 #endif
@@ -465,58 +425,73 @@ retry:
 
 			n = sctp_recvmsg(fd, buf, sizeof(buf),
 				NULL, 0, &sndrcvinfo, &flags);
-			if (n <= 0)
+			if (UNLIKELY(n <= 0))
 				break;
-		} while (keep_stressing(args));
+			if (n >= (ssize_t)sizeof(pid_t)) {
+				const pid_t *pidptr = (pid_t *)buf;
+				const pid_t pid = *pidptr;
+
+				if (UNLIKELY(pid != mypid)) {
+					pr_fail("%s: server received unexpected data "
+						"contents, got 0x%" PRIxMAX ", "
+						"expected 0x%" PRIxMAX "\n",
+						args->name, (intmax_t)pid,
+						(intmax_t)mypid);
+					rc = EXIT_FAILURE;
+					break;
+				}
+			}
+		} while (stress_continue_flag());
 		(void)shutdown(fd, SHUT_RDWR);
 		(void)close(fd);
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 #if defined(AF_UNIX) &&		\
     defined(HAVE_SOCKADDR_UN)
 	if (sctp_domain == AF_UNIX) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
 #else
 	UNEXPECTED
 #endif
-	return EXIT_SUCCESS;
+	return rc;
 }
 
 /*
  *  stress_sctp_server()
  *	server writer
  */
-static int stress_sctp_server(
-	const stress_args_t *args,
-	const pid_t pid,
+static int OPTIMIZE3 stress_sctp_server(
+	stress_args_t *args,
 	const pid_t mypid,
 	const int sctp_port,
 	const int sctp_domain,
-	const int sctp_sched,
+	const int sctp_sched_type,
 	const char *sctp_if)
 {
 	char ALIGN64 buf[SOCKET_BUF];
-	int fd, status;
+	int fd;
 	int so_reuseaddr = 1;
 	socklen_t addr_len = 0;
 	struct sockaddr *addr = NULL;
 	int rc = EXIT_SUCCESS;
+	int idx = 0;
 
-	(void)sctp_sched;
+	(void)sctp_sched_type;
 
 	if (stress_sig_stop_stressing(args->name, SIGALRM)) {
 		rc = EXIT_FAILURE;
 		goto die;
 	}
-	if ((fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP)) < 0) {
+	fd = socket(sctp_domain, SOCK_STREAM, IPPROTO_SCTP);
+	if (fd < 0) {
 		if (errno == EPROTONOSUPPORT) {
 			if (args->instance == 0)
 				pr_inf_skip("%s: SCTP protocol not supported, skipping stressor\n",
 					args->name);
-			rc = EXIT_NOT_IMPLEMENTED;
+			rc = EXIT_NO_RESOURCE;
 			goto die;
 		}
 		rc = stress_exit_status(errno);
@@ -567,60 +542,55 @@ static int stress_sctp_server(
 
 #if defined(HAVE_SCTP_SCHED_TYPE) &&	\
     defined(HAVE_SCTP_ASSOC_VALUE)
-	if (sctp_sched > -1) {
+	if (sctp_sched_type > -1) {
 		struct sctp_assoc_value val;
 
-		(void)memset(&val, 0, sizeof(val));
-		val.assoc_value = (uint32_t)sctp_sched;
+		(void)shim_memset(&val, 0, sizeof(val));
+		val.assoc_value = (uint32_t)sctp_sched_type;
 		(void)setsockopt(fd, SOL_SCTP, SCTP_STREAM_SCHEDULER, &val, sizeof(val));
 	}
 #endif
-
 	do {
 		int sfd;
 
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			break;
 
 		sfd = accept(fd, (struct sockaddr *)NULL, NULL);
-		if (sfd >= 0) {
+		if (LIKELY(sfd >= 0)) {
 			size_t i;
+			const int c = stress_ascii32[idx++ & 0x1f];
+			pid_t *pidptr = (pid_t *)buf;
 
-			(void)memset(buf, 'A' + (get_counter(args) % 26), sizeof(buf));
-
+			(void)shim_memset(buf, c, sizeof(buf));
+			*pidptr = mypid;
 			for (i = 16; i < sizeof(buf); i += 16) {
 				ssize_t ret = sctp_sendmsg(sfd, buf, i,
 						NULL, 0, 0, 0,
 						LOCALTIME_STREAM, 0, 0);
-				if (ret < 0) {
+				if (UNLIKELY(ret < 0))
 					break;
-				} else {
-					inc_counter(args);
-				}
+
+				stress_bogo_inc(args);
 			}
 			stress_sctp_sockopts(sfd);
 			(void)close(sfd);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 die_close:
 	(void)close(fd);
 die:
 #if defined(AF_UNIX) &&		\
     defined(HAVE_SOCKADDR_UN)
-	if (addr && sctp_domain == AF_UNIX) {
-		struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
+	if (addr && (sctp_domain == AF_UNIX)) {
+		const struct sockaddr_un *addr_un = (struct sockaddr_un *)addr;
 
 		(void)shim_unlink(addr_un->sun_path);
 	}
 #else
 	UNEXPECTED
 #endif
-	if (pid) {
-		(void)kill(pid, SIGKILL);
-		(void)shim_waitpid(pid, &status, 0);
-	}
-
 	return rc;
 }
 
@@ -635,19 +605,30 @@ static void stress_sctp_sigpipe(int signum)
  *  stress_sctp
  *	stress SCTP by heavy SCTP network I/O
  */
-static int stress_sctp(const stress_args_t *args)
+static int stress_sctp(stress_args_t *args)
 {
 	pid_t pid, mypid = getpid();
 	int sctp_port = DEFAULT_SCTP_PORT;
 	int sctp_domain = AF_INET;
-	int sctp_sched = -1;	/* Undefined */
-	int ret, reserved_port;
+	int sctp_sched_type = -1; /* undefined */
+	size_t sctp_sched = 1; /* default to fcfs */
+	int ret, reserved_port, parent_cpu;
 	char *sctp_if = NULL;
 
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
 	(void)stress_get_setting("sctp-domain", &sctp_domain);
 	(void)stress_get_setting("sctp-if", &sctp_if);
 	(void)stress_get_setting("sctp-port", &sctp_port);
-	(void)stress_get_setting("sctp-sched", &sctp_sched);
+	if (stress_get_setting("sctp-sched", &sctp_sched)) {
+#if defined(HAVE_SCTP_SCHED_TYPE) &&	\
+    defined(HAVE_SCTP_ASSOC_VALUE)
+		sctp_sched_type = stress_sctp_scheds[sctp_sched].sched_type;
+#else
+		pr_inf("sctp-sched option ignored, no SCTP scheduler types available\n");
+#endif
+	}
 
 	if (sctp_if) {
 		struct sockaddr if_addr;
@@ -664,6 +645,8 @@ static int stress_sctp(const stress_args_t *args)
 		return EXIT_FAILURE;
 
 	sctp_port += args->instance;
+	if (sctp_port > MAX_PORT)
+		sctp_port -= (MAX_PORT - MIN_PORT + 1);
 	reserved_port = stress_net_reserve_ports(sctp_port, sctp_port);
 	if (reserved_port < 0) {
 		pr_inf_skip("%s: cannot reserve port %d, skipping stressor\n",
@@ -676,24 +659,35 @@ static int stress_sctp(const stress_args_t *args)
 		args->name, (intmax_t)args->pid, sctp_port);
 
 	ret = EXIT_FAILURE;
+
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 again:
+	parent_cpu = stress_get_cpu();
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
-		if (!keep_stressing(args))
+		if (UNLIKELY(!stress_continue(args)))
 			goto finish;
 		pr_fail("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	} else if (pid == 0) {
-		ret = stress_sctp_client(args, mypid, sctp_port, sctp_domain, sctp_sched, sctp_if);
-		/* Inform parent we're all done */
-		(void)kill(getppid(), SIGALRM);
+		 (void)stress_change_cpu(args, parent_cpu);
+		ret = stress_sctp_client(args, mypid, sctp_port, sctp_domain, sctp_sched_type, sctp_if);
 		_exit(ret);
 	} else {
-		ret = stress_sctp_server(args, pid, mypid, sctp_port, sctp_domain, sctp_sched, sctp_if);
+		int status;
+
+		ret = stress_sctp_server(args, mypid, sctp_port, sctp_domain, sctp_sched_type, sctp_if);
+		(void)stress_kill_pid_wait(pid, &status);
+		if (WIFEXITED(status)) {
+			if (WEXITSTATUS(status) != EXIT_SUCCESS) {
+				ret = WEXITSTATUS(status);
+			}
+		}
 	}
 
 finish:
@@ -706,17 +700,19 @@ finish:
 	return ret;
 }
 
-stressor_info_t stress_sctp_info = {
+const stressor_info_t stress_sctp_info = {
 	.stressor = stress_sctp,
 	.class = CLASS_NETWORK,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_sctp_info = {
+const stressor_info_t stress_sctp_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_NETWORK,
-	.opt_set_funcs = opt_set_funcs,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without netinet/sctp.h or libsctp support"
 };
