@@ -175,7 +175,7 @@ static int stress_seccomp_supported(const char *name)
 	pid = fork();
 	if (pid < 0) {
 		pr_inf_skip("%s stressor will be skipped, the check for seccomp "
-			"failed, fork failed: errno=%d (%s)\n",
+			"failed, fork failed, errno=%d (%s)\n",
 			name, errno, strerror(errno));
 		return -1;
 	}
@@ -187,7 +187,7 @@ static int stress_seccomp_supported(const char *name)
 	}
 	if (shim_waitpid(pid, &status, 0) < 0) {
 		pr_inf_skip("%s stressor will be skipped, the check for seccomp "
-			"failed, wait failed: errno=%d (%s)\n",
+			"failed, wait failed, errno=%d (%s)\n",
 			name, errno, strerror(errno));
 		return -1;
 	}
@@ -434,8 +434,8 @@ static int stress_seccomp(stress_args_t *args)
 			/* Wait for child to exit or get killed by seccomp */
 			if (shim_waitpid(pid, &status, 0) < 0) {
 				if (errno != EINTR)
-					pr_dbg("%s: waitpid failed, errno=%d (%s)\n",
-						args->name, errno, strerror(errno));
+					pr_dbg("%s: waitpid() on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+						args->name, (intmax_t)pid, errno, strerror(errno));
 			} else {
 				/* Did the child hit a weird error? */
 				if (WIFEXITED(status) &&
