@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 #if defined(HAVE_SEM_SYSV)
 #include <sys/sem.h>
@@ -35,42 +37,38 @@ static const stress_help_t help[] = {
 	{ NULL,	"sem-sysv N",		"start N workers doing System V semaphore operations" },
 	{ NULL,	"sem-sysv-ops N",	"stop after N System V sem bogo operations" },
 	{ NULL,	"sem-sysv-procs N",	"number of processes to start per worker" },
+	{ NULL, "sem-sysv-setall",	"exercise semctl SETALL (will alter the processes' semaphore set)"},
 	{ NULL,	NULL,			NULL }
 };
 
-#if defined(HAVE_SEM_SYSV)
+#if defined(HAVE_SEM_SYSV) &&	\
+    defined(HAVE_KEY_T)
 typedef union stress_semun {
 	int              val;	/* Value for SETVAL */
 	struct semid_ds *buf;	/* Buffer for IPC_STAT, IPC_SET */		/* cppcheck-suppress unusedStructMember */
-	unsigned short  *array;	/* Array for GETALL, SETALL */			/* cppcheck-suppress unusedStructMember */
+	unsigned short int *array;	/* Array for GETALL, SETALL */		/* cppcheck-suppress unusedStructMember */
 	struct seminfo  *__buf;	/* Buffer for IPC_INFO (Linux-specific) */	/* cppcheck-suppress unusedStructMember */
 } stress_semun_t;
 #endif
 
-static int stress_set_semaphore_sysv_procs(const char *opt)
-{
-	uint64_t semaphore_sysv_procs;
-
-	semaphore_sysv_procs = stress_get_uint64(opt);
-	stress_check_range("sem-sysv-procs", semaphore_sysv_procs,
-		MIN_SEM_SYSV_PROCS, MAX_SEM_SYSV_PROCS);
-	return stress_set_setting("sem-sysv-procs", TYPE_ID_UINT64, &semaphore_sysv_procs);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_sem_sysv_procs,	stress_set_semaphore_sysv_procs },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_sem_sysv_procs,  "sem-sysv-procs",  TYPE_ID_UINT64, MIN_SEM_SYSV_PROCS, MAX_SEM_SYSV_PROCS, NULL },
+	{ OPT_sem_sysv_setall, "sem-sysv-setall", TYPE_ID_BOOL, 0, 1, 0 },
+	END_OPT,
 };
 
-#if defined(HAVE_SEM_SYSV)
+#if defined(HAVE_SEM_SYSV) &&	\
+    defined(HAVE_KEY_T)
 /*
  *  stress_semaphore_sysv_init()
  *	initialise a System V semaphore
  */
-static void stress_semaphore_sysv_init(void)
+static void stress_semaphore_sysv_init(const uint32_t instances)
 {
 	int count = 0, sem_id;
 
+	(void)instances;
+
 	/* Exercise invalid nsems, EINVAL */
 	sem_id = semget((key_t)stress_mwc16(), -1, IPC_CREAT | S_IRUSR | S_IWUSR);
 	if (sem_id != -1)
@@ -92,7 +90,8 @@ static void stress_semaphore_sysv_init(void)
 		(void)semctl(sem_id, 0, IPC_RMID);
 
 	while (count < 100) {
-		g_shared->sem_sysv.key_id = (key_t)stress_mwc16();
+		/* use odd key so it won't clash with even core-resource sem keys */
+		g_shared->sem_sysv.key_id = (key_t)stress_mwc16() | 1;
 		g_shared->sem_sysv.sem_id =
 			semget(g_shared->sem_sysv.key_id, 3,
 				IPC_CREAT | S_IRUSR | S_IWUSR);
@@ -113,16 +112,6 @@ static void stress_semaphore_sysv_init(void)
 		/* Clean up */
 		(void)semctl(g_shared->sem_sysv.sem_id, 0, IPC_RMID);
 	}
-
-	if (g_opt_sequential) {
-		pr_inf_skip("semaphore init (System V) failed: errno=%d: "
-			"(%s), skipping semaphore stressor\n",
-			errno, strerror(errno));
-	} else {
-		pr_fail("semaphore init (System V) failed: errno=%d: "
-			"(%s)\n", errno, strerror(errno));
-		_exit(EXIT_FAILURE);
-	}
 }
 
 /*
@@ -155,7 +144,7 @@ static void stress_semaphore_sysv_get_procinfo(bool *get_procinfo)
 		char buffer[1024];
 
 		ret = read(fd, buffer, sizeof(buffer));
-		if (ret <= 0)
+		if (UNLIKELY(ret <= 0))
 			break;
 	}
 	(void)close(fd);
@@ -166,7 +155,9 @@ static void stress_semaphore_sysv_get_procinfo(bool *get_procinfo)
  *  stress_semaphore_sysv_thrash()
  *	exercise the semaphore
  */
-static int stress_semaphore_sysv_thrash(const stress_args_t *args)
+static int OPTIMIZE3 stress_semaphore_sysv_thrash(
+	stress_args_t *args,
+	const bool semaphore_sysv_setall)
 {
 	const int sem_id = g_shared->sem_sysv.sem_id;
 	int rc = EXIT_SUCCESS;
@@ -181,14 +172,19 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 #endif
 
 #if defined(__linux__)
-		if (get_procinfo)
-			stress_semaphore_sysv_get_procinfo(&get_procinfo);
+		/* periodically get proc info */
+		if (get_procinfo) {
+			static uint8_t procinfo_count = 0;
+
+			if (procinfo_count++ == 0)
+				stress_semaphore_sysv_get_procinfo(&get_procinfo);
+		}
 #endif
 
 #if defined(HAVE_SEMTIMEDOP)
 		if (got_semtimedop) {
-			struct timespec timeout;
-			struct sembuf sems[STRESS_MAX_SEMS * 3];
+			struct timespec timeout ALIGN64;
+			struct sembuf sems[STRESS_MAX_SEMS * 3] ALIGN64;
 
 			for (i = 0; i < STRESS_MAX_SEMS * 3; i += 3) {
 				sems[i].sem_num = 1;
@@ -210,7 +206,6 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 #else
 		UNEXPECTED
 #endif
-
 		for (i = 0; i < 1000; i++) {
 			struct sembuf semwait, semsignal;
 
@@ -224,12 +219,12 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 
 #if defined(HAVE_SEMTIMEDOP)
 			if (got_semtimedop) {
-				struct timespec timeout;
+				struct timespec timeout ALIGN64;
 
 				timeout.tv_sec = 1;
 				timeout.tv_nsec = 0;
 				ret = semtimedop(sem_id, &semwait, 1, &timeout);
-				if (ret < 0 && ((errno == ENOSYS) || (errno == EINVAL))) {
+				if (UNLIKELY((ret < 0) && ((errno == ENOSYS) || (errno == EINVAL)))) {
 					got_semtimedop = false;
 					ret = semop(sem_id, &semwait, 1);
 				}
@@ -239,7 +234,7 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 #else
 			ret = semop(sem_id, &semwait, 1);
 #endif
-			if (ret < 0) {
+			if (UNLIKELY(ret < 0)) {
 				if (errno == EAGAIN)
 					goto timed_out;
 				if (errno != EINTR) {
@@ -249,7 +244,7 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 				}
 				break;
 			}
-			if (semop(sem_id, &semsignal, 1) < 0) {
+			if (UNLIKELY(semop(sem_id, &semsignal, 1) < 0)) {
 				if (errno != EINTR) {
 					pr_fail("%s: semop signal failed, errno=%d (%s)\n",
 						args->name, errno, strerror(errno));
@@ -258,20 +253,22 @@ static int stress_semaphore_sysv_thrash(const stress_args_t *args)
 				break;
 			}
 timed_out:
-			if (!keep_stressing(args))
+			stress_bogo_inc(args);
+			if (UNLIKELY(!stress_continue(args)))
 				break;
-			inc_counter(args);
 		}
 #if defined(IPC_STAT)
 		{
 			struct semid_ds ds;
 			stress_semun_t s;
+#if defined(GETALL)
 			size_t nsems;
+#endif
 
-			(void)memset(&ds, 0, sizeof(ds));
+			(void)shim_memset(&ds, 0, sizeof(ds));
 
 			s.buf = &ds;
-			if (semctl(sem_id, 2, IPC_STAT, &s) < 0) {
+			if (UNLIKELY(semctl(sem_id, 2, IPC_STAT, &s) < 0)) {
 				pr_fail("%s: semctl IPC_STAT failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -283,23 +280,22 @@ timed_out:
 				UNEXPECTED
 #endif
 			}
-
 #if defined(GETALL)
 			/* Avoid zero array size allocation */
 			nsems = ds.sem_nsems;
 			if (nsems < 64)
 				nsems = 64;
-			s.array = calloc(nsems, sizeof(*s.array));
+			s.array = (unsigned short int *)calloc(nsems, sizeof(*s.array));
 			if (s.array) {
 				VOID_RET(int, semctl(sem_id, 2, GETALL, s));
-#if defined(SETALL) && 0
+#if defined(SETALL)
 				/*
 				 *  SETALL across the semaphores will clobber the state
 				 *  and cause waits on semaphores because of the unlocked
 				 *  state change. Currently this is disabled.
 				 */
-				if (ret == 0)
-					ret = semctl(sem_id, 2, SETALL, s);
+				if ((ret == 0) && (semaphore_sysv_setall))
+					VOID_RET(int, semctl(sem_id, 2, SETALL, s));
 #endif
 				free(s.array);
 			}
@@ -345,7 +341,7 @@ timed_out:
 			stress_semun_t s;
 
 			s.__buf = &si;
-			if (semctl(sem_id, 0, IPC_INFO, &s) < 0) {
+			if (UNLIKELY(semctl(sem_id, 0, IPC_INFO, &s) < 0)) {
 				pr_fail("%s: semctl IPC_INFO failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -373,7 +369,7 @@ timed_out:
 			stress_semun_t s;
 
 			s.__buf = &si;
-			if (semctl(sem_id, 0, SEM_INFO, &s) < 0) {
+			if (UNLIKELY(semctl(sem_id, 0, SEM_INFO, &s) < 0)) {
 				pr_fail("%s: semctl SEM_INFO failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
 				rc = EXIT_FAILURE;
@@ -388,6 +384,8 @@ timed_out:
 			VOID_RET(int, semctl(sem_id, ~0, IPC_INFO, &s));
 		}
 #endif
+		if (UNLIKELY(!stress_continue(args)))
+			break;
 #if defined(GETVAL)
 		VOID_RET(int, semctl(sem_id, 0, GETVAL, 0));
 
@@ -464,6 +462,8 @@ timed_out:
 			timeout.tv_sec = 0;
 			timeout.tv_nsec = 10000;
 			VOID_RET(int, semtimedop(sem_id, &semwait, (size_t)-1, &timeout));
+			if (UNLIKELY(!stress_continue(args)))
+				break;
 		}
 #else
 		UNEXPECTED
@@ -491,7 +491,7 @@ timed_out:
 		{
 			struct sembuf semwait;
 
-			semwait.sem_num = (unsigned short)-1;
+			semwait.sem_num = (unsigned short int)-1;
 			semwait.sem_op = -1;
 			semwait.sem_flg = SEM_UNDO;
 
@@ -508,10 +508,7 @@ timed_out:
     defined(HAVE_SYSCALL)
 		(void)syscall(__NR_semctl, sem_id, 0, INT_MAX, 0);
 #endif
-	} while ((rc == EXIT_SUCCESS)  && keep_stressing(args));
-
-	if (rc == EXIT_FAILURE)
-		(void)kill(getppid(), SIGALRM);
+	} while ((rc == EXIT_SUCCESS) && stress_continue(args));
 
 	return rc;
 }
@@ -520,36 +517,51 @@ timed_out:
  *  semaphore_sysv_spawn()
  *	spawn a process
  */
-static pid_t semaphore_sysv_spawn(const stress_args_t *args)
+static pid_t semaphore_sysv_spawn(
+	stress_args_t *args,
+	stress_pid_t **s_pids_head,
+	stress_pid_t *s_pid,
+	const bool semaphore_sysv_setall)
 {
-	pid_t pid;
-
 again:
-	pid = fork();
-	if (pid < 0) {
-		if (stress_redo_fork(errno))
+	s_pid->pid = fork();
+	if (s_pid->pid < 0) {
+		if (stress_redo_fork(args, errno))
 			goto again;
 		return -1;
-	}
-	if (pid == 0) {
+	} else if (s_pid->pid == 0) {
+		s_pid->pid = getpid();
+		stress_sync_start_wait_s_pid(s_pid);
+
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
 
-		_exit(stress_semaphore_sysv_thrash(args));
+		_exit(stress_semaphore_sysv_thrash(args, semaphore_sysv_setall));
+	} else {
+		stress_sync_start_s_pid_list_add(s_pids_head, s_pid);
 	}
-	return pid;
+	return s_pid->pid;
 }
 
 /*
  *  stress_sem_sysv()
  *	stress system by sem ops
  */
-static int stress_sem_sysv(const stress_args_t *args)
+static int stress_sem_sysv(stress_args_t *args)
 {
-	pid_t pids[MAX_SEM_SYSV_PROCS];
+	stress_pid_t *s_pids, *s_pids_head = NULL;
 	uint64_t i;
 	uint64_t semaphore_sysv_procs = DEFAULT_SEM_SYSV_PROCS;
 	int rc = EXIT_SUCCESS;
+	bool semaphore_sysv_setall = false;
+
+	if (stress_sigchld_set_handler(args) < 0)
+		return EXIT_NO_RESOURCE;
+
+	if (!g_shared->sem_sysv.init) {
+		pr_inf_skip("%s: skipping stressor, semaphore not initialised\n", args->name);
+		return EXIT_NO_RESOURCE;
+	}
 
 	if (!stress_get_setting("sem-sysv-procs", &semaphore_sysv_procs)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
@@ -557,63 +569,57 @@ static int stress_sem_sysv(const stress_args_t *args)
 		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
 			semaphore_sysv_procs = MIN_SEM_SYSV_PROCS;
 	}
-
-	if (!g_shared->sem_sysv.init) {
-		pr_err("%s: aborting, semaphore not initialised\n", args->name);
-		return EXIT_FAILURE;
-	}
+	(void)stress_get_setting("sem-sysv-setall", &semaphore_sysv_setall);
 
 	if (stress_sighandler(args->name, SIGCHLD, stress_sighandler_nop, NULL) < 0)
 		return EXIT_NO_RESOURCE;
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+	s_pids = stress_sync_s_pids_mmap(MAX_SEM_SYSV_PROCS);
+	if (s_pids == MAP_FAILED) {
+		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, MAX_SEM_SYSV_PROCS);
+		return EXIT_NO_RESOURCE;
+	}
 
-	(void)memset(pids, 0, sizeof(pids));
 	for (i = 0; i < semaphore_sysv_procs; i++) {
-		pids[i] = semaphore_sysv_spawn(args);
-		if (!keep_stressing_flag() || pids[i] < 0)
+		stress_sync_start_init(&s_pids[i]);
+
+		if (semaphore_sysv_spawn(args, &s_pids_head, &s_pids[i], semaphore_sysv_setall) < 0)
+			goto reap;
+		if (UNLIKELY(!stress_continue_flag()))
 			goto reap;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_sync_start_cont_list(s_pids_head);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	/* Wait for termination */
-	while (keep_stressing(args))
+	while (stress_continue(args))
 		pause();
 reap:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-
-	for (i = 0; i < semaphore_sysv_procs; i++) {
-		if (pids[i] > 0)
-			(void)stress_killpid(pids[i]);
-	}
-	for (i = 0; i < semaphore_sysv_procs; i++) {
-		if (pids[i] > 0) {
-			int status;
-
-			(void)shim_waitpid(pids[i], &status, 0);
-			if (WIFEXITED(status) &&
-			    (WEXITSTATUS(status) != EXIT_SUCCESS)) {
-				rc = EXIT_FAILURE;
-			}
-		}
-	}
+	stress_kill_and_wait_many(args, s_pids, semaphore_sysv_procs, SIGALRM, true);
+	stress_sync_s_pids_munmap(s_pids, MAX_SEM_SYSV_PROCS);
 
 	return rc;
 }
 
-stressor_info_t stress_sem_sysv_info = {
+const stressor_info_t stress_sem_sysv_info = {
 	.stressor = stress_sem_sysv,
 	.init = stress_semaphore_sysv_init,
 	.deinit = stress_semaphore_sysv_deinit,
-	.class = CLASS_OS | CLASS_SCHEDULER,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_sem_sysv_info = {
+const stressor_info_t stress_sem_sysv_info = {
 	.stressor = stress_unimplemented,
-	.class = CLASS_OS | CLASS_SCHEDULER,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_OS | CLASS_SCHEDULER | CLASS_IPC,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/sem.h"
 };
