@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,7 +18,12 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-capabilities.h"
+#include "core-madvise.h"
+#include "core-out-of-memory.h"
+
+#include <sys/ioctl.h>
 
 #if defined(__sun__)
 /* Disable for SunOs/Solaris because */
@@ -28,13 +33,14 @@
 #include <sys/swap.h>
 #endif
 
-#define SHIM_EXT2_IOC_GETFLAGS		_IOR('f', 1, long)
-#define SHIM_EXT2_IOC_SETFLAGS		_IOW('f', 2, long)
+#define SHIM_EXT2_IOC_GETFLAGS		_IOR('f', 1, long int)
+#define SHIM_EXT2_IOC_SETFLAGS		_IOW('f', 2, long int)
 #define SHIM_FS_NOCOW_FL		0x00800000 /* No Copy-on-Write file */
 
 static const stress_help_t help[] = {
 	{ NULL,	"swap N",	"start N workers exercising swapon/swapoff" },
 	{ NULL,	"swap-ops N",	"stop after N swapon/swapoff operations" },
+	{ NULL,	"swap-self",	"attempt to swap stressors pages out" },
 	{ NULL,	NULL,		NULL }
 };
 
@@ -98,29 +104,76 @@ static int stress_swap_supported(const char *name)
 	return 0;
 }
 
+#endif
+
+static const stress_opt_t opts[] = {
+	{ OPT_swap_self, "swap-self", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
+};
+
+#if defined(HAVE_SYS_SWAP_H) &&	\
+    defined(HAVE_SWAP)
+
+#if defined(MADV_PAGEOUT) &&	\
+    defined(__linux__)
+
 /*
- *  stress_swapoff()
- *	swapoff and retry if EINTR occurs
+ *  stress_swap_self()
+ *	swap pages out of process
  */
-static int stress_swapoff(const char *path)
+static void stress_swap_self(const size_t page_size)
 {
-	int i;
+	char buffer[4096];
+	FILE *fp;
+	const uintmax_t max_addr = UINTMAX_MAX - (UINTMAX_MAX >> 1);
 
-	for (i = 0; i < 25; i++) {
-		int ret;
+	fp = fopen("/proc/self/maps", "r");
+	if (!fp)
+		return;
 
-		errno = 0;
-		ret = swapoff(path);
-		if (ret == 0)
-			return ret;
-		if ((ret < 0) && (errno != EINTR))
-			break;
+	/*
+	 * Look for field 0060b000-0060c000 r--p 0000b000 08:01 1901726
+	 */
+	while (fgets(buffer, sizeof(buffer), fp)) {
+		uint64_t begin, end, len, offset;
+		char tmppath[1024];
+		char prot[6];
+
+		tmppath[0] = '\0';
+		if (sscanf(buffer, "%" SCNx64 "-%" SCNx64
+		           " %5s %" SCNx64 " %*x:%*x %*d %1023s", &begin, &end, prot, &offset, tmppath) != 5) {
+			continue;
+		}
+#if 0
+		if ((prot[2] != 'x') && (prot[1] != 'w'))
+			continue;
+		if (tmppath[0] == '\0')
+			continue;
+#endif
+		/* Avoid VDSO */
+		if (strncmp("[v", tmppath, 2) == 0)
+			continue;
+
+		if ((begin > UINTPTR_MAX) || (end > UINTPTR_MAX))
+			continue;
+
+		/* Ignore bad ranges */
+		if ((begin >= end) || (begin == 0) || (end == 0) || (end >= max_addr))
+			continue;
+
+		len = end - begin;
+		/* Skip invalid ranges */
+		if ((len < page_size) || (len > 0x80000000UL))
+			continue;
+
+		(void)madvise((void *)(uintptr_t)begin, len, MADV_PAGEOUT);
 	}
-	return -1;
+	(void)fclose(fp);
 }
+#endif
 
-static int stress_swap_zero(
-	const stress_args_t *args,
+static int32_t stress_swap_zero(
+	stress_args_t *args,
 	const int fd,
 	const uint32_t npages,
 	const uint8_t *page)
@@ -135,16 +188,21 @@ static int stress_swap_zero(
 
 	for (i = 0; i < npages; i++) {
 		if (write(fd, page, args->page_size) < 0) {
+			if (errno == ENOSPC) {
+				pr_inf("%s: out of free space creating swap "
+					"file, skipping stressor\n", args->name);
+				return (int32_t)i;
+			}
 			pr_fail("%s: write failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
 			return -1;
 		}
 	}
-	return 0;
+	return (int32_t)i;
 }
 
 static int stress_swap_set_size(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const int fd,
 	const uint32_t npages,
 	const int bad_flags)
@@ -166,7 +224,7 @@ static int stress_swap_set_size(
 	if (bad_flags & SWAP_HDR_BAD_SIGNATURE)
 		signature[0]++;	/* Invalid */
 
-	(void)memset(&swap_info, 0, sizeof(swap_info));
+	(void)shim_memset(&swap_info, 0, sizeof(swap_info));
 	for (i = 0; i < sizeof(swap_info.sws_uuid); i++)
 		swap_info.sws_uuid[i] = stress_mwc8();
 	(void)snprintf((char *)swap_info.sws_volume,
@@ -207,55 +265,96 @@ static int stress_swap_set_size(
 	return 0;
 }
 
-static void stress_swap_check_swapped(
-	void *addr,
-	const size_t page_size,
-	const uint32_t npages,
-	uint64_t *swapped_out,
-	uint64_t *swapped_total)
+static void stress_swap_check_swapped(uint64_t *swapped_out)
 {
-	unsigned char *vec;
-	register size_t n = 0;
+	FILE *fp;
+	char buf[4096];
+	uint64_t swapout = 0;
+	static uint64_t prev_swapout = 0;
 
-	*swapped_total += npages;
-
-	vec = calloc((size_t)npages, sizeof(*vec));
-	if (!vec)
+	fp = fopen("/proc/vmstat", "r");
+	if (!fp)
 		return;
 
-	if (shim_mincore(addr, page_size * (size_t)npages, vec) == 0) {
-		register uint32_t i;
+	while (fgets(buf, sizeof(buf), fp) != NULL) {
+		if (strncmp(buf, "pswpout", 7) == 0) {
+			swapout = (uint64_t)atoll(buf + 8);
+			break;
+		}
+	}
+	(void)fclose(fp);
 
-		for (i = 0; i < npages; i++)
-			n += ((vec[i] & 1) == 0);
+	if (prev_swapout == 0) {
+		prev_swapout = swapout;
+		return;
 	}
 
-	*swapped_out += n;
-	free(vec);
+	*swapped_out += swapout - prev_swapout;
+	prev_swapout = swapout;
+}
+
+static void stress_swap_clean_dir(stress_args_t *args)
+{
+	char path[PATH_MAX];
+	DIR *dir;
+	const struct dirent *d;
+
+	stress_temp_dir(path, sizeof(path), args->name,
+		args->pid, args->instance);
+	dir = opendir(path);
+	if (!dir)
+		return;
+
+	while ((d = readdir(dir)) != NULL) {
+		struct stat stat_buf;
+		char filename[PATH_MAX];
+
+		stress_mk_filename(filename, sizeof(filename), path, d->d_name);
+		if (shim_stat(filename, &stat_buf) == 0) {
+			if (S_ISREG(stat_buf.st_mode)) {
+				(void)stress_swapoff(filename);
+				(void)unlink(filename);
+			}
+		}
+	}
+	(void)closedir(dir);
+	(void)rmdir(path);
 }
 
 /*
- *  stress_swap()
+ *  stress_swap_child()
  *	stress swap operations
  */
-static int stress_swap(const stress_args_t *args)
+static int stress_swap_child(stress_args_t *args, void *context)
 {
 	char filename[PATH_MAX];
 	int fd, ret;
 	uint8_t *page;
 	uint64_t swapped_out = 0;
-	uint64_t swapped_total = 0;
+	int32_t max_swap_pages;
 	const size_t page_size = args->page_size;
-	double swapped_percent;
+	bool swap_self = false;
+	double t, duration, rate;
 
-	page = calloc(1, page_size);
-	if (!page) {
-		pr_inf_skip("%s: failed to allocate 1 page: errno=%d (%s), skipping stressor\n",
+	(void)context;
+
+	if (!stress_get_setting("swap-self", &swap_self)) {
+		if (g_opt_flags & OPT_FLAGS_AGGRESSIVE)
+			swap_self = true;
+	}
+
+	page = mmap(NULL, page_size, PROT_READ | PROT_WRITE,
+			MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+	if (page == MAP_FAILED) {
+		pr_inf_skip("%s: failed to allocate 1 page, errno=%d (%s), skipping stressor\n",
 			args->name, errno, strerror(errno));
 		ret = EXIT_NO_RESOURCE;
 		goto tidy_ret;
 	}
+	stress_set_vma_anon_name(page, page_size, "swap-page");
+	(void)stress_madvise_mergeable(page, page_size);
 
+	stress_swap_clean_dir(args);
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
 		ret = stress_exit_status(-ret);
@@ -274,7 +373,7 @@ static int stress_swap(const stress_args_t *args)
 
 #if defined(__linux__)
 	{
-		unsigned long flags;
+		unsigned long int flags;
 
 		/*
 		 *  Disable Copy-on-Write on file where possible, since
@@ -288,18 +387,25 @@ static int stress_swap(const stress_args_t *args)
 	}
 #endif
 
-	if (stress_swap_zero(args, fd, MAX_SWAP_PAGES, page) < 0) {
+	max_swap_pages = stress_swap_zero(args, fd, MAX_SWAP_PAGES, page);
+	if (max_swap_pages < 0) {
 		ret = EXIT_FAILURE;
 		goto tidy_close;
+	} else if (max_swap_pages < MAX_SWAP_PAGES) {
+		ret = EXIT_NO_RESOURCE;
+		goto tidy_close;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
+	t = stress_time_now();
 	do {
 		int swapflags = 0;
 		int bad_flags;
 		char *ptr;
-		uint32_t npages = (stress_mwc32() % (MAX_SWAP_PAGES - MIN_SWAP_PAGES)) +
+		uint32_t npages = stress_mwc32modn(MAX_SWAP_PAGES - MIN_SWAP_PAGES) +
 				  MIN_SWAP_PAGES;
 		const size_t mmap_size = (size_t)npages * page_size;
 
@@ -315,7 +421,7 @@ static int stress_swap(const stress_args_t *args)
 #endif
 		/* Periodically create bad swap header */
 		if (stress_mwc8() < 16) {
-			const size_t idx = stress_mwc8() % SIZEOF_ARRAY(bad_header_flags);
+			const size_t idx = stress_mwc8modn(SIZEOF_ARRAY(bad_header_flags));
 			bad_flags = bad_header_flags[idx];
 		} else {
 			bad_flags = SWAP_HDR_SANE;	/* No bad header */
@@ -329,20 +435,30 @@ static int stress_swap(const stress_args_t *args)
 		if ((bad_flags == SWAP_HDR_SANE) && (ret < 0)) {
 			switch (errno) {
 			case EPERM:
-			case EINVAL:
 				/*
 				 * We may hit EPERM if we request
-				 * too many swap files
+				 * too many swap files, so delay, retry to
+				 * keep the pressure up.
 				 */
-				pr_inf_skip("%s: cannot enable swap file on the filesystem, skipping test\n",
-					args->name);
+				if (stress_check_capability(SHIM_CAP_SYS_ADMIN)) {
+					(void)shim_usleep(100000);
+					continue;
+				}
+				pr_inf_skip("%s: cannot enable swap%s, skipping test\n",
+					args->name, stress_get_fs_type(filename));
+				ret = EXIT_NO_RESOURCE;
+				break;
+			case EINVAL:
+				pr_inf_skip("%s: cannot enable swap%s, skipping test\n",
+					args->name, stress_get_fs_type(filename));
 				ret = EXIT_NO_RESOURCE;
 				break;
 			case EBUSY:
 				continue;
 			default:
-				pr_fail("%s: swapon failed, errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_fail("%s: swapon failed%s, errno=%d (%s)\n",
+					args->name, stress_get_fs_type(filename),
+					errno, strerror(errno));
 				ret = EXIT_FAILURE;
 				break;
 			}
@@ -356,36 +472,42 @@ static int stress_swap(const stress_args_t *args)
 			const char *p_end = ptr + mmap_size;
 			char *p;
 
+			(void)shim_madvise(ptr, mmap_size, MADV_WILLNEED);
 			/* Add simple check value to start of each page */
 			for (i = 0, p = ptr; p < p_end; p += page_size, i++) {
 				uintptr_t *up = (uintptr_t *)(uintptr_t)p;
 
-				(void)memset(p, (int)i, page_size);
+				(void)shim_memset(p, (int)i, page_size);
 				*up = (uintptr_t)p;
 			}
 #if defined(MADV_PAGEOUT)
 			(void)shim_madvise(ptr, mmap_size, MADV_PAGEOUT);
 #endif
-			stress_swap_check_swapped(ptr, page_size, npages,
-				&swapped_out, &swapped_total);
+#if defined(MADV_PAGEOUT) &&	\
+    defined(__linux__)
+			if (swap_self)
+				stress_swap_self(args->page_size);
+#endif
+			stress_swap_check_swapped(&swapped_out);
 
 			/* Check page has check address value */
 			for (i = 0, p = ptr; p < p_end; p += page_size, i++) {
-				uintptr_t *up = (uintptr_t *)(uintptr_t)p;
+				const uintptr_t *up = (uintptr_t *)(uintptr_t)p;
 
 				if (*up != (uintptr_t)p) {
-					pr_fail("%s: failed: address %p contains "
+					pr_fail("%s: failed, address %p contains "
 						"%" PRIuPTR " and not %" PRIuPTR "\n",
 						args->name, (void *)p, *up, (uintptr_t)p);
 				}
 			}
-			(void)munmap(ptr, mmap_size);
+			(void)stress_munmap_retry_enomem(ptr, mmap_size);
 		}
 
 		ret = stress_swapoff(filename);
 		if ((bad_flags == SWAP_HDR_SANE) && (ret < 0)) {
-			pr_fail("%s: swapoff failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+			pr_fail("%s: swapoff failed%s, errno=%d (%s)\n",
+				args->name, stress_get_fs_type(filename),
+				errno, strerror(errno));
 			ret = EXIT_FAILURE;
 			goto tidy_close;
 		}
@@ -405,13 +527,12 @@ static int stress_swap(const stress_args_t *args)
 		if (ret == 0)
 			VOID_RET(int, stress_swapoff(filename));/* Should never happen */
 
-		inc_counter(args);
-	} while (keep_stressing(args));
+		stress_bogo_inc(args);
+	} while (stress_continue(args));
 
-	swapped_percent = (swapped_total == 0) ?
-		0.0 : (100.0 * (double)swapped_out) / (double)swapped_total;
-	pr_inf("%s: %" PRIu64 " of %" PRIu64 " (%.2f%%) pages were swapped out\n",
-		args->name, swapped_out, swapped_total, swapped_percent);
+	duration = stress_time_now() - t;
+	rate = (duration > 0.0) ? swapped_out / duration : 0.0;
+	stress_metrics_set(args, 0, "pages swapped out per second", rate, STRESS_METRIC_GEOMETRIC_MEAN);
 
 	ret = EXIT_SUCCESS;
 tidy_close:
@@ -423,23 +544,36 @@ tidy_rm:
 	(void)stress_temp_dir_rm_args(args);
 tidy_free:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	free(page);
+	(void)munmap((void *)page, page_size);
 tidy_ret:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
+	stress_swap_clean_dir(args);
 	return ret;
 }
 
-stressor_info_t stress_swap_info = {
+static int stress_swap(stress_args_t *args)
+{
+	int ret;
+
+	ret = stress_oomable_child(args, NULL, stress_swap_child, STRESS_OOMABLE_NORMAL);
+	stress_swap_clean_dir(args);
+	return ret;
+}
+
+const stressor_info_t stress_swap_info = {
 	.stressor = stress_swap,
 	.supported = stress_swap_supported,
 	.class = CLASS_VM | CLASS_OS,
+	.opts = opts,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_swap_info = {
+const stressor_info_t stress_swap_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_VM | CLASS_OS,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "built without sys/swap.h or swap() system call"
 };
