@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2024 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,12 +18,17 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
 #include "core-cpu-cache.h"
 #include "core-killpid.h"
 #include "core-madvise.h"
 #include "core-out-of-memory.h"
+
+#include <sys/ioctl.h>
 #include <sys/socket.h>
+#include <sys/times.h>
 #include <sched.h>
+#include <time.h>
 
 #if defined(HAVE_TERMIOS_H)
 #include <termios.h>
@@ -301,7 +306,7 @@ static void bad_bind(stress_bad_addr_t *ba, volatile uint64_t *counter)
 static void bad_cacheflush(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	(*counter)++;
-	VOID_RET(int, shim_cacheflush(ba->addr, 4096, SHIM_DCACHE));
+	stress_cpu_data_cache_flush(ba->addr, 4096);
 }
 #endif
 
@@ -571,9 +576,10 @@ static void bad_execve3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unreadable) {
 		char name[PATH_MAX];
-		static char *newargv[] = { NULL, NULL };
 
 		if (stress_get_proc_self_exe(name, sizeof(name)) == 0) {
+			static char *newargv[] = { NULL, NULL };
+
 			(*counter)++;
 			VOID_RET(int, execve(name, newargv, ba->addr));
 		}
@@ -735,7 +741,7 @@ static void bad_getpeername2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 	if (ba->unwriteable) {
 		struct sockaddr saddr;
 
-		(void)memset(&saddr, 0, sizeof(saddr));
+		(void)shim_memset(&saddr, 0, sizeof(saddr));
 		(*counter)++;
 		VOID_RET(int, getpeername(0, &saddr, (socklen_t *)ba->addr));
 	}
@@ -754,8 +760,8 @@ static void bad_getpeername3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 static void bad_get_mempolicy1(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	(*counter)++;
-	VOID_RET(long, shim_get_mempolicy((int *)ba->addr,
-			(unsigned long *)inc_addr(ba->addr, 1), 1,
+	VOID_RET(long int, shim_get_mempolicy((int *)ba->addr,
+			(unsigned long int *)inc_addr(ba->addr, 1), 1,
 			inc_addr(ba->addr, 2), 0UL));
 }
 
@@ -764,15 +770,15 @@ static void bad_get_mempolicy2(stress_bad_addr_t *ba, volatile uint64_t *counter
 	int mode = 0;
 
 	(*counter)++;
-	VOID_RET(long, shim_get_mempolicy(&mode, (unsigned long *)ba->addr, 1, ba->addr, 0UL));
+	VOID_RET(long int, shim_get_mempolicy(&mode, (unsigned long int *)ba->addr, 1, ba->addr, 0UL));
 }
 
 static void bad_get_mempolicy3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
-	unsigned long nodemask = 1;
+	unsigned long int nodemask = 1;
 
 	(*counter)++;
-	VOID_RET(long, shim_get_mempolicy((int *)ba->addr, &nodemask, 1, ba->addr, 0UL));
+	VOID_RET(long int, shim_get_mempolicy((int *)ba->addr, &nodemask, 1, ba->addr, 0UL));
 }
 
 
@@ -898,7 +904,7 @@ static void bad_getsockname2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 	if (ba->unwriteable) {
 		struct sockaddr saddr;
 
-		(void)memset(&ba->addr, 0, sizeof(saddr));
+		(void)shim_memset(&ba->addr, 0, sizeof(saddr));
 		(*counter)++;
 		VOID_RET(int, getsockname(0, &saddr, (socklen_t *)ba->addr));
 	}
@@ -1141,6 +1147,60 @@ static void bad_lremovexattr3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 }
 #endif
 
+#if defined(__NR_lsm_get_self_attr)
+static void bad_lsm_get_self_attr1(stress_bad_addr_t *ba, volatile uint64_t *counter)
+{
+	if (ba->unwriteable) {
+		size_t size = 1024;
+
+		(*counter)++;
+		VOID_RET(int, syscall(__NR_lsm_get_self_attr, 0, ba->addr, &size, 0));
+	}
+}
+
+static void bad_lsm_get_self_attr2(stress_bad_addr_t *ba, volatile uint64_t *counter)
+{
+	if (ba->unwriteable) {
+		unsigned char ctxt[1024];
+
+		(*counter)++;
+		VOID_RET(int, syscall(__NR_lsm_get_self_attr, 0, ctxt, ba->addr, 0));
+	}
+}
+#endif
+
+#if defined(__NR_lsm_set_self_attr)
+static void bad_lsm_set_self_attr(stress_bad_addr_t *ba, volatile uint64_t *counter)
+{
+	if (ba->unreadable) {
+		(*counter)++;
+		VOID_RET(int, syscall(__NR_lsm_set_self_attr, 0, ba->addr, 1024, 0));
+	}
+}
+#endif
+
+#if defined(__NR_lsm_list_modules)
+static void bad_lsm_list_modules1(stress_bad_addr_t *ba, volatile uint64_t *counter)
+{
+	if (ba->unreadable) {
+		size_t size = 64;
+
+		(*counter)++;
+		VOID_RET(int, syscall(__NR_lsm_list_modules, (uint64_t *)ba->addr, &size, 0));
+	}
+}
+
+static void bad_lsm_list_modules2(stress_bad_addr_t *ba, volatile uint64_t *counter)
+{
+	if (ba->unreadable) {
+		uint64_t ids[64];
+
+		(*counter)++;
+		VOID_RET(int, syscall(__NR_lsm_list_modules, ids, (size_t *)ba->addr, 0));
+	}
+}
+#endif
+
 static void bad_lstat1(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	(*counter)++;
@@ -1169,7 +1229,7 @@ static void bad_lstat3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 static void bad_madvise(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	(*counter)++;
-	VOID_RET(int, madvise((void *)ba->addr, 8192, MADV_NORMAL));
+	VOID_RET(int, shim_madvise((void *)ba->addr, 8192, SHIM_MADV_NORMAL));
 }
 #else
 UNEXPECTED
@@ -1193,28 +1253,28 @@ static void bad_migrate_pages1(stress_bad_addr_t *ba, volatile uint64_t *counter
 {
 	if (ba->unreadable) {
 		(*counter)++;
-		VOID_RET(long, shim_migrate_pages(getpid(), 1, (unsigned long *)ba->addr,
-						(unsigned long *)inc_addr(ba->addr, 1)));
+		VOID_RET(long int, shim_migrate_pages(getpid(), 1, (unsigned long int *)ba->addr,
+						(unsigned long int *)inc_addr(ba->addr, 1)));
 	}
 }
 
 static void bad_migrate_pages2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unreadable) {
-		unsigned long nodes = 0;
+		unsigned long int nodes = 0;
 
 		(*counter)++;
-		VOID_RET(long, shim_migrate_pages(getpid(), 1, &nodes, (unsigned long *)ba->addr));
+		VOID_RET(long int, shim_migrate_pages(getpid(), 1, &nodes, (unsigned long int *)ba->addr));
 	}
 }
 
 static void bad_migrate_pages3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unreadable) {
-		unsigned long nodes = 0;
+		unsigned long int nodes = 0;
 
 		(*counter)++;
-		VOID_RET(long, shim_migrate_pages(getpid(), 1, (unsigned long *)ba->addr, &nodes));
+		VOID_RET(long int, shim_migrate_pages(getpid(), 1, (unsigned long int *)ba->addr, &nodes));
 	}
 }
 
@@ -1248,17 +1308,17 @@ static void bad_mlock2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 static void bad_move_pages1(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	(*counter)++;
-	VOID_RET(long, shim_move_pages(getpid(), (unsigned long)1, (void **)ba->addr,
+	VOID_RET(long int, shim_move_pages(getpid(), (unsigned long int)1, (void **)ba->addr,
 					(const int *)inc_addr(ba->addr, 1), (int *)inc_addr(ba->addr, 2), 0));
 }
 
 static void bad_move_pages2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unreadable) {
-		int nodes, status;
+		int nodes = 0, status;
 
 		(*counter)++;
-		VOID_RET(long, shim_move_pages(getpid(), (unsigned long)1, (void **)ba->addr, &nodes, &status, 0));
+		VOID_RET(long int, shim_move_pages(getpid(), (unsigned long int)1, (void **)ba->addr, &nodes, &status, 0));
 	}
 }
 
@@ -1270,7 +1330,7 @@ static void bad_move_pages3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 
 		pages[0] = ba->addr;
 		(*counter)++;
-		VOID_RET(long, shim_move_pages(getpid(), (unsigned long)1, pages, (int *)ba->addr, &status, 0));
+		VOID_RET(long int, shim_move_pages(getpid(), (unsigned long int)1, pages, (int *)ba->addr, &status, 0));
 	}
 }
 
@@ -1282,11 +1342,19 @@ static void bad_move_pages4(stress_bad_addr_t *ba, volatile uint64_t *counter)
 
 		pages[0] = ba->addr;
 		(*counter)++;
-		VOID_RET(long, shim_move_pages(getpid(), (unsigned long)1, pages, &nodes, (int *)ba->addr, 0));
+		VOID_RET(long int, shim_move_pages(getpid(), (unsigned long int)1, pages, &nodes, (int *)ba->addr, 0));
 	}
 }
 #endif
 
+#if defined(__NR_seal)
+static void bad_mseal(stress_bad_addr_t *ba, volatile uint64_t *counter)
+{
+	(*counter)++;
+	VOID_RET(long int, shim_mseal((void **)ba->addr, 4096, 0));
+}
+#endif
+
 #if defined(HAVE_MLOCK)
 static void bad_munlock(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
@@ -1360,8 +1428,14 @@ static void bad_open(stress_bad_addr_t *ba, volatile uint64_t *counter)
 static void bad_pipe(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	if (ba->unwriteable) {
+		int *fds = (int *)ba->addr;
+
 		(*counter)++;
-		VOID_RET(int, pipe((int *)ba->addr));
+		if (pipe(fds) == 0) {
+			/* Should not get here */
+			(void)close(fds[0]);
+			(void)close(fds[1]);
+		}
 	}
 }
 
@@ -1418,11 +1492,12 @@ static void bad_preadv2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 static void bad_ptrace(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	(*counter)++;
-	VOID_RET(long, ptrace(PTRACE_GETREGS, getpid(), (void *)ba->addr, (void *)inc_addr(ba->addr, 1)));
+	VOID_RET(long int, ptrace(PTRACE_GETREGS, getpid(), (void *)ba->addr, (void *)inc_addr(ba->addr, 1)));
 }
 #endif
 
-#if defined(HAVE_POLL_H)
+#if defined(HAVE_POLL_H) &&	\
+    defined(HAVE_POLL)
 static void bad_poll(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	(*counter)++;
@@ -1437,11 +1512,11 @@ UNEXPECTED
 static void bad_ppoll1(stress_bad_addr_t *ba, volatile uint64_t *counter)
 {
 	void *addr = (char *)ba->addr;
-	struct timespec *ts = (struct timespec *)inc_addr(addr, sizeof(struct pollfd));
-	sigset_t *ss = (sigset_t *)(inc_addr(addr, sizeof(struct pollfd) + sizeof(struct timespec)));
+	const struct timespec *ts = (struct timespec *)inc_addr(addr, sizeof(struct pollfd));
+	const sigset_t *ss = (sigset_t *)(inc_addr(addr, sizeof(struct pollfd) + sizeof(struct timespec)));
 
 	(*counter)++;
-	VOID_RET(int, ppoll((struct pollfd *)addr, (nfds_t)1, ts, ss));
+	VOID_RET(int, shim_ppoll((struct pollfd *)addr, (nfds_t)1, ts, ss));
 }
 
 static void bad_ppoll2(stress_bad_addr_t *ba, volatile uint64_t *counter)
@@ -1454,7 +1529,7 @@ static void bad_ppoll2(stress_bad_addr_t *ba, volatile uint64_t *counter)
 	ts.tv_nsec = 0;
 
 	(*counter)++;
-	VOID_RET(int, ppoll((struct pollfd *)ba->addr, (nfds_t)16, &ts, &sigmask));
+	VOID_RET(int, shim_ppoll((struct pollfd *)ba->addr, (nfds_t)16, &ts, &sigmask));
 }
 
 static void bad_ppoll3(stress_bad_addr_t *ba, volatile uint64_t *counter)
@@ -1469,7 +1544,7 @@ static void bad_ppoll3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 		(void)sigemptyset(&sigmask);
 
 		(*counter)++;
-		VOID_RET(int, ppoll(&pfd, (nfds_t)1, (struct timespec *)ba->addr, &sigmask));
+		VOID_RET(int, shim_ppoll(&pfd, (nfds_t)1, (struct timespec *)ba->addr, &sigmask));
 	}
 }
 
@@ -1486,7 +1561,7 @@ static void bad_ppoll4(stress_bad_addr_t *ba, volatile uint64_t *counter)
 		ts.tv_nsec = 0;
 
 		(*counter)++;
-		VOID_RET(int, ppoll(&pfd, (nfds_t)1, &ts, (sigset_t *)ba->addr));
+		VOID_RET(int, shim_ppoll(&pfd, (nfds_t)1, &ts, (sigset_t *)ba->addr));
 	}
 }
 #endif
@@ -1773,7 +1848,7 @@ static void bad_setitimer3(stress_bad_addr_t *ba, volatile uint64_t *counter)
 	if (ba->unwriteable) {
 		struct itimerval newval;
 
-		(void)memset(&newval, 0, sizeof(newval));
+		(void)shim_memset(&newval, 0, sizeof(newval));
 		(*counter)++;
 		VOID_RET(int, setitimer(ITIMER_PROF, &newval, (struct itimerval *)ba->addr));
 	}
@@ -2145,6 +2220,17 @@ static stress_bad_syscall_t bad_syscalls[] = {
 	bad_lremovexattr1,
 	bad_lremovexattr2,
 	bad_lremovexattr3,
+#endif
+#if defined(__NR_lsm_get_self_attr)
+	bad_lsm_get_self_attr1,
+	bad_lsm_get_self_attr2,
+#endif
+#if defined(__NR_lsm_set_self_attr)
+	bad_lsm_set_self_attr,
+#endif
+#if defined(__NR_lsm_list_modules)
+	bad_lsm_list_modules1,
+	bad_lsm_list_modules2,
 #endif
 	bad_lstat1,
 	bad_lstat2,
@@ -2171,6 +2257,9 @@ static stress_bad_syscall_t bad_syscalls[] = {
 	bad_move_pages3,
 	bad_move_pages4,
 #endif
+#if defined(__NR_seal)
+	bad_mseal,
+#endif
 #if defined(HAVE_MSYNC)
 	bad_msync,
 #endif
@@ -2184,7 +2273,8 @@ static stress_bad_syscall_t bad_syscalls[] = {
 #endif
 	bad_open,
 	bad_pipe,
-#if defined(HAVE_POLL_H)
+#if defined(HAVE_POLL_H) &&	\
+    defined(HAVE_POLL)
 	bad_poll,
 #endif
 #if defined(HAVE_POLL_H) &&	\
@@ -2299,7 +2389,7 @@ static inline int stress_do_syscall(stress_args_t *args)
 	/* force update of 1 bit mwc random val */
 	(void)stress_mwc1();
 
-	if (!stress_continue(args))
+	if (UNLIKELY(!stress_continue(args)))
 		return 0;
 	pid = fork();
 	if (pid < 0) {
@@ -2310,6 +2400,11 @@ static inline int stress_do_syscall(stress_args_t *args)
 #endif
 		size_t k;
 
+		for (k = 0; k < SIZEOF_ARRAY(sigs); k++) {
+			if (stress_sighandler(args->name, sigs[k], stress_sig_handler_exit, NULL) < 0)
+				_exit(EXIT_FAILURE);
+		}
+
 		/* Try to limit child from spawning */
 		limit_procs(2);
 
@@ -2320,10 +2415,6 @@ static inline int stress_do_syscall(stress_args_t *args)
 		if (stress_drop_capabilities(args->name) < 0) {
 			_exit(EXIT_NO_RESOURCE);
 		}
-		for (k = 0; k < SIZEOF_ARRAY(sigs); k++) {
-			if (stress_sighandler(args->name, sigs[k], stress_sig_handler_exit, NULL) < 0)
-				_exit(EXIT_FAILURE);
-		}
 
 		stress_parent_died_alarm();
 		(void)sched_settings_apply(true);
@@ -2359,13 +2450,14 @@ static inline int stress_do_syscall(stress_args_t *args)
 		}
 		_exit(EXIT_SUCCESS);
 	} else {
-		int ret, status;
+		pid_t ret;
+		int status;
 
 		ret = shim_waitpid(pid, &status, 0);
 		if (ret < 0) {
 			if (errno != EINTR)
-				pr_dbg("%s: waitpid(): errno=%d (%s)\n",
-					args->name, errno, strerror(errno));
+				pr_dbg("%s: waitpid() on PID %" PRIdMAX" failed, errno=%d (%s)\n",
+					args->name, (intmax_t)pid, errno, strerror(errno));
 			(void)stress_kill_pid_wait(pid, &status);
 		}
 		rc = WEXITSTATUS(status);
@@ -2387,7 +2479,7 @@ static int stress_sysbadaddr_child(stress_args_t *args, void *context)
 
 			state->addr_index = 0;
 			while (state->addr_index < SIZEOF_ARRAY(bad_addrs)) {
-				void *addr = bad_addrs[state->addr_index].addr;
+				const void *addr = bad_addrs[state->addr_index].addr;
 
 				if (addr)
 					stress_do_syscall(args);
@@ -2431,6 +2523,7 @@ static int stress_sysbadaddr(stress_args_t *args)
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
+	stress_set_vma_anon_name(state, sizeof(*state), "state");
 
 	ro_page = stress_mmap_populate(NULL, page_size,
 		PROT_READ, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
@@ -2441,6 +2534,7 @@ static int stress_sysbadaddr(stress_args_t *args)
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
+	stress_set_vma_anon_name(ro_page, page_size, "ro-page");
 	(void)stress_madvise_mergeable(ro_page, page_size);
 
 	rw_page = stress_mmap_populate(NULL, page_size << 1,
@@ -2453,6 +2547,7 @@ static int stress_sysbadaddr(stress_args_t *args)
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
+	stress_set_vma_anon_name(rw_page, page_size << 1, "rw-page");
 	(void)stress_madvise_mergeable(rw_page, page_size << 1);
 
 	rx_page = stress_mmap_populate(NULL, page_size,
@@ -2465,6 +2560,7 @@ static int stress_sysbadaddr(stress_args_t *args)
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
+	stress_set_vma_anon_name(rx_page, page_size, "rx-page");
 	(void)stress_madvise_mergeable(rx_page, page_size);
 
 	no_page = stress_mmap_populate(NULL, page_size,
@@ -2476,6 +2572,7 @@ static int stress_sysbadaddr(stress_args_t *args)
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
+	stress_set_vma_anon_name(no_page, page_size, "no-page");
 
 	wo_page = stress_mmap_populate(NULL, page_size,
 		PROT_WRITE, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
@@ -2486,6 +2583,7 @@ static int stress_sysbadaddr(stress_args_t *args)
 		ret = EXIT_NO_RESOURCE;
 		goto cleanup;
 	}
+	stress_set_vma_anon_name(wo_page, page_size, "wo-page");
 	(void)stress_madvise_mergeable(wo_page, page_size);
 
 	/*
@@ -2495,6 +2593,8 @@ static int stress_sysbadaddr(stress_args_t *args)
 	 */
 	wx_page = stress_mmap_populate(NULL, page_size,
 		PROT_WRITE | PROT_EXEC, MAP_ANONYMOUS | MAP_SHARED, -1, 0);
+	if (wx_page != MAP_FAILED)
+		stress_set_vma_anon_name(wo_page, page_size, "wo-page");
 	/*
 	 * Unmap last page, so we know we have an unmapped
 	 * page following the r/w page
@@ -2508,6 +2608,8 @@ static int stress_sysbadaddr(stress_args_t *args)
 		bad_addrs[i].addr = bad_addrs[i].func(args);
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	ret = stress_oomable_child(args, NULL, stress_sysbadaddr_child, STRESS_OOMABLE_DROP_CAP);
@@ -2524,7 +2626,7 @@ cleanup:
 	return ret;
 }
 
-stressor_info_t stress_sysbadaddr_info = {
+const stressor_info_t stress_sysbadaddr_info = {
 	.stressor = stress_sysbadaddr,
 	.class = CLASS_OS,
 	.help = help
