@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C)      2022 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,9 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+
+#include <time.h>
 
 #define MIN_TIMER_FREQ		(1)
 #define MAX_TIMER_FREQ		(100000000)
@@ -25,8 +28,8 @@
 
 static const stress_help_t help[] = {
 	{ "T N", "timer N",	"start N workers producing timer events" },
-	{ NULL, "timer-ops N",	"stop after N timer bogo events" },
 	{ NULL, "timer-freq F",	"run timer(s) at F Hz, range 1 to 1000000000" },
+	{ NULL, "timer-ops N",	"stop after N timer bogo events" },
 	{ NULL, "timer-rand",	"enable random timer frequency" },
 	{ NULL, NULL,		NULL }
 };
@@ -36,37 +39,20 @@ static const stress_help_t help[] = {
     defined(HAVE_TIMER_DELETE) &&	\
     defined(HAVE_TIMER_GETOVERRUN) &&	\
     defined(HAVE_TIMER_SETTIME)
-static volatile uint64_t timer_counter = 0;
+static volatile uint64_t timer_counter;
+static volatile uint64_t timer_settime_failure;
+static uint64_t timer_overruns;
 static uint64_t max_ops;
 static timer_t timerid;
-static uint64_t overruns = 0;
 static double rate_ns;
-static double start;
+static double time_end;
+static bool timer_rand;
 #endif
 
-/*
- *  stress_set_timer_freq()
- *	set timer frequency from given option
- */
-static int stress_set_timer_freq(const char *opt)
-{
-	uint64_t timer_freq;
-
-	timer_freq = stress_get_uint64(opt);
-	stress_check_range("timer-freq", timer_freq,
-		MIN_TIMER_FREQ, MAX_TIMER_FREQ);
-	return stress_set_setting("timer-freq", TYPE_ID_UINT64, &timer_freq);
-}
-
-static int stress_set_timer_rand(const char *opt)
-{
-	return stress_set_setting_true("timer-rand", opt);
-}
-
-static const stress_opt_set_func_t opt_set_funcs[] = {
-	{ OPT_timer_freq,	stress_set_timer_freq },
-	{ OPT_timer_rand,	stress_set_timer_rand },
-	{ 0,			NULL }
+static const stress_opt_t opts[] = {
+	{ OPT_timer_freq, "timer-freq", TYPE_ID_UINT64, MIN_TIMER_FREQ, MAX_TIMER_FREQ, NULL },
+	{ OPT_timer_rand, "timer-rand", TYPE_ID_BOOL, 0, 1, NULL },
+	END_OPT,
 };
 
 #if defined(HAVE_LIB_RT) &&		\
@@ -79,16 +65,13 @@ static const stress_opt_set_func_t opt_set_funcs[] = {
  *  stress_timer_set()
  *	set timer, ensure it is never zero
  */
-static void stress_timer_set(struct itimerspec *timer)
+static void OPTIMIZE3 stress_timer_set(struct itimerspec *timer)
 {
 	double rate;
-	bool timer_rand = false;
 
-	(void)stress_get_setting("timer-rand", &timer_rand);
-
-	if (timer_rand) {
+	if (UNLIKELY(timer_rand)) {
 		/* Mix in some random variation */
-		const double r = ((double)(stress_mwc32() % 10000) - 5000.0) / 40000.0;
+		const double r = ((double)stress_mwc32modn(10000) - 5000.0) / 40000.0;
 		rate = rate_ns + (rate_ns * r);
 	} else {
 		rate = rate_ns;
@@ -105,12 +88,12 @@ static void stress_timer_set(struct itimerspec *timer)
 }
 
 /*
- *  stress_timer_keep_stressing(args)
+ *  stress_timer_stress_continue(args)
  *      returns true if we can keep on running a stressor
  */
-static bool HOT OPTIMIZE3 stress_timer_keep_stressing(void)
+static bool OPTIMIZE3 stress_timer_stress_continue(void)
 {
-	return (LIKELY(keep_stressing_flag()) &&
+	return (LIKELY(stress_continue_flag()) &&
 		LIKELY(!max_ops || (timer_counter < max_ops)));
 }
 
@@ -121,9 +104,7 @@ static bool HOT OPTIMIZE3 stress_timer_keep_stressing(void)
 static inline void stress_proc_self_timer_read(void)
 {
 #if defined(__linux__)
-	char buf[1024];
-
-	VOID_RET(ssize_t, system_read("/proc/self/timers", buf, sizeof(buf)));
+	(void)stress_system_discard("/proc/self/timers");
 #endif
 }
 
@@ -131,14 +112,14 @@ static inline void stress_proc_self_timer_read(void)
  *  stress_timer_handler()
  *	catch timer signal and cancel if no more runs flagged
  */
-static void MLOCKED_TEXT stress_timer_handler(int sig)
+static void MLOCKED_TEXT OPTIMIZE3 stress_timer_handler(int sig)
 {
 	struct itimerspec timer;
 	sigset_t mask;
 
 	(void)sig;
 
-	if (!stress_timer_keep_stressing())
+	if (UNLIKELY(!stress_timer_stress_continue()))
 		goto cancel;
 	timer_counter++;
 
@@ -146,47 +127,55 @@ static void MLOCKED_TEXT stress_timer_handler(int sig)
 		if (sigismember(&mask, SIGINT))
 			goto cancel;
 	/* High freq timer, check periodically for timeout */
-	if ((timer_counter & 65535) == 0) {
-		if ((stress_time_now() - start) > (double)g_opt_timeout)
+	if (UNLIKELY((timer_counter & 65535) == 0)) {
+		if (stress_time_now() > time_end)
 			goto cancel;
 		stress_proc_self_timer_read();
 	}
-	if (keep_stressing_flag()) {
+	if (LIKELY(stress_continue_flag())) {
 		const int ret = timer_getoverrun(timerid);
 
 		if (ret > 0)
-			overruns += (uint64_t)ret;
+			timer_overruns += (uint64_t)ret;
 		stress_timer_set(&timer);
-		(void)timer_settime(timerid, 0, &timer, NULL);
+		if (timer_settime(timerid, 0, &timer, NULL) < 0)
+			timer_settime_failure++;
 		return;
 	}
 
 cancel:
-	keep_stressing_set_flag(false);
+	stress_continue_set_flag(false);
 	/* Cancel timer if we detect no more runs */
-	(void)memset(&timer, 0, sizeof(timer));
-	(void)timer_settime(timerid, 0, &timer, NULL);
+	(void)shim_memset(&timer, 0, sizeof(timer));
+	if (UNLIKELY(timer_settime(timerid, 0, &timer, NULL) < 0))
+		timer_settime_failure++;
 }
 
 /*
  *  stress_timer
  *	stress timers
  */
-static int stress_timer(const stress_args_t *args)
+static int stress_timer(stress_args_t *args)
 {
 	struct sigevent sev;
 	struct itimerspec timer;
 	sigset_t mask;
 	uint64_t timer_freq = DEFAULT_TIMER_FREQ;
-	int n = 0;
+	int n = 0, rc = EXIT_SUCCESS;
+
+	time_end = args->time_end;
+	timer_counter = 0;
+	timer_settime_failure = 0;
+	timer_overruns = 0;
 
 	(void)sigemptyset(&mask);
 	(void)sigaddset(&mask, SIGINT);
 	(void)sigprocmask(SIG_SETMASK, &mask, NULL);
 
-	max_ops = args->max_ops;
-	start = stress_time_now();
+	max_ops = args->bogo.max_ops;
 
+	timer_rand = false;
+	(void)stress_get_setting("timer-rand", &timer_rand);
 	if (!stress_get_setting("timer-freq", &timer_freq)) {
 		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
 			timer_freq = MAX_TIMER_FREQ;
@@ -199,16 +188,26 @@ static int stress_timer(const stress_args_t *args)
 	if (stress_sighandler(args->name, SIGRTMIN, stress_timer_handler, NULL) < 0)
 		return EXIT_FAILURE;
 
-	(void)memset(&sev, 0, sizeof(sev));
+	(void)shim_memset(&sev, 0, sizeof(sev));
 	sev.sigev_notify = SIGEV_SIGNAL;
 	sev.sigev_signo = SIGRTMIN;
 	sev.sigev_value.sival_ptr = &timerid;
 	if (timer_create(CLOCK_REALTIME, &sev, &timerid) < 0) {
+		if ((errno == EAGAIN) || (errno == ENOMEM) || (errno == ENOTSUP)) {
+			pr_inf_skip("%s: could not create timer, out of "
+				"resources, skipping stressor\n",
+				args->name);
+			return EXIT_NO_RESOURCE;
+		}
 		pr_fail("%s: timer_create failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
 		return EXIT_FAILURE;
 	}
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
+	stress_set_proc_state(args->name, STRESS_STATE_RUN);
+
 	stress_timer_set(&timer);
 	if (timer_settime(timerid, 0, &timer, NULL) < 0) {
 		pr_fail("%s: timer_settime failed, errno=%d (%s)\n",
@@ -216,20 +215,18 @@ static int stress_timer(const stress_args_t *args)
 		return EXIT_FAILURE;
 	}
 
-	stress_set_proc_state(args->name, STRESS_STATE_RUN);
-
 	do {
 		struct timespec req;
 
-		if (n++ >= 1024) {
+		if (UNLIKELY(n++ >= 1024)) {
 			n = 0;
 
 			/* Exercise nanosleep on non-permitted timespec object values */
-			(void)memset(&req, 0, sizeof(req));
+			(void)shim_memset(&req, 0, sizeof(req));
 			req.tv_sec = -1;
 			VOID_RET(int, nanosleep(&req, NULL));
 
-			(void)memset(&req, 0, sizeof(req));
+			(void)shim_memset(&req, 0, sizeof(req));
 			req.tv_nsec = STRESS_NANOSECOND;
 			VOID_RET(int, nanosleep(&req, NULL));
 		}
@@ -237,16 +234,26 @@ static int stress_timer(const stress_args_t *args)
 		req.tv_sec = 0;
 		req.tv_nsec = 10000000;
 		(void)nanosleep(&req, NULL);
-		set_counter(args, timer_counter);
-	} while (keep_stressing(args));
+		stress_bogo_set(args, timer_counter);
+	} while (stress_continue(args));
+
+	/* stop timer */
+	(void)shim_memset(&timer, 0, sizeof(timer));
+	VOID_RET(int, timer_settime(timerid, 0, &timer, NULL));
 
 	if (timer_delete(timerid) < 0) {
 		pr_fail("%s: timer_delete failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
-		return EXIT_FAILURE;
+		rc = EXIT_FAILURE;
 	}
 	pr_dbg("%s: %" PRIu64 " timer overruns (instance %" PRIu32 ")\n",
-		args->name, overruns, args->instance);
+		args->name, timer_overruns, args->instance);
+
+	if (timer_settime_failure) {
+		pr_fail("%s: %" PRIu64 " timer settime calls failed\n",
+			args->name, timer_settime_failure);
+		rc = EXIT_FAILURE;
+	}
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
 #if defined(__linux__)
@@ -254,7 +261,7 @@ static int stress_timer(const stress_args_t *args)
 	{
 		/* Re-delete already deleted timer */
 		VOID_RET(int, timer_delete(timerid));
-	
+
 		/*
 		 * The manual states that EINVAL is returned when
 		 * an invalid timerid is used, in practice this
@@ -266,20 +273,23 @@ static int stress_timer(const stress_args_t *args)
 	}
 #endif
 
-	return EXIT_SUCCESS;
+	return rc;
 }
 
-stressor_info_t stress_timer_info = {
+const stressor_info_t stress_timer_info = {
 	.stressor = stress_timer,
-	.class = CLASS_INTERRUPT | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
+	.class = CLASS_SIGNAL | CLASS_INTERRUPT | CLASS_OS,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_timer_info = {
-	.stressor = stress_not_implemented,
-	.class = CLASS_INTERRUPT | CLASS_OS,
-	.opt_set_funcs = opt_set_funcs,
-	.help = help
+const stressor_info_t stress_timer_info = {
+	.stressor = stress_unimplemented,
+	.class = CLASS_SIGNAL | CLASS_INTERRUPT | CLASS_OS,
+	.opts = opts,
+	.verify = VERIFY_ALWAYS,
+	.help = help,
+	.unimplemented_reason = "built without librt, timer_create(), timer_delete(), timer_getoverrun() or timer_settime()"
 };
 #endif
