@@ -112,7 +112,8 @@ static const touch_method_t touch_methods[] = {
  */
 static void stress_touch_opts(const char *opt_name, const char *opt_arg, stress_type_id_t *type_id, void *value)
 {
-	char *str, *ptr, *token;
+	char *str, *ptr;
+	const char *token;
 	int open_flags = 0;
 
 	str = stress_const_optdup(opt_arg);
@@ -161,7 +162,7 @@ static void stress_touch_dir_clean(stress_args_t *args)
 {
 	char tmp[PATH_MAX];
 	DIR *dir;
-	struct dirent *d;
+	const struct dirent *d;
 
 	sync();
 	stress_temp_dir(tmp, sizeof(tmp), args->name,
@@ -298,7 +299,7 @@ static int stress_touch(stress_args_t *args)
 	stress_pid_t *s_pids, *s_pids_head = NULL;
 	size_t i;
 
-	s_pids = stress_s_pids_mmap(TOUCH_PROCS);
+	s_pids = stress_sync_s_pids_mmap(TOUCH_PROCS);
 	if (s_pids == MAP_FAILED) {
 		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, TOUCH_PROCS);
 		return EXIT_NO_RESOURCE;
@@ -307,7 +308,7 @@ static int stress_touch(stress_args_t *args)
 	touch_lock = stress_lock_create("counter");
 	if (!touch_lock) {
 		pr_inf_skip("%s: cannot create lock, skipping stressor\n", args->name);
-		(void)stress_s_pids_munmap(s_pids, TOUCH_PROCS);
+		(void)stress_sync_s_pids_munmap(s_pids, TOUCH_PROCS);
 		return EXIT_NO_RESOURCE;
 	}
 
@@ -324,7 +325,7 @@ static int stress_touch(stress_args_t *args)
 	ret = stress_temp_dir_mk_args(args);
 	if (ret < 0) {
 		(void)stress_lock_destroy(touch_lock);
-		(void)stress_s_pids_munmap(s_pids, TOUCH_PROCS);
+		(void)stress_sync_s_pids_munmap(s_pids, TOUCH_PROCS);
 		return stress_exit_status(-ret);
 	}
 
@@ -358,7 +359,7 @@ static int stress_touch(stress_args_t *args)
 	stress_touch_dir_clean(args);
 	(void)stress_temp_dir_rm_args(args);
 	(void)stress_lock_destroy(touch_lock);
-	(void)stress_s_pids_munmap(s_pids, TOUCH_PROCS);
+	(void)stress_sync_s_pids_munmap(s_pids, TOUCH_PROCS);
 
 	return EXIT_SUCCESS;
 }
