@@ -96,11 +96,11 @@ static bool stress_vma_continue_flag;
 
 static bool stress_vma_continue(stress_args_t *args)
 {
-	if (UNLIKELY(!g_stress_continue_flag))
+	if (UNLIKELY(!stress_continue_flag()))
 		return false;
-	if (LIKELY(args->max_ops == 0))
+	if (LIKELY(args->bogo.max_ops == 0))
 		return true;
-        return stress_vma_metrics->s.metrics[STRESS_VMA_MMAP] < args->max_ops;
+        return stress_vma_metrics->s.metrics[STRESS_VMA_MMAP] < args->bogo.max_ops;
 }
 
 #if defined(__linux__) &&		\
@@ -660,7 +660,7 @@ static void stress_vma_loop(
 		(void)sleep(10);
 		stress_vma_continue_flag = false;
 		VOID_RET(int, kill(pid, SIGKILL));
-		VOID_RET(int, shim_waitpid(pid, &status, 0));
+		VOID_RET(pid_t, shim_waitpid(pid, &status, 0));
 	} while (stress_vma_continue(args));
 }
 
@@ -671,7 +671,7 @@ static int stress_vma_child(stress_args_t *args, void *void_ctxt)
 	stress_vma_context_t *ctxt = (stress_vma_context_t *)void_ctxt;
 	int ret;
 
-	s_pids = stress_s_pids_mmap(STRESS_VMA_PROCS);
+	s_pids = stress_sync_s_pids_mmap(STRESS_VMA_PROCS);
 	if (s_pids == MAP_FAILED) {
 		pr_inf_skip("%s: failed to mmap %d PIDs, skipping stressor\n", args->name, STRESS_VMA_PROCS);
 		return EXIT_NO_RESOURCE;
@@ -707,7 +707,7 @@ static int stress_vma_child(stress_args_t *args, void *void_ctxt)
 	} while (stress_continue(args));
 
 	ret = stress_kill_and_wait_many(args, s_pids, i, SIGALRM, false);
-	(void)stress_s_pids_munmap(s_pids, STRESS_VMA_PROCS);
+	(void)stress_sync_s_pids_munmap(s_pids, STRESS_VMA_PROCS);
 
 	return ret;
 }
