@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2013-2021 Canonical, Ltd.
- * Copyright (C) 2022-2023 Colin Ian King.
+ * Copyright (C) 2022-2025 Colin Ian King.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -18,6 +18,8 @@
  *
  */
 #include "stress-ng.h"
+#include "core-builtin.h"
+#include "core-killpid.h"
 
 static const stress_help_t help[] = {
 	{ NULL,	"wait N",	"start N workers waiting on child being stop/resumed" },
@@ -40,8 +42,8 @@ static const stress_help_t help[] = {
  *	spawn a process
  */
 static pid_t spawn(
-	const stress_args_t *args,
-	void (*func)(const stress_args_t *args, const pid_t pid),
+	stress_args_t *args,
+	void (*func)(stress_args_t *args, const pid_t pid),
 	const pid_t pid_arg)
 {
 	pid_t pid;
@@ -49,7 +51,7 @@ static pid_t spawn(
 again:
 	pid = fork();
 	if (pid < 0) {
-		if (stress_redo_fork(errno))
+		if (stress_redo_fork(args, errno))
 			goto again;
 		return -1;
 	}
@@ -68,18 +70,18 @@ again:
  *	stopped and continued by the killer process
  */
 static void NORETURN runner(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const pid_t pid)
 {
 	(void)pid;
 
-	pr_dbg("%s: wait: runner started [%d]\n", args->name, (int)getpid());
+	pr_dbg("%s: runner started [%d]\n", args->name, (int)getpid());
 
 	do {
 		(void)pause();
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
-	(void)kill(getppid(), SIGALRM);
+	(void)shim_kill(getppid(), SIGALRM);
 	_exit(EXIT_SUCCESS);
 }
 
@@ -88,19 +90,19 @@ static void NORETURN runner(
  *	this continually stops and continues the runner process
  */
 static void NORETURN killer(
-	const stress_args_t *args,
+	stress_args_t *args,
 	const pid_t pid)
 {
 	double start = stress_time_now();
-	uint64_t last_counter = get_counter(args);
+	uint64_t last_counter = stress_bogo_get(args);
 	pid_t ppid = getppid();
 
-	pr_dbg("%s: wait: killer started [%d]\n", args->name, (int)getpid());
+	pr_dbg("%s: killer started [%d]\n", args->name, (int)getpid());
 
 	do {
-		(void)kill(pid, SIGSTOP);
+		(void)shim_kill(pid, SIGSTOP);
 		(void)shim_sched_yield();
-		(void)kill(pid, SIGCONT);
+		(void)shim_kill(pid, SIGCONT);
 
 		/*
 		 *  The waits may be blocked and
@@ -109,25 +111,25 @@ static void NORETURN killer(
 		 *  so we don't get stuck in the parent
 		 *  waiter indefinitely.
 		 */
-		if (last_counter == get_counter(args)) {
+		if (last_counter == stress_bogo_get(args)) {
 			const double now = stress_time_now();
 
 			if (now - start > ABORT_TIMEOUT) {
 				/* unblock waiting parent */
-				(void)kill(ppid, SIGUSR1);
+				(void)shim_kill(ppid, SIGUSR1);
 				start = now;
 			}
 		} else {
 			start = stress_time_now();
-			last_counter = get_counter(args);
+			last_counter = stress_bogo_get(args);
 		}
-	} while (keep_stressing(args));
+	} while (stress_continue(args));
 
 	/* forcefully kill runner, wait is in parent */
-	(void)kill(pid, SIGKILL);
+	(void)stress_kill_pid(pid);
 
 	/* tell parent to wake up! */
-	(void)kill(getppid(), SIGALRM);
+	(void)shim_kill(getppid(), SIGALRM);
 	_exit(EXIT_SUCCESS);
 }
 
@@ -135,15 +137,15 @@ static void NORETURN killer(
  *  stress_wait_continued()
  *	check WIFCONTINUED
  */
-static void stress_wait_continued(const stress_args_t *args, const int status)
+static void stress_wait_continued(stress_args_t *args, int status)
 {
 #if defined(WIFCONTINUED)
 	if (WIFCONTINUED(status))
-		inc_counter(args);
+		stress_bogo_inc(args);
 #else
 	(void)status;
 
-	inc_counter(args);
+	stress_bogo_inc(args);
 #endif
 }
 
@@ -166,9 +168,9 @@ static pid_t syscall_shim_waitpid(pid_t pid, int *wstatus, int options)
  *  stress_wait
  *	stress wait*() family of calls
  */
-static int stress_wait(const stress_args_t *args)
+static int stress_wait(stress_args_t *args)
 {
-	int status, ret = EXIT_SUCCESS;
+	int ret = EXIT_SUCCESS;
 	pid_t pid_r, pid_k, wret;
 	int options = 0;
 #if defined(HAVE_WAIT4)
@@ -182,12 +184,14 @@ static int stress_wait(const stress_args_t *args)
 	options |= WCONTINUED;
 #endif
 
-	pr_dbg("%s: waiter started [%d]\n",
-		args->name, (int)args->pid);
+	pr_dbg("%s: waiter started [%" PRIdMAX "]\n",
+		args->name, (intmax_t)args->pid);
 
 	if (stress_sighandler(args->name, SIGUSR1, stress_sighandler_nop, NULL) < 0)
 		return EXIT_FAILURE;
 
+	stress_set_proc_state(args->name, STRESS_STATE_SYNC_WAIT);
+	stress_sync_start_wait(args);
 	stress_set_proc_state(args->name, STRESS_STATE_RUN);
 
 	pid_r = spawn(args, runner, 0);
@@ -206,6 +210,7 @@ static int stress_wait(const stress_args_t *args)
 	}
 
 	do {
+		int status;
 #if defined(HAVE_WAIT4) || defined(HAVE_WAIT3)
 		struct rusage usage;
 #endif
@@ -213,39 +218,41 @@ static int stress_wait(const stress_args_t *args)
 		 *  Exercise waitpid
 		 */
 		wret = syscall_shim_waitpid(pid_r, &status, options);
-		if ((wret < 0) && (errno != EINTR) && (errno != ECHILD)) {
-			pr_fail("%s: waitpid failed, errno=%d (%s)\n",
-				args->name, errno, strerror(errno));
+		if (UNLIKELY((wret < 0) && (errno != EINTR) && (errno != ECHILD))) {
+			pr_fail("%s: waitpid on PID %" PRIdMAX " failed, errno=%d (%s)\n",
+				args->name, (intmax_t)pid_r, errno, strerror(errno));
 			break;
 		}
 		stress_wait_continued(args, status);
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 		/*
 		 *  Exercise wait
 		 */
 		wret = shim_wait(&status);
-		if ((wret < 0) && (errno != EINTR) && (errno != ECHILD)) {
+		if (UNLIKELY((wret < 0) && (errno != EINTR) && (errno != ECHILD))) {
 			pr_fail("%s: wait failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			ret = EXIT_FAILURE;
 			break;
 		}
 		stress_wait_continued(args, status);
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #if defined(HAVE_WAIT3)
 		/*
 		 *  Exercise wait3 if available
 		 */
 		wret = shim_wait3(&status, options, &usage);
-		if ((wret < 0) && (errno != EINTR) && (errno != ECHILD)) {
+		if (UNLIKELY((wret < 0) && (errno != EINTR) && (errno != ECHILD))) {
 			pr_fail("%s: wait3 failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			ret = EXIT_FAILURE;
 			break;
 		}
 		stress_wait_continued(args, status);
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 #endif
 
@@ -254,52 +261,56 @@ static int stress_wait(const stress_args_t *args)
 		 *  Exercise wait4 if available
 		 */
 		wret = shim_wait4(pid_r, &status, options, &usage);
-		if ((wret < 0) && (errno != EINTR) && (errno != ECHILD)) {
+		if (UNLIKELY((wret < 0) && (errno != EINTR) && (errno != ECHILD))) {
 			pr_fail("%s: wait4 failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			ret = EXIT_FAILURE;
 			break;
 		}
 		stress_wait_continued(args, status);
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 		/*
 		 *  Exercise PID -1 -> any child process
 		 */
 		wret = shim_wait4(-1, &status, options, &usage);
-		if ((wret < 0) && (errno != EINTR) && (errno != ECHILD)) {
-			pr_fail("%s: wait4 on pid -1 failed, errno=%d (%s)\n",
+		if (UNLIKELY((wret < 0) && (errno != EINTR) && (errno != ECHILD))) {
+			pr_fail("%s: wait4 on PID -1 failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			ret = EXIT_FAILURE;
 			break;
 		}
 		stress_wait_continued(args, status);
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 		/*
 		 *  Exercise PID 0 -> process group of caller
 		 */
 		wret = shim_wait4(0, &status, options, &usage);
-		if ((wret < 0) && (errno != EINTR) && (errno != ECHILD)) {
-			pr_fail("%s: wait4 on pid 0 failed, errno=%d (%s)\n",
+		if (UNLIKELY((wret < 0) && (errno != EINTR) && (errno != ECHILD))) {
+			pr_fail("%s: wait4 on PID 0 failed, errno=%d (%s)\n",
 				args->name, errno, strerror(errno));
+			ret = EXIT_FAILURE;
 			break;
 		}
 		stress_wait_continued(args, status);
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 		/*
 		 *  Exercise -ve PID -> PGID number
 		 */
 		wret = shim_wait4(-pgrp, &status, options, &usage);
-		if ((wret < 0) && (errno != EINTR) && (errno != ECHILD)) {
+		if (UNLIKELY((wret < 0) && (errno != EINTR) && (errno != ECHILD))) {
 			pr_fail("%s: wait4 on pgrp %" PRIdMAX " failed, errno=%d (%s)\n",
 				args->name, (intmax_t)pgrp, errno, strerror(errno));
+			ret = EXIT_FAILURE;
 			break;
 		}
 		stress_wait_continued(args, status);
-		if (!keep_stressing_flag())
+		if (UNLIKELY(!stress_continue_flag()))
 			break;
 
 		/*
@@ -320,73 +331,75 @@ static int stress_wait(const stress_args_t *args)
 		if (options) {
 			siginfo_t info;
 
-			(void)memset(&info, 0, sizeof(info));
+			(void)shim_memset(&info, 0, sizeof(info));
 			wret = waitid(P_PID, (id_t)pid_r, &info, options);
-			if ((wret < 0) && (errno != EINTR) && (errno != ECHILD)) {
+			if (UNLIKELY((wret < 0) && (errno != EINTR) && (errno != ECHILD))) {
 				pr_fail("%s: waitid failed, errno=%d (%s)\n",
 					args->name, errno, strerror(errno));
+				ret = EXIT_FAILURE;
 				break;
 			}
 			/*
 			 *  Need to look into this, but on a heavily loaded
 			 *  system we can get info.si_pid set to zero(!)
 			 */
-			if ((info.si_pid != pid_r) && (info.si_pid != 0)) {
+			if (UNLIKELY((info.si_pid != pid_r) && (info.si_pid != 0))) {
 				pr_fail("%s: waitid returned PID %ld but expected PID %ld\n",
 					args->name, (long int)info.si_pid, (long int)pid_r);
+				ret = EXIT_FAILURE;
 			}
-			if ((info.si_signo != SIGCHLD) && (info.si_signo != 0)) {
+			if (UNLIKELY((info.si_signo != SIGCHLD) && (info.si_signo != 0))) {
 				pr_fail("%s: waitid returned si_signo %d (%s) but expected SIGCHLD\n",
 					args->name, info.si_signo, stress_strsignal(info.si_signo));
+				ret = EXIT_FAILURE;
 			}
-			if ((info.si_status != EXIT_SUCCESS) &&
-			    (info.si_status != SIGSTOP) &&
-			    (info.si_status != SIGCONT) &&
-			    (info.si_status != SIGKILL)) {
+			if (UNLIKELY((info.si_status != EXIT_SUCCESS) &&
+				     (info.si_status != SIGSTOP) &&
+				     (info.si_status != SIGCONT) &&
+				     (info.si_status != SIGKILL))) {
 				pr_fail("%s: waitid returned unexpected si_status %d\n",
 					args->name, info.si_status);
+				ret = EXIT_FAILURE;
 			}
 #if defined(CLD_EXITED) &&	\
     defined(CLD_KILLED) &&	\
     defined(CLD_STOPPED) &&	\
     defined(CLD_CONTINUED)
-			if ((info.si_code != CLD_EXITED) &&
-			    (info.si_code != CLD_KILLED) &&
-			    (info.si_code != CLD_STOPPED) &&
-			    (info.si_code != CLD_CONTINUED) &&
-			    (info.si_code != 0)) {
+			if (UNLIKELY((info.si_code != CLD_EXITED) &&
+				     (info.si_code != CLD_KILLED) &&
+				     (info.si_code != CLD_STOPPED) &&
+				     (info.si_code != CLD_CONTINUED) &&
+				     (info.si_code != 0))) {
 				pr_fail("%s: waitid returned unexpected si_code %d\n",
 					args->name, info.si_code);
+				ret = EXIT_FAILURE;
 			}
 #endif
 			stress_wait_continued(args, status);
-			if (!keep_stressing_flag())
-				break;
 		}
 #endif
-	} while (keep_stressing_flag() && (!args->max_ops || (get_counter(args) < args->max_ops)));
+	} while (stress_continue_flag() && (!args->bogo.max_ops || (stress_bogo_get(args) < args->bogo.max_ops)));
 
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)kill(pid_k, SIGKILL);
-	(void)shim_waitpid(pid_k, &status, 0);
+	(void)stress_kill_pid_wait(pid_k, NULL);
 tidy:
 	stress_set_proc_state(args->name, STRESS_STATE_DEINIT);
-	(void)kill(pid_r, SIGKILL);
-	(void)shim_waitpid(pid_r, &status, 0);
+	(void)stress_kill_pid_wait(pid_r, NULL);
 
 	return ret;
 }
 
-stressor_info_t stress_wait_info = {
+const stressor_info_t stress_wait_info = {
 	.stressor = stress_wait,
 	.class = CLASS_SCHEDULER | CLASS_OS,
 	.verify = VERIFY_ALWAYS,
 	.help = help
 };
 #else
-stressor_info_t stress_wait_info = {
+const stressor_info_t stress_wait_info = {
 	.stressor = stress_unimplemented,
 	.class = CLASS_SCHEDULER | CLASS_OS,
+	.verify = VERIFY_ALWAYS,
 	.help = help,
 	.unimplemented_reason = "disabled for GNU/HURD because it causes the kernel to assert"
 };
