@@ -48,6 +48,12 @@ static const stress_help_t help[] = {
 #define DATA_SIZE_64K 	(KB * 64)	/* Must be a multiple of 64 bytes */
 #define DATA_SIZE DATA_SIZE_64K
 
+#define ZLIB_MIN_COMPRESSION	(0)
+#define ZLIB_MAX_COMPRESSION	(Z_BEST_COMPRESSION)
+
+#define ZLIB_MIN_MEM_LEVEL	(1)
+#define ZLIB_MAX_MEM_LEVEL	(9)
+
 typedef void (*stress_zlib_rand_data_func)(stress_args_t *args,
 	uint64_t *RESTRICT data, uint64_t *RESTRICT data_end);
 
@@ -1349,9 +1355,10 @@ static const char *stress_zlib_method(const size_t i)
 	return (i < SIZEOF_ARRAY(zlib_rand_data_methods)) ? zlib_rand_data_methods[i].name : NULL;
 }
 
+
 static const stress_opt_t opts[] = {
-	{ OPT_zlib_level,        "zlib-level",        TYPE_ID_UINT32, 0, Z_BEST_COMPRESSION, NULL },
-	{ OPT_zlib_mem_level,    "zlib-mem-level",    TYPE_ID_UINT32, 1, 9, NULL },
+	{ OPT_zlib_level,        "zlib-level",        TYPE_ID_UINT32, ZLIB_MIN_COMPRESSION, ZLIB_MAX_COMPRESSION, NULL },
+	{ OPT_zlib_mem_level,    "zlib-mem-level",    TYPE_ID_UINT32, ZLIB_MIN_MEM_LEVEL, ZLIB_MAX_MEM_LEVEL, NULL },
 	{ OPT_zlib_method,       "zlib-method",       TYPE_ID_SIZE_T_METHOD, 0, 0, stress_zlib_method },
 	{ OPT_zlib_window_bits,  "zlib-window-bits",  TYPE_ID_CALLBACK, 0, 0, stress_zlib_window_bits },
 	{ OPT_zlib_stream_bytes, "zlib-stream-bytes", TYPE_ID_UINT64_BYTES_VM, 0, MAX_MEM_LIMIT, NULL },
@@ -1393,15 +1400,25 @@ static const char *stress_zlib_err(const int zlib_err)
  *	get all zlib arguments at once
  */
 static void stress_zlib_get_args(stress_zlib_args_t *params) {
-	params->level = Z_BEST_COMPRESSION;
+	params->level = ZLIB_MAX_COMPRESSION;
 	params->mem_level = 8;
 	params->method = 0;
 	params->window_bits = 15;
 	params->stream_bytes = 0;
 	params->strategy = Z_DEFAULT_STRATEGY;
 
-	(void)stress_get_setting("zlib-level", &params->level);
-	(void)stress_get_setting("zlib-mem-level", &params->mem_level);
+	if (!stress_get_setting("zlib-level", &params->level)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			params->level = Z_BEST_COMPRESSION;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			params->level = 0;
+	}
+	if (!stress_get_setting("zlib-mem-level", &params->mem_level)) {
+		if (g_opt_flags & OPT_FLAGS_MAXIMIZE)
+			params->level = ZLIB_MAX_MEM_LEVEL;
+		if (g_opt_flags & OPT_FLAGS_MINIMIZE)
+			params->level = ZLIB_MIN_MEM_LEVEL;
+	}
 	(void)stress_get_setting("zlib-method", &params->method);
 	(void)stress_get_setting("zlib-window-bits", &params->window_bits);
 	(void)stress_get_setting("zlib-stream-bytes", &params->stream_bytes);
@@ -1448,7 +1465,7 @@ static int stress_zlib_inflate(
 	do {
 		ret = inflateInit2(&stream_inf, zlib_args.window_bits);
 		if (UNLIKELY(ret != Z_OK)) {
-			pr_fail("%s: zlib inflateInit error: %s\n",
+			pr_fail("%s: zlib inflateInit error, %s\n",
 				args->name, stress_zlib_err(ret));
 			zlib_checksum->error = true;
 			goto zlib_checksum_error;
@@ -1464,7 +1481,7 @@ static int stress_zlib_inflate(
 			} else if ((sz != sizeof(def_size)) || (sz < 0)) {
 				(void)inflateEnd(&stream_inf);
 				if (UNLIKELY((errno != EINTR) && (errno != EPIPE))) {
-					pr_fail("%s: zlib pipe read size error: %s (ret=%zd errno=%d)\n",
+					pr_fail("%s: zlib pipe read size error, %s (ret=%zd errno=%d)\n",
 						args->name, strerror(errno), sz, errno);
 					zlib_checksum->error = true;
 					goto zlib_checksum_error;
@@ -1483,7 +1500,7 @@ static int stress_zlib_inflate(
 			} else if ((sz != def_size) || (sz < 0)) {
 				(void)inflateEnd(&stream_inf);
 				if (UNLIKELY((errno != EINTR) && (errno != EPIPE))) {
-					pr_fail("%s: zlib pipe read buffer error: %s (ret=%zd errno=%d)\n",
+					pr_fail("%s: zlib pipe read buffer error, %s (ret=%zd errno=%d)\n",
 						args->name, strerror(errno), sz, errno);
 					zlib_checksum->error = true;
 					goto zlib_checksum_error;
@@ -1509,7 +1526,7 @@ static int stress_zlib_inflate(
 				case Z_NEED_DICT:
 				case Z_DATA_ERROR:
 				case Z_MEM_ERROR:
-					pr_fail("%s: zlib inflate error: %s\n",
+					pr_fail("%s: zlib inflate error, %s\n",
 						args->name, stress_zlib_err(ret));
 					(void)inflateEnd(&stream_inf);
 					goto zlib_checksum_error;
@@ -1590,9 +1607,13 @@ static int stress_zlib_deflate(
 				Z_DEFLATED, zlib_args.window_bits,
 				zlib_args.mem_level, zlib_args.strategy);
 		if (UNLIKELY(ret != Z_OK)) {
-			pr_fail("%s: zlib deflateInit error: %s\n",
+			pr_fail("%s: zlib deflateInit error, %s\n",
 				args->name, stress_zlib_err(ret));
 			zlib_checksum->error = true;
+			(void)deflateEnd(&stream_def);
+			stream_def.zalloc = Z_NULL;
+			stream_def.zfree = Z_NULL;
+			stream_def.opaque = Z_NULL;
 			ret = EXIT_FAILURE;
 			goto zlib_checksum_error;
 		}
@@ -1641,7 +1662,7 @@ static int stress_zlib_deflate(
 
 				ret = deflate(&stream_def, flush);
 				if (UNLIKELY(ret == Z_STREAM_ERROR)) {
-					pr_fail("%s: zlib deflate error: %s\n",
+					pr_fail("%s: zlib deflate error, %s\n",
 						args->name, stress_zlib_err(ret));
 					(void)deflateEnd(&stream_def);
 					ret = EXIT_FAILURE;
@@ -1662,7 +1683,7 @@ static int stress_zlib_deflate(
 				} else if (sz != sizeof(def_size)) {
 					(void)deflateEnd(&stream_def);
 					if (UNLIKELY((errno != EINTR) && (errno != EPIPE) && (errno != 0))) {
-						pr_fail("%s: zlib pipe write size error: %s (ret=%zd errno=%d)\n",
+						pr_fail("%s: zlib pipe write size error, %s (ret=%zd errno=%d)\n",
 							args->name, strerror(errno), sz, errno);
 						ret = EXIT_FAILURE;
 						goto zlib_checksum_error;
@@ -1681,7 +1702,7 @@ static int stress_zlib_deflate(
 				} else if (sz != def_size) {
 					(void)deflateEnd(&stream_def);
 					if (UNLIKELY((errno != EINTR) && (errno != EPIPE) && (errno != 0))) {
-						pr_fail("%s: zlib pipe write buffer error: %s (ret=%zd errno=%d)\n",
+						pr_fail("%s: zlib pipe write buffer error, %s (ret=%zd errno=%d)\n",
 							args->name, strerror(errno), sz, errno);
 						ret = EXIT_FAILURE;
 						goto zlib_checksum_error;
@@ -1772,7 +1793,7 @@ again:
 		(void)close(fds[0]);
 		(void)close(fds[1]);
 
-		if (!stress_continue(args))
+		if (UNLIKELY(!stress_continue(args)))
 			return EXIT_SUCCESS;
 		pr_err("%s: fork failed, errno=%d (%s)\n",
 			args->name, errno, strerror(errno));
